package com.perdemia.permissionanalyzer.application;

import java.io.IOException;
import java.net.URL;
import java.util.logging.Handler;
import java.util.logging.LogManager;

import com.perdemia.permissionanalyzer.application.logging.EclipseLogListener;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.logging.LogFormatter;
import com.perdemia.permissionanalyzer.application.logging.PluginFileLogHandler;
import com.perdemia.permissionanalyzer.data.util.DBUtil;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Activator implements BundleActivator {
	private EclipseLogListener logListener = new EclipseLogListener();
	private static Logger logger = LoggerFactory.getLogger(Activator.class.getName());
	private static BundleContext context;

	public static BundleContext getContext() {
		return context;
	}

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bundleContext) throws Exception {
		Activator.context = bundleContext;
		
		System.setProperty("java.net.useSystemProxies", "true");
		
		try {
			// System.setProperty("java.util.logging.config.file", Platform.getLocation().toFile().getAbsolutePath() + File.separator + "logging.properties");
			URL confURL = bundleContext.getBundle().getEntry("logging.properties");
			System.setProperty("java.util.logging.config.file", FileLocator.toFileURL(confURL).getFile());
			LogManager logManager = LogManager.getLogManager();
			logManager.readConfiguration();
			
			java.util.logging.Logger rootJdkLogger = java.util.logging.Logger.getLogger("");
			
			// add the custom PluginFileLogHandler which will log to a file in the Application folder
			rootJdkLogger.addHandler(new PluginFileLogHandler());
			
			// add the GUI log handler which will notify listens using StatusMessage objects
			rootJdkLogger.addHandler(new GUILogHandler());
			
			// set the custom formatters for all the handlers
		    for (Handler handler : rootJdkLogger.getHandlers()) {
		    	handler.setFormatter(new LogFormatter());
		    }
		    
		} catch (IOException e) {
			logger.error("Failed to load file logging.properties", e);
		}

	    Platform.addLogListener(logListener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bundleContext) throws Exception {
		Platform.removeLogListener(logListener);
		DBUtil.closeDBConnection();
		Activator.context = null;
	}

}
