package com.perdemia.permissionanalyzer.application.handlers.menu;

import gnu.trove.list.array.TIntArrayList;

import java.lang.reflect.InvocationTargetException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.attribute.UserPrincipal;
import java.nio.file.attribute.UserPrincipalLookupService;
import java.nio.file.attribute.UserPrincipalNotFoundException;
import java.sql.PreparedStatement;

import javax.inject.Named;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ChangeOwnerDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.DomainCache;
import com.perdemia.permissionanalyzer.data.util.MemberCache;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

public class ChangeOwnerHandler {
	private static final Logger logger = LoggerFactory.getLogger(ChangeOwnerHandler.class.getName());
	private Member newOwner;
	
	@Execute
	public void execute(final Shell shell, EPartService partService, ESelectionService selectionService,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.changeowner.param.allfiles") final String allFiles){
		StructuredSelection selection = (StructuredSelection) selectionService.getSelection(FilePart.ID);
		FileInfo file = (FileInfo) selection.getFirstElement();
		MPart filePart = partService.findPart(FilePart.ID);
		final FilePart filePartObject = (FilePart) filePart.getObject();
		
		if (allFiles.equalsIgnoreCase("true")) {
			// Change the owner of all the files
			// determine the number of files
			// start with a progress monitor
			ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
			try {
				progressDialog.run(true, true, new IRunnableWithProgress() {

					@Override
					public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						try {
							monitor.setTaskName("Getting all files from the database...");
													
							// execute the query and extract all the applicable file ids
							PreparedStatement ps = FilterHelper.getInstance().buildFilterQuery(ObjectType.FILE, false, false, 0, "FILEINFO.PARENT_FILE_IDS ASC, FILEINFO.NAME_LOWERCASE ASC", null);
							final TIntArrayList uniqueFileIds = FilterHelper.getInstance().getApplicableFileIds(ps, false, false);
							final int fileCount = uniqueFileIds.size();
							
							Display.getDefault().syncExec(new Runnable(){
								public void run() {
									try {
										newOwner = askForNewOwner(shell, fileCount);
										if (newOwner != null) {
											UserPrincipalLookupService lookupService = FileSystems.getDefault().getUserPrincipalLookupService();
											final UserPrincipal userPrincipal = lookupService.lookupPrincipalByName(newOwner.getDisplayName());
											ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
											try {
												progressDialog.run(true, true, new IRunnableWithProgress() {
													@Override
													public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
														monitor.setTaskName("Applying owner");
														monitor.beginTask("Setting owner on " + fileCount + " directories/files", fileCount);
														try {
															for (int counter = 0; counter < uniqueFileIds.size(); counter++) {
																int fileId = uniqueFileIds.getQuick(counter);
																final FileInfo file = FileInfo.getById(fileId);
																monitor.subTask(file.determineFullPath());
																Files.setOwner(Paths.get(file.determineFullPath()), userPrincipal);
																file.setOwner(newOwner);
																file.save();
																
																Display.getDefault().syncExec(new Runnable(){
																	public void run() {
																		// refresh the UI
																		if (filePartObject.getTreeViewer().getTree().isVisible()) {
																			filePartObject.getTreeViewer().refresh(file);
																		} else {
																			filePartObject.getTableViewer().refresh(file);
																		}
																	}
																});																
																
																monitor.worked(1);
															}
														} catch (Exception e) {
															logger.error("Error while applying the new owner: " + e.getMessage(), e);
															SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying the new owner, make sure you run Permission Analyzer as Administrator: \r\n\r\n" + GUILogHandler.getNestedMessage(e));
														}
													}
												});
											} catch (InvocationTargetException | InterruptedException e) {
												SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying owner: " + e.getMessage());
												logger.error("Error while applying owner", e);
											}
										}
									} catch (Exception ex) {
										if (ex instanceof UserPrincipalNotFoundException) {
											logger.error("Could not find the account name " + (newOwner != null ? newOwner.getDisplayName() : ""), ex);
											SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Could not find the account name " + (newOwner != null ? newOwner.getDisplayName() : ""));
										} else {
											logger.error("Error while changing the owner to " + (newOwner != null ? newOwner.getDisplayName() : ""), ex);
											SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while changing the owner to " + (newOwner != null ? newOwner.getDisplayName() : "") + ", make sure you run Permission Analyzer as Administrator: \r\n\r\n" + ex.getMessage());
										}
									}
								}
							});
							
						} catch (Exception e) {
							logger.error("Error while applying the new owner: " + e.getMessage(), e);
							SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying the new owner: " + GUILogHandler.getNestedMessage(e));
						}
					}
				});
			} catch (InvocationTargetException | InterruptedException e) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying owner: " + e.getMessage());
				logger.error("Error while applying owner", e);
			}
			
		} else {
			// Change the owner of one particular file
			if (file != null) {
				try {
					newOwner = askForNewOwner(shell, 1);
					if (newOwner != null) {
						UserPrincipalLookupService lookupService = FileSystems.getDefault().getUserPrincipalLookupService();
						UserPrincipal userPrincipal = lookupService.lookupPrincipalByName(newOwner.getDisplayName());
						Files.setOwner(Paths.get(file.determineFullPath()), userPrincipal);
						
						file.setOwner(newOwner);
						file.save();
						
						// refresh the UI
						if (filePartObject.getTreeViewer().getTree().isVisible()) {
							filePartObject.getTreeViewer().refresh(file);
						} else {
							filePartObject.getTableViewer().refresh(file);
						}
					}
				} catch (Exception ex) {
					if (ex instanceof UserPrincipalNotFoundException) {
						logger.error("Could not find the account name " + (newOwner != null ? newOwner.getDisplayName() : ""), ex);
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Could not find the account name " + (newOwner != null ? newOwner.getDisplayName() : ""));
					} else {
						logger.error("Error while changing the owner to " + (newOwner != null ? newOwner.getDisplayName() : ""), ex);
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while changing the owner to " + (newOwner != null ? newOwner.getDisplayName() : "") + ", make sure you run Permission Analyzer as Administrator: \r\n\r\n" + ex.getMessage());
					}					
				}		
			}
		}		
	}
	
	private Member askForNewOwner(Shell shell, int fileCount) throws DBException {
		ChangeOwnerDialog dialog = new ChangeOwnerDialog(shell, fileCount);
		if (dialog.open() == IDialogConstants.OK_ID) {
			// change the owner
			String accountName = dialog.getAccountName();
			String domainName;
			if (accountName.indexOf("\\") >= 0) {
				domainName = accountName.substring(0, accountName.indexOf("\\"));
				accountName = accountName.substring(accountName.indexOf("\\") + 1);
			} else {
				domainName = AclScanner.getLocalMachineName();
			}
			int domainId = DomainCache.getDomainId(domainName);
			Member owner = MemberCache.getMemberByName(accountName, domainId);
			
			if (owner == null) {
				owner = new Member(accountName);
				owner.setDomainId(domainId);
				owner.save();
				
			}
			// abuse the displayname to remember the account name that has been entered
			owner.setDisplayName(accountName);
			
			return owner;
		}
		
		return null;
	}
}
