/*******************************************************************************
u * Copyright (c) 2010 - 2013 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Lars Vogel <lars.Vogel@gmail.com> - Bug 419770
 *******************************************************************************/
package com.perdemia.permissionanalyzer.application.handlers.menu;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.inject.Named;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.equinox.p2.core.IProvisioningAgent;
import org.eclipse.equinox.p2.operations.ProvisioningJob;
import org.eclipse.equinox.p2.operations.ProvisioningSession;
import org.eclipse.equinox.p2.operations.UpdateOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.util.LicenseHelper;

// Require-Bundle: org.eclipse.equinox.p2.core|engine|operation|metadata.repository
// Feature: org.eclipse.equinox.p2.core.feature
//
// !!! Do not run from within IDE. Update only works in an exported product !!!
//
public class CheckForUpdatesHandler {
	private static final Logger logger = LoggerFactory.getLogger(CheckForUpdatesHandler.class.getName());
	// private static final String REPOSITORY_LOC = System.getProperty("UpdateHandler.Repo", "http://localhost/repository");
	//private static final String REPOSITORY_LOC = System.getProperty("UpdateHandler.Repo", "file://C:/Docs/Perdemia/build/old9/repository");
	private static final String REPOSITORY_LOC = System.getProperty("UpdateHandler.Repo", "http://www.permissionanalyzer.com/update");//
	private static boolean doUpdate;
	//private static ProxySelector originalProxySelector;
	
//	static {
//		originalProxySelector = ProxySelector.getDefault();
//	}
	
	@Execute
	public void execute(final IProvisioningAgent agent, final Shell shell, final UISynchronize sync, final IWorkbench workbench,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.checkforupdates.param.silentmode") final String silentmode) {
		
		if (silentmode == null) {
			try {
				LicenseHelper.validateLicenseOnline();
	
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(null);
				progressDialog.run(true, true, new IRunnableWithProgress() {
					@Override
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						checkForUpdates(agent, shell, sync, workbench, monitor, false);
					}
				});
			} catch (InvocationTargetException e) {
				MessageDialog.openError(shell, "Error", e.getMessage());
				e.printStackTrace();
			} catch (InterruptedException e) {
				MessageDialog.openError(shell, "Error", e.getMessage());
				e.printStackTrace();
			}
			
		} else {
			// don't use a progress monitor
			Display.getDefault().asyncExec(new Runnable(){
				public void run() {
					checkForUpdates(agent, shell, sync, workbench, null, true);
				}
			});
		}

		/*
		 * Job j = new Job("Update Job") {
		 * 
		 * @Override protected IStatus run(final IProgressMonitor monitor) { return checkForUpdates(agent, shell, sync, workbench, monitor); } }; j.schedule();
		 */
	}

	private IStatus checkForUpdates(final IProvisioningAgent agent, final Shell shell, final UISynchronize sync, final IWorkbench workbench, IProgressMonitor monitor, boolean silentMode) {
		ProgressMonitorDialog progressDialog = null;
		
		try {
			/* 1. configure update operation */
			final ProvisioningSession session = new ProvisioningSession(agent);
			final UpdateOperation operation = new UpdateOperation(session);
			configureUpdate(operation);
			
			IStatus status = null;
			SubMonitor sub = null;
			if (monitor != null) {
				monitor.setTaskName("Checking for application updates...");
				sub = SubMonitor.convert(monitor, "Checking for application updates...", 200);
				
				// run check if updates are available (causing I/O)
				status = operation.resolveModal(sub.newChild(100));
			} else {
				status = operation.resolveModal(null);
			}
			
			
			// Inform the user
			if (status.getCode() == UpdateOperation.STATUS_NOTHING_TO_UPDATE) {
				if (monitor != null) { 
					showMessage(shell, sync, "No updates", "No updates have been found.", false);
				}
				return Status.CANCEL_STATUS;
				
			} else if (monitor != null && monitor.isCanceled()) {
				return Status.CANCEL_STATUS;
				
			} 	else {
				showConfirmation(shell, sync, "New version", "A new application version has been found, do you want to install this update?");
			}

			if (!doUpdate) {
				return Status.CANCEL_STATUS;
			}

			/* 3. run installation */
			
			progressDialog = new ProgressMonitorDialog(null);
			if (monitor == null) {
				monitor = progressDialog.getProgressMonitor();
				monitor.setTaskName("Installing application updates...");
				progressDialog.open();
				sub = SubMonitor.convert(monitor, "Installing application updates...", 100);
			}
			
			final ProvisioningJob provisioningJob = operation.getProvisioningJob(monitor);

			// updates cannot run from within Eclipse IDE!
			if (provisioningJob == null) {
				showMessage(shell, sync, "Error", "Could not update the application: error while creating the update job", true);
				return Status.CANCEL_STATUS;
			}
			// configureProvisioningJob(provisioningJob, shell, sync, workbench);

			final IStatus statusUpdate = provisioningJob.runModal(sub.newChild(100));

			if (statusUpdate.isOK()) {
				// restart?
				sync.syncExec(new Runnable() {
					@Override
					public void run() {
						boolean restart = MessageDialog.openQuestion(shell, "Restart", "Updates have been installed, do you want to restart the application?");
						if (restart) {
							workbench.restart();
						}
					}
				});
			} else {
				showMessage(shell, sync, "Error", "Error while applying update: " + statusUpdate.getMessage(), true);
				logger.error("Error while applying update: " + statusUpdate.getMessage(), statusUpdate.getException());
				logger.error(statusUpdate.toString());
			}

		} catch (Exception ex) {
			logger.error("Error while checking for updates: " + ex.getMessage(), ex);
			if (!silentMode) {
				showMessage(shell, sync, "Error", "Error while checking for updates: " + ex.getMessage(), true);
			}
						
		} finally {
			if (progressDialog != null) {
				progressDialog.close();
			}
		}
		
		//ProxySelector.setDefault(originalProxySelector);
		//Authenticator.setDefault(null);
		return Status.OK_STATUS;
	}

	/*
	 * private void configureProvisioningJob(ProvisioningJob provisioningJob, final Shell shell, final UISynchronize sync, final IWorkbench workbench) {
	 * 
	 * // Register a job change listener to track // installation progress and notify user upon success provisioningJob.addJobChangeListener(new JobChangeAdapter() {
	 * 
	 * @Override public void done(IJobChangeEvent event) { if (event.getResult().isOK()) { sync.syncExec(new Runnable() {
	 * 
	 * @Override public void run() { boolean restart = MessageDialog.openQuestion(shell, "Updates installed, restart?", "Updates have been installed successfully, do you want to restart?"); if (restart) { workbench.restart(); } } });
	 * 
	 * } else { showMessage(shell, sync, "Error", "Error while applying update: " + event.getResult().getMessage(), true); } super.done(event); } });
	 * 
	 * }
	 */

	private void showMessage(final Shell parent, final UISynchronize sync, final String title, final String msg, final boolean error) {
		sync.syncExec(new Runnable() {
			@Override
			public void run() {
				if (error) {
					MessageDialog.openError(parent, title, msg);
				} else {
					MessageDialog.openInformation(parent, title, msg);
				}
			}
		});
	}

	private void showConfirmation(final Shell parent, final UISynchronize sync, final String title, final String msg) {
		sync.syncExec(new Runnable() {
			@Override
			public void run() {
				doUpdate = MessageDialog.openConfirm(parent, title, msg);
			}
		});
	}

	private UpdateOperation configureUpdate(final UpdateOperation operation) {

		// create uri and check for validity
		URI uri = null;
		try {
			uri = new URI(REPOSITORY_LOC);
		} catch (final URISyntaxException e) {
			e.printStackTrace();
			return null;
		}

		// set location of artifact and metadata repo
		operation.getProvisioningContext().setArtifactRepositories(new URI[] { uri });
		operation.getProvisioningContext().setMetadataRepositories(new URI[] { uri });
		return operation;

		/*
		 * final ProvisioningUI ui = ProvUIActivator.getDefault().getProvisioningUI(); IArtifactRepositoryManager artifactManager = ProvUI.getArtifactRepositoryManager(ui.getSession()); artifactManager.addRepository(new URI(REPOSITORY_LOC);
		 * 
		 * IMetadataRepositoryManager metadataManager = ProvUI.getMetadataRepositoryManager(ui.getSession()); metadataManager.addRepository(new URI(REPOSITORY_LOC);
		 */
	}

	/*
	 * public static void setRepositories() throws InvocationTargetException { try { final MetadataRepositoryElement element = new MetadataRepositoryElement(null, new URI(REPOSITORY_LOC), true); ElementUtils.updateRepositoryUsingElements(new
	 * MetadataRepositoryElement[] {element}, null); } catch (URISyntaxException e) { e.printStackTrace(); throw new InvocationTargetException(e); } }
	 */
	/*
	 * public static void addUpdateSite(IProvisioningAgent provisioningAgent, Logger logger) throws InvocationTargetException { // Load repository manager IMetadataRepositoryManager metadataManager = (IMetadataRepositoryManager)
	 * provisioningAgent.getService(IMetadataRepositoryManager.SERVICE_NAME); if (metadataManager == null) { logger.error("Metadata manager was null"); Throwable throwable = new Throwable("Could not load Metadata Repository Manager");
	 * throwable.fillInStackTrace(); throw new InvocationTargetException(throwable); }
	 * 
	 * // Load artifact manager IArtifactRepositoryManager artifactManager = (IArtifactRepositoryManager) provisioningAgent.getService(IArtifactRepositoryManager.SERVICE_NAME); if (artifactManager == null) { logger.error("Artifact manager was null");
	 * Throwable throwable = new Throwable("Could not load Artifact Repository Manager"); throwable.fillInStackTrace(); throw new InvocationTargetException(throwable); }
	 * 
	 * // Load repo try { URI repoLocation = new URI("http://localhost/respository"); logger.info("Adding repository " + repoLocation); metadataManager.loadRepository(repoLocation, null); artifactManager.loadRepository(repoLocation, null); } catch
	 * (ProvisionException pe) { logger.error("Caught provisioning exception " + pe.getMessage(), pe); throw new InvocationTargetException(pe); } catch (URISyntaxException e) { logger.error("Caught URI syntax exception " + e.getMessage(), e); throw
	 * new InvocationTargetException(e); } }
	 */

	/*
	public static IProxyService getProxyService() {
		BundleContext bc = Activator.getContext();
		ServiceReference serviceReference = bc.getServiceReference(IProxyService.class.getName());
		IProxyService service = (IProxyService) bc.getService(serviceReference);
		return service;
	}*/

	//public void setProxy() {
		
		//ProxySelector.setDefault(new CustomProxySelector());
		
		/*
		Authenticator.setDefault(new Authenticator() {
		    @Override
		    protected PasswordAuthentication getPasswordAuthentication() {
               return new PasswordAuthentication("login", "password".toCharArray());
		    }
		});*/
		
		/*
		IProxyData[] proxyData = getProxyService().getProxyData();
		for (IProxyData data : proxyData) {
			if (IProxyData.HTTP_PROXY_TYPE.equals(data.getType()) || IProxyData.HTTPS_PROXY_TYPE.equals(data.getType())) {
				data.setHost("proxyhost");
				data.setPort(8080);
				//if (authRequired) {
				//	data.setUserid(user);
				//	data.setPassword(password);
				//} else {
				//	data.setUserid(null);
				//	data.setPassword(null);
				//}
			}
		}
		getProxyService().setSystemProxiesEnabled(false);
		getProxyService().setProxiesEnabled(true);*/
	//}
}