package com.perdemia.permissionanalyzer.application.handlers.menu;

import java.io.File;
import java.lang.reflect.Field;
import java.util.logging.Handler;

import com.perdemia.permissionanalyzer.application.logging.PluginFileLogHandler;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OpenLogFileHandler {
	private final static Logger logger = LoggerFactory.getLogger(OpenLogFileHandler.class.getName());
	
	/** Use reflection to get the actual file path to the log file */
	private static Field files;
	
	static {
		try {
			files = Class.forName("java.util.logging.FileHandler").getDeclaredField("files");
			files.setAccessible(true);
		} catch (Exception ex) {
			logger.error("Error while determining path to the log file.", ex);
			MessageDialog.openError(null, "Error", "Error while determining path to the log file: " + ex.getMessage());
		}
		
	}
	
	@Execute
	public void execute(Shell shell){
		try {
			File logFile = getLogFile();
			if (logFile != null) {
				Program.launch(logFile.getCanonicalPath());
			} else {
				throw new Exception("Could not determine log file path");
			}
		    
		    /* old AWT method
			if (System.getProperty("os.name").toLowerCase().contains("windows")) {
				String cmd = "rundll32 url.dll,FileProtocolHandler " + logFile.getCanonicalPath();
				Runtime.getRuntime().exec(cmd);
			} else {
				Desktop.getDesktop().edit(logFile);
			}*/
		    
		} catch (Exception e) {
			MessageDialog.openError(null, "Error", "Error while opening the application log file:\r\n\r\n" + e.getMessage());
		}
	}
	
	/**
	 * When the application crashes the lock files are not cleaned up. In that case we will get a log file name like "application0.log.5".
	 * It seems that java logging doesn't have a public method to determine the actual log file path. This method uses reflection to get the 
	 * <code>Files</code> property of the class <code>java.util.logging.FileHandler</code>.
	 * 
	 * @return File reference to the actual log file
	 */
	public static File getLogFile() {
		java.util.logging.Logger rootJdkLogger = java.util.logging.Logger.getLogger("");
		
		// lookup the log file
		try {
		    for (Handler handler : rootJdkLogger.getHandlers()) {
		    	if (handler instanceof PluginFileLogHandler) {
		    		File[] logFiles = (File[]) files.get(handler);
		    		return logFiles[0];
		    	}
		    }
		} catch (IllegalAccessException ex) {
			return null;
		}
	    
	    return null;
	}

}
