package com.perdemia.permissionanalyzer.application.handlers.menu;

import java.util.HashMap;
import java.util.Map;

import javax.inject.Named;

import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDetailsDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.parts.FilterFolderPart;
import com.perdemia.permissionanalyzer.application.parts.FilterMemberPart;
import com.perdemia.permissionanalyzer.application.parts.FilterPermissionPart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.data.FilterSet;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.Report;

public class SaveReportHandler {

	@Execute
	public void execute(Shell shell, EPartService partService, final ECommandService commandService, final EHandlerService handlerService,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.savereport.param.ispolicy") final String isPolicy){
		MPart part = partService.findPart(FilePart.ID);
		FilePart filePart = (FilePart) part.getObject();
		
		part = partService.findPart(FilterMemberPart.ID);
		FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
		
		part = partService.findPart(FilterPermissionPart.ID);
		FilterPermissionPart filterPermissionPart = (FilterPermissionPart) part.getObject();
		
		part = partService.findPart(FilterFolderPart.ID);
		FilterFolderPart filterFolderPart = (FilterFolderPart) part.getObject();
		
		FilterSet memberFilterSet = filterMemberPart != null ? filterMemberPart.getSelectedFilterSet() : null;
		FilterSet permissionFilterSet = filterPermissionPart != null ? filterPermissionPart.getSelectedFilterSet() : null;
		FilterSet folderFilterSet = filterFolderPart != null ? filterFolderPart.getSelectedFilterSet() : null;
		
		if (FilterHelper.getInstance().hasUnappliedChanges()) {
			if (MessageDialog.openConfirm(shell, "Unapplied filters", "Some of the filters have not been applied yet and will not be used.\r\n\r\nDo you want to apply the filters first?")) {
				// trigger the refresh command
				ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.applyfilter", null);
				handlerService.executeHandler(applyFilterCommand);
			}
		}
		
		Report selectedReport = FilterHelper.getInstance().getSelectedReport();
		boolean isSelectedReportPolicy = selectedReport != null && selectedReport.isPolicy();
		boolean isSavingPolicy = isPolicy != null;
		String currentEntity = isSelectedReportPolicy ? "policy" : "report";
		String newEntity = isSavingPolicy ? "policy" : "report";
		
		// existing report and of the same type? (policy or report)
		if (selectedReport != null && isSelectedReportPolicy == isSavingPolicy) {
			int result = SyncMessageDialog.showQuestionWithCancelMessage("Use current " + currentEntity + "?", "Do you want to save your changes in the current " + currentEntity +  " \"" + selectedReport.getName() + "\"?");
			if (result == IDialogConstants.NO_ID) {
				// start a new report or policy
				selectedReport = null;
			} else if (result == IDialogConstants.CANCEL_ID) {
				return;
			}
		} else {
			// either we don't have a current report/policy, or we want to save another type (policy or report)
			selectedReport = null;
		}
		
		ReportDetailsDialog dialog = new ReportDetailsDialog(shell, isSavingPolicy, selectedReport, memberFilterSet, permissionFilterSet, folderFilterSet);
    	
    	if (dialog.open() == IDialogConstants.OK_ID) {
    		// enter the report or policy name in the main File Tree
    		String reportName = "    (" + newEntity + ": " + dialog.getSelectedReport().getName() + ")";;
			FilterHelper.getInstance().setSelectedReport(dialog.getSelectedReport());
			 
    		if (filePart.getTreeViewer().getTree().isVisible()) {
				filePart.getTreeViewer().getTree().getColumn(0).setText("Path - " + FilterHelper.getInstance().getItemCount() + " items found" + reportName);
    		} else {
    			filePart.getTableViewer().getTable().getColumn(0).setText("Path - " + FilterHelper.getInstance().getItemCount() + " items found" + reportName);
    		}  
    		
    		// refresh the data to display the report data
    		Map<String, Object> parameters = new HashMap<String, Object>();
    		if (isSavingPolicy) {
    			parameters.put("com.perdemia.permissionanalyzer.application.commands.managereports.param.ispolicy", "true");
    		}
    		
			ParameterizedCommand manageReportsDialogCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.managereports", parameters);
			handlerService.executeHandler( manageReportsDialogCommand );	
    	}
	}
	
	@CanExecute
	public boolean canExecute() {
		return Member.isUserGroupCountAllowed() && !LicenseHelper.getEdition().equals(Edition.SCAN_AGENT);
	}
}	
