package com.perdemia.permissionanalyzer.application.handlers.toolbar;

import java.util.List;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.parts.AllMembersPart;
import com.perdemia.permissionanalyzer.application.parts.EffectivePermissionsPart;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.data.Member;

public class CopyMembersToClipboardHandler {
	private static final Logger logger = LoggerFactory.getLogger(CopyMembersToClipboardHandler.class.getName());
	
	@Execute
	public void execute(Shell shell, MPart mPart){
		ApplicationUtil.setBusyCursor();
		
		if (mPart != null && mPart.getObject() != null) {
			try {
				List<Member> members;
							
				if (mPart.getObject() instanceof EffectivePermissionsPart) {
					EffectivePermissionsPart effectivePermissionsPart = (EffectivePermissionsPart) mPart.getObject();
					
					// in case no member filter has been selected AND the search box is empty, the TreeViewer will not be completely expanded
					// see EffectivePermissionsPartContentProvider#inputChanged
					// in that case we need to retrieve all members, like a search "*" would do (for our CSV export)
					if (!FilterHelper.getInstance().isApplyMemberFilter() && effectivePermissionsPart.getContentProviderConfig().getMemberSearch().isEmpty()) {
						// explicitly load all members, since the TreeViewer has been filled lazy loaded
						members = effectivePermissionsPart.getContentProvider().searchMembers();
					} else {
						members = effectivePermissionsPart.getContentProvider().getDirectMembers();
					}
				} else {
					AllMembersPart allMembersPart = (AllMembersPart) mPart.getObject();
					
					int result = SyncMessageDialog.showQuestionWithCancelMessage("Copy all members?", "Do you want to copy all the members in this tab?\r\nIf you select No then all the ACE's of the selected member will be copied" +
								"\r\n\r\n(If you want to copy the entire content of this tab, then create an export with report type \"Users/groups and all their explicit permissions\")");
					
					if (result == IDialogConstants.CANCEL_ID) {
						ApplicationUtil.setDefaultCursor();
						return;
					}

					if (result == IDialogConstants.NO_ID) {
						// don't copy the members, but copy the ACE's of the selected member
						CopyAclItemsToClipboardHandler copyAclHandler = new CopyAclItemsToClipboardHandler();
						copyAclHandler.execute(shell, mPart);
						ApplicationUtil.setDefaultCursor();
						return;
					}
					
					members = allMembersPart.getFilteredMembers();
				}
				
				StringBuilder membersText = new StringBuilder("Account name,Display name\r\n");
	
				for (Member member : members) {
					membersText.append(member.getName(true));
					membersText.append(",");
					membersText.append(member.getDisplayName());
					membersText.append("\r\n");
				}
				
				Clipboard clipboard = new Clipboard(Display.getDefault());
				TextTransfer textTransfer = TextTransfer.getInstance();
		        clipboard.setContents(new String[]{membersText.toString()}, new Transfer[]{textTransfer});
		        clipboard.dispose();
		        SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "Copied " + members.size() + " members to the clipboard.");
		        
			} catch (Exception ex) {
				logger.error("Error while copying members to the clipboard from " + mPart.getObject().getClass().getSimpleName(), ex);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while copying members to the clipboard: " + ex.getMessage());
			}
		}
		
		ApplicationUtil.setDefaultCursor();
	}
}
