package com.perdemia.permissionanalyzer.application.handlers.toolbar;

import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDetailsDialog;
import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.data.FileInfo;

public class ExportHandler {
	
	@Execute
	public void execute(Shell shell, EPartService partService, final ECommandService commandService, final EHandlerService handlerService){
		MPart part = partService.findPart(FilePart.ID);
		FilePart fileTreePart = (FilePart) part.getObject();
		
		StructuredSelection selection;
		if (fileTreePart.getTreeViewer().getTree().isVisible()) {
			selection = (StructuredSelection) fileTreePart.getTreeViewer().getSelection();
		} else {
			selection = (StructuredSelection) fileTreePart.getTableViewer().getSelection();
		}
		
		if (FilterHelper.getInstance().hasUnappliedChanges()) {
			if (MessageDialog.openConfirm(shell, "Unapplied filters", "Some of the filters have not been applied yet and will not be used in the export.\r\n\r\nDo you want to apply the filters first?")) {
				// trigger the refresh command
				ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.applyfilter", null);
				handlerService.executeHandler(applyFilterCommand);
			}
		}
		
		ReportDetailsDialog dialog = new ReportDetailsDialog(shell, (FileInfo) selection.getFirstElement());
    	
    	if (dialog.open() == IDialogConstants.OK_ID) {
    		if (dialog.isExportSelectedFile()) {
    			// only selected file and children
    			ExportHelper.exportReport(dialog.getSelectedReport(), (FileInfo) selection.getFirstElement(), dialog.isExportSubdirectories(), true);
    		} else {
    			// all files/folders 
    			ExportHelper.exportReport(dialog.getSelectedReport(), null, true, true);
    		}
    	}
	}
}
