package com.perdemia.permissionanalyzer.application.handlers.toolbar;

import gnu.trove.list.array.TIntArrayList;

import java.lang.reflect.InvocationTargetException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Named;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.impl.PartImpl;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ModifyAceDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.parts.AclPart;
import com.perdemia.permissionanalyzer.application.parts.AllMembersPart;
import com.perdemia.permissionanalyzer.application.parts.EffectivePermissionsPart;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.PermissionLookup;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;
import com.perdemia.permissionanalyzer.data.util.CustomProcessor;
import com.perdemia.permissionanalyzer.scanner.acl.AceReader;
import com.perdemia.permissionanalyzer.scanner.acl.JnaAceReader;
import com.sun.jna.platform.win32.Advapi32Util;
import com.sun.jna.platform.win32.WinNT;

public class ModifyACEHandler {
	private static final Logger logger = LoggerFactory.getLogger(ModifyACEHandler.class.getName());
	private AclItem selectedAclItem;
	private boolean hide;
		
	@CanExecute
	boolean isVisible(@Named(IServiceConstants.ACTIVE_PART) MPart part) {
		return !hide;
	}

	@SuppressWarnings("restriction")
	@PostConstruct
	public void postConstruct(MApplication application, EModelService modelService) {
		hide = "true".equalsIgnoreCase(System.getProperty("hideModifyAce")) || PreferenceHelper.getInstance().isHideModifyAce();
		
		if (hide) {
			PartImpl part = (PartImpl) modelService.find("com.perdemia.permissionanalyzer.application.parts.effectivepermissions", application);
			if (part != null) {
				part.getToolbar().getChildren().get(2).setVisible(false);
			}
			part = (PartImpl) modelService.find("com.perdemia.permissionanalyzer.application.parts.acl", application);
			if (part != null) {
				part.getToolbar().getChildren().get(2).setVisible(false);
			}
			part = (PartImpl) modelService.find("com.perdemia.permissionanalyzer.application.parts.allmembers", application);
			if (part != null) {
				part.getToolbar().getChildren().get(0).setVisible(false);
			}
		}
	}
	
	@Execute
	public void execute(Shell shell, MPart viewpart){
		ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
		
		if (viewpart != null && viewpart.getObject() != null) {
			if (viewpart.getObject() instanceof AclPart) {
				AclPart aclPart = (AclPart) viewpart.getObject();
				selectedAclItem = aclPart.getSelectedAclItem(); // selected ACE in the GUI
			} else if (viewpart.getObject() instanceof EffectivePermissionsPart) {
				EffectivePermissionsPart effectivePermissionsPart = (EffectivePermissionsPart) viewpart.getObject();
				selectedAclItem = effectivePermissionsPart.getSelectedAclItem(); // selected ACE in the GUI
			} else {
				AllMembersPart allMembersPart = (AllMembersPart) viewpart.getObject();
				selectedAclItem = allMembersPart.getSelectedAclItem(); // selected ACE in the GUI
			}
			
			if (selectedAclItem != null) {
				try {
					final Path path = Paths.get(selectedAclItem.getFile().determineFullPath());
					//final Path path = Paths.get("c:\\temp");
					
					// get the security descriptor for this file
					// NOTE: The Advapi32Util.getFileSecurityDescriptor method has been modified! In case the path startsWith \\ and 
					// the length is longer than 248 characters it will convert to path to a short notation (~1 etc). Otherwise the 
					// getNamedSecurityInfo will throw an error "The filename, directory name, or volume label syntax is incorrect."
					final WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor = Advapi32Util.getFileSecurityDescriptor(path.toFile(), false);;
					
					// read the ACE's
					AceReader aceReader = new JnaAceReader();
					List<AclItem> aclItems = aceReader.getAclList(path, true, null, secDescriptor);
					
					int aclItemIndex = 0;
					for (int counter = 0; counter < aclItems.size(); counter++) {
						AclItem nextAclItem = aclItems.get(counter);
						if (nextAclItem.getMember(false).getName(true).equalsIgnoreCase(selectedAclItem.getMember(false).getName(true)) &&
								nextAclItem.getPermissionId().equals(selectedAclItem.getPermissionId()) &&
								nextAclItem.isGenericAll() == selectedAclItem.isGenericAll() &&
								nextAclItem.isGenericWrite() == selectedAclItem.isGenericWrite() &&
								nextAclItem.isGenericExecute() == selectedAclItem.isGenericExecute() &&
								nextAclItem.isGenericRead() == selectedAclItem.isGenericRead() &&
								nextAclItem.isDeny() == selectedAclItem.isDeny() &&
								nextAclItem.isFlagDirectoryInherit() == selectedAclItem.isFlagDirectoryInherit() &&
								nextAclItem.isFlagFileInherit() == selectedAclItem.isFlagFileInherit() &&
								nextAclItem.isFlagInheritOnly() == selectedAclItem.isFlagInheritOnly() &&
								nextAclItem.isFlagNoPropagateInherit() == selectedAclItem.isFlagNoPropagateInherit()) {
							
							// found matching aclItem
							aclItemIndex = counter;
							break;
						}
					}
					
					if (aclItemIndex == -1) {
						logger.warn("The selected Access Control Entry could not be found on the selected folder, member: " + selectedAclItem.getMember(true).getName(true) + ", ace: " + selectedAclItem.toString());
						logger.warn("AceList on the selected folder while modifying ACE: " + aclItems.toString());
						MessageDialog.openError(shell, "ACE not found", "The selected Access Control Entry could not be found on the selected folder.");
						return;
					}
					
					// show the Modify ACE dialog
					ModifyAceDialog dialog = new ModifyAceDialog(shell, selectedAclItem.clone());
					
					int result = dialog.open();
					
					final AclItem modifiedAclItem = dialog.getAclItem();
					
					if (result != IDialogConstants.CANCEL_ID) {
						
						ApplicationUtil.setBusyCursor();
						progressDialog.open();
						progressDialog.getProgressMonitor().beginTask("Updating ACL data on file system...", IProgressMonitor.UNKNOWN);
												
						// DACL
						WinNT.ACL dacl = secDescriptor.getDiscretionaryACL();
						
						// List of ACE's
						//WinNT.ACCESS_ACEStructure[] aces = dacl.getACEStructures();
						WinNT.ACE_HEADER[] aceHeaders = dacl.getACEs();
												
						if (result == ModifyAceDialog.UPDATE_ID || result == ModifyAceDialog.ADD_ID) {
							WinNT.ACE_HEADER aceHeader = aceHeaders[aclItemIndex];
							
							if (aceHeader instanceof WinNT.ACCESS_ACEStructure) {
								WinNT.ACCESS_ACEStructure ace = (WinNT.ACCESS_ACEStructure) aceHeader;
								
								// copy the information from the dialog to the ACE							
								// allow or deny
								ace.AceType = modifiedAclItem.isDeny() ? WinNT.ACCESS_DENIED_ACE_TYPE : WinNT.ACCESS_ALLOWED_ACE_TYPE;
															
								// flags
								byte flags = 0;
								// existing flags?
								if ((ace.AceFlags & WinNT.INHERITED_ACE) > 0) {
									flags |= WinNT.INHERITED_ACE;
								}
															
								// flags set by user
								if (modifiedAclItem.isFlagDirectoryInherit()) {
									flags |= WinNT.CONTAINER_INHERIT_ACE;
								}
								if (modifiedAclItem.isFlagFileInherit()) {
									flags |= WinNT.OBJECT_INHERIT_ACE;
								}
								if (modifiedAclItem.isFlagInheritOnly()) {
									flags |= WinNT.INHERIT_ONLY_ACE;
								}
								if (modifiedAclItem.isFlagNoPropagateInherit()) {
									flags |= WinNT.NO_PROPAGATE_INHERIT_ACE;
								}
								ace.AceFlags = flags;
															
								// permissions
								int accessMask = 0;
								
								// synchronize
								if ((ace.Mask & SpecialPermission.SYNCHRONIZE.getBitValue()) > 0) {
									accessMask |= SpecialPermission.SYNCHRONIZE.getBitValue();
								}
								for (int counter = 0; counter < 13; counter++) {
									if (PermissionLookup.containsSpecialPermission(modifiedAclItem.getPermissionId(), counter)) {
										accessMask |= SpecialPermission.values()[counter].getBitValue();
									}
								}							
								
								// generic permissions
								if (modifiedAclItem.isGenericAll()) {
									accessMask |= JnaAceReader.GENERIC_ALL_BIT;
								}
								if (modifiedAclItem.isGenericExecute()) {
									accessMask |= JnaAceReader.GENERIC_EXECUTE_BIT;
								}
								if (modifiedAclItem.isGenericRead()) {
									accessMask |= JnaAceReader.GENERIC_READ_BIT;
								}
								if (modifiedAclItem.isGenericWrite()) {
									accessMask |= JnaAceReader.GENERIC_WRITE_BIT;
								}
								
								ace.Mask = accessMask;
								
								// add the AclEntry?
								if (result == ModifyAceDialog.ADD_ID) {
									// TODO add a new ace to the array and update the DACL, use AddAccessAllowedAceEx and AddAccessDenyAceEx?
									//WinNT.ACCESS_ACEStructure[] newAces = Arrays.copyOf(aces, aces.length + 1);
									//newAces[aces.length] = <<<NEW ACE BASED on ace>>;
								 //   dacl.setACEStructures(newAces);
								}
								
								ace.write();
							} else {
								logger.error("Error while modifying Access Control Entry, unsupported ACE type " + aceHeader.AceType);
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while modifying Access Control Entry, unsupported ACE type " + aceHeader.AceType);
							}
						} else if (result == ModifyAceDialog.DELETE_ID) {
							// TODO removing an ACE from the DACL doesn't work, the array is just a layer over the memory and structure fields
							// remove this ace from the array and update the DACL							
//							WinNT.ACCESS_ACEStructure[] newAces = Arrays.copyOf(aces, aces.length - 1);
//							System.arraycopy(aces, 0, newAces, 0, aclItemIndex);
//
//					        if (aclItemIndex < aces.length - 1) {
//					            System.arraycopy(aces, aclItemIndex + 1, newAces, aclItemIndex, aces.length - aclItemIndex - 1);
//					        }
//					        
//					        dacl.setACEStructures(newAces);
						}
											
						/*
						 ACL changes made with AclFileAttributeView.setAcl are only set on the folder/file and are not propagated to child folders/file.
						 This is because setAcl relies on the low-level API method SetFileSecurity. 
						 See https://support.microsoft.com/en-us/kb/266461
						 Instead of the low-level API call it is also possible to use the more abstract method SetNamedSecurityInfo
						 See https://msdn.microsoft.com/en-us/library/aa379579%28v=vs.85%29.aspx
						 
						 This abstract method SetNamedSecurityInfo is support by the Java Native Access project:
						 See https://github.com/twall/jna
						*/	
						progressDialog.run(true, false, new IRunnableWithProgress(){

							@Override
							public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
								try {
									Advapi32Util.setFileSecurityDescriptor(path.toFile(), secDescriptor, false, false, true, false, false, false);
								
									// process the changes in the database (child folders/files have been changed by the setFileSecurityDescriptor call)
									monitor.beginTask("Updating ACL data in the database...", IProgressMonitor.UNKNOWN);
								
									changeAclItem(selectedAclItem, modifiedAclItem);
									
									SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "The permission(s) have been modified, apply your filter again to view the updated permission(s)");
								} catch (Exception e) {
									logger.error("Error while modifying ACE " + selectedAclItem.toString(), e);
									SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while modifying ACE: " + GUILogHandler.getNestedMessage(e));
								}
							}
							
						});
					}									
					
				} catch (Exception e) {
					logger.error("Error while modifying ace " + selectedAclItem.toString(), e);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while modifying ACE: " + GUILogHandler.getNestedMessage(e));
					
				} finally {
					progressDialog.close();
					ApplicationUtil.setDefaultCursor();
				}
				
				
			} else {
				MessageDialog.openError(shell, "No ACE selected", "Please select a Access Control Entry to modify.");
			}
		}
	}
	
	/**
	 * Changes a AclItem in the database and makes sure that the inherited AclItems are also updated based on the new ACE Flags.
	 * 
	 * @param currentAce
	 * @param newAce
	 */
	private void changeAclItem(AclItem currentAce, AclItem newAce) throws DBException, SQLException {
		// determine the changes we have to process
		boolean noPropagateEnabled = !currentAce.isFlagNoPropagateInherit() && newAce.isFlagNoPropagateInherit();
		boolean noPropagateDisabled = currentAce.isFlagNoPropagateInherit() && !newAce.isFlagNoPropagateInherit();
		boolean directoryInheritEnabled = !currentAce.isFlagDirectoryInherit() && newAce.isFlagDirectoryInherit();
		boolean directoryInheritDisabled = currentAce.isFlagDirectoryInherit() && !newAce.isFlagDirectoryInherit();
		boolean fileInheritEnabled = !currentAce.isFlagFileInherit() && newAce.isFlagFileInherit();
		boolean fileInheritDisabled = currentAce.isFlagFileInherit() && !newAce.isFlagFileInherit();
		
//		boolean deleteFromIndirectSubfolders = (currentAce.isFlagDirectoryInherit() && !newAce.isFlagDirectoryInherit() && !currentAce.isFlagNoPropagateInherit()) || (noPropagateEnabled);
//		boolean deleteFromDirectSubfolders = (currentAce.isFlagDirectoryInherit() && !newAce.isFlagDirectoryInherit() && currentAce.isFlagNoPropagateInherit());
//		boolean deleteFromIndirectSubfiles = (currentAce.isFlagFileInherit() && !newAce.isFlagFileInherit() && !currentAce.isFlagNoPropagateInherit()) || (noPropagateEnabled);
//		boolean deleteFromDirectSubfiles = (currentAce.isFlagFileInherit() && !newAce.isFlagFileInherit() && currentAce.isFlagNoPropagateInherit());		
				
		// if file inherit is still enabled then the folders will be updated instead of deleted! Folders will get the file inherit flag with the inherit only flag.
		// except when the no propagation flag has been enabled, in that case folder cannot propagate and won't get an ACE.
		// the second rule will check if the ACE had only file inheritance, whereafter the no propagation flag was enabled. In that case the ACE is removed from all subfolders.
		boolean deleteFromSubFolders = (directoryInheritDisabled && (!newAce.isFlagFileInherit() || newAce.isFlagNoPropagateInherit())) || 
						(noPropagateEnabled && newAce.isFlagFileInherit() && !newAce.isFlagDirectoryInherit()) ||
						(fileInheritDisabled && !newAce.isFlagDirectoryInherit()); 
		boolean deleteFromSubFiles = fileInheritDisabled;
		
		boolean updateInSubFiles = currentAce.isFlagFileInherit() && newAce.isFlagFileInherit();
		boolean updateInSubFolders = (currentAce.isFlagDirectoryInherit() && newAce.isFlagDirectoryInherit()) || updateInSubFiles;
		
		boolean addInheritOnly = currentAce.isFlagFileInherit() && newAce.isFlagFileInherit() && !newAce.isFlagDirectoryInherit() && !newAce.isFlagNoPropagateInherit();
		boolean removeInheritOnly = currentAce.isFlagFileInherit() && directoryInheritEnabled;
		//boolean addToSubFolders = !currentAce.isFlagDirectoryInherit() && newAce.isFlagDirectoryInherit();
		//boolean addToSubFiles = !currentAce.isFlagFileInherit() && newAce.isFlagFileInherit();
		
		// why currentAce.isFlagNoPropagateInherit(), what if the currentAce has no flags at all? 
		//boolean addToIndirectSubfiles = (fileInheritEnabled && currentAce.isFlagNoPropagateInherit()) || noPropagateDisabled;
		boolean addToIndirectSubfiles = (fileInheritEnabled && !newAce.isFlagNoPropagateInherit()) || (noPropagateDisabled && newAce.isFlagFileInherit());
		boolean addToDirectSubfiles = fileInheritEnabled;
		//boolean addToDirectSubfolders = (directoryInheritEnabled) || addToDirectSubfiles;
		boolean addToDirectSubfolders = (!currentAce.isFlagFileInherit() && !currentAce.isFlagDirectoryInherit() && (directoryInheritEnabled || fileInheritEnabled)) ||
				(currentAce.isFlagFileInherit() && !currentAce.isFlagDirectoryInherit() && noPropagateDisabled); // was only files with noprop, meaning no subfolders. Now the subfolders will get an ACE with FILE_INHERTI and INHERIT_ONLY
		//boolean addToIndirectSubfolders = (!currentAce.isFlagDirectoryInherit() && newAce.isFlagDirectoryInherit() && !newAce.isFlagNoPropagateInherit()) || noPropagateDisabled || addToIndirectSubfiles;
		boolean addToIndirectSubfolders = (addToDirectSubfolders && !newAce.isFlagNoPropagateInherit()) || (noPropagateDisabled && newAce.isFlagDirectoryInherit());
		
		
		//-----------------------------
		// DELETE ACTIONS
		//-----------------------------
		String deleteQuery = null;
		if (deleteFromSubFolders && deleteFromSubFiles) {
			// delete the ace on all sub folders and files
			deleteQuery = "DELETE FROM {0}.ACLITEM WHERE INHERITED_FROM_FOLDER_ID = {1} AND PERMISSION_ID = {2} AND MEMBER_ID = {3}";
			
		} else if (deleteFromSubFolders) {
			// only delete the ace on the sub folders
			deleteQuery = "DELETE FROM {0}.ACLITEM WHERE INHERITED_FROM_FOLDER_ID = {1} AND PERMISSION_ID = {2} AND MEMBER_ID = {3} AND IS_DIRECTORY = TRUE";
			
		} else if (deleteFromSubFiles) {
			// only delete the ace on the sub files
			deleteQuery = "DELETE FROM {0}.ACLITEM WHERE INHERITED_FROM_FOLDER_ID = {1} AND PERMISSION_ID = {2} AND MEMBER_ID = {3} AND IS_DIRECTORY = FALSE";
		}			
		
		if (deleteQuery != null) {
			deleteQuery = MessageFormat.format(deleteQuery, DBObject.SCHEMA, currentAce.getFileId(), String.valueOf(currentAce.getPermissionId()), currentAce.getMemberId());
			DBObject.executeStatement(deleteQuery);
			deleteQuery = null; //reset
		}
		
		if (noPropagateEnabled) {
			// delete all indirect children
			// ("delete a from a inner join b" doesn't work in H2, therefore the IN statement is being used)
			deleteQuery = "DELETE FROM {0}.ACLITEM WHERE FILEINFO_ID IN (SELECT ID FROM {0}.FILEINFO WHERE PARENT_ID <> {1}) AND INHERITED_FROM_FOLDER_ID = {1} AND PERMISSION_ID = {2} AND MEMBER_ID = {3}";
		}
		
		if (deleteQuery != null) {
			deleteQuery = MessageFormat.format(deleteQuery, DBObject.SCHEMA, currentAce.getFileId(), String.valueOf(currentAce.getPermissionId()), currentAce.getMemberId());
			DBObject.executeStatement(deleteQuery);
		} 
		
		
		
		/*
		String deleteQuery = "DELETE FROM {0}.ACLITEM INNER JOIN {0}.FILEINFO ON ACLITEM.FILE_ID = FILEINFO.ID WHERE " +
				"INHERITED_FROM_FOLDER_ID = {1} AND PERMISSION_ID = {2} AND MEMBER_ID = {3}";		
		String deleteQuery1 = null;
		String deleteQuery2 = null;
		String deleteQuery3 = null;
		String deleteQuery4 = null;
		if (deleteFromIndirectSubfolders) {
			deleteQuery1 = deleteQuery;
		}
		if (deleteFromDirectSubfolders) {
			deleteQuery2 = deleteQuery;
		}
		if (deleteFromIndirectSubfiles) {
			deleteQuery3 = deleteQuery;
		}
		if (deleteFromDirectSubfiles) {
			deleteQuery4 = deleteQuery;
		}
		
		
//		1) if query1 != query3, then filter on is_directory for query1 en 3
//		2) if query2 != query4, then filter on is_directory for query2 en 4
//		3) if query1 != query2, then filter on parent_id for query1 en 2
//		4) if query3 != query4, then filter on parent_id for query3 en 4
//		5) if all queries then don't filter on is_directory and parent_id
		
		//1)
		if (deleteFromIndirectSubfolders != deleteFromIndirectSubfiles) {
			if (deleteFromIndirectSubfolders) {
				deleteQuery1 += " AND IS_DIRECTORY = TRUE";
			}
			if (deleteFromIndirectSubfiles) {
				deleteQuery3 += " AND IS_DIRECTORY = FALSE";
			}
		}
		//2)
		if (deleteFromDirectSubfolders != deleteFromDirectSubfiles) {
			if (deleteFromDirectSubfolders) {
				deleteQuery2 += " AND IS_DIRECTORY = TRUE";
			}
			if (deleteFromDirectSubfiles) {
				deleteQuery4 += " AND IS_DIRECTORY = FALSE";
			}
		}
		//3)
		if (deleteFromIndirectSubfolders != deleteFromDirectSubfolders) {
			if (deleteFromIndirectSubfolders) {
				deleteQuery1 += " AND PARENT_ID <> {4}";
			}
			if (deleteFromDirectSubfolders) {
				deleteQuery2 += " AND PARENT_ID = {4}";
			}
		}
		//4)
		if (deleteFromIndirectSubfiles != deleteFromDirectSubfiles) {
			if (deleteFromIndirectSubfiles) {
				deleteQuery3 += " AND PARENT_ID <> {4}";
			}
			if (deleteFromDirectSubfiles) {
				deleteQuery4 += " AND PARENT_ID = {4}";
			}
		}
		
		// add all the query to a Set to prevent duplicates
		Set<String> deleteQueries = new HashSet<String>();
		if (deleteQuery1 != null) {
			deleteQueries.add(deleteQuery1);
		}
		if (deleteQuery2 != null) {
			deleteQueries.add(deleteQuery1);
		}
		if (deleteQuery3 != null) {
			deleteQueries.add(deleteQuery1);
		}
		if (deleteQuery4 != null) {
			deleteQueries.add(deleteQuery1);
		}
		
		for (String query : deleteQueries) {
			query = MessageFormat.format(query, DBObject.SCHEMA, currentAce.getFileId(), currentAce.getPermissionId(), currentAce.getMemberId(), currentAce.getFileId());
			DBObject.executeQuery(deleteQuery);
		}*/
		
		
		//-----------------------------
		// UPDATE ACTIONS
		//-----------------------------
		
		// update the currentAce so that the GUI is updated as well
		currentAce.setDeny(newAce.isDeny());
		currentAce.setFlagDirectoryInherit(newAce.isFlagDirectoryInherit());
		currentAce.setFlagFileInherit(newAce.isFlagFileInherit());
		currentAce.setFlagInheritedAce(newAce.isFlagInheritedAce());
		currentAce.setFlagInheritOnly(newAce.isFlagInheritOnly());
		currentAce.setFlagNoPropagateInherit(newAce.isFlagNoPropagateInherit());
		currentAce.setPermissionId(newAce.getPermissionId());
		currentAce.setGenericAll(newAce.isGenericAll());
		currentAce.setGenericRead(newAce.isGenericRead());
		currentAce.setGenericExecute(newAce.isGenericExecute());
		currentAce.setGenericWrite(newAce.isGenericWrite());
		currentAce.save();
		
		// flags for the subfolders (only inherited when the current folder has no 'No Propagate' flag (otherwise the inheritance flags are removed from the subfolders)
		boolean flagDirectoryInherit = newAce.isFlagNoPropagateInherit() ? false : newAce.isFlagDirectoryInherit();
		boolean flagFileInherit = newAce.isFlagNoPropagateInherit() ? false : newAce.isFlagFileInherit();
		if (newAce.isFlagNoPropagateInherit()) {
			removeInheritOnly = true;
		}
		
		// update directories
		// (UPDATE {0}.ACLITEM INNER JOIN {0}.FILEINFO ON ACLITEM.FILE_ID = FILEINFO.ID SET .... doesn't work in H2. Therefore we copied the flag IS_DIRECTORY to the ACLITEM table)
		if (updateInSubFolders || addInheritOnly || removeInheritOnly) {
			String updateQuery = "UPDATE {0}.ACLITEM SET PERMISSION_ID = {1}, FLAG_DIRECTORY_INHERIT = {2}, FLAG_FILE_INHERIT = {3}, IS_DENY = {4}, " + 
			(addInheritOnly || removeInheritOnly ? "FLAG_INHERIT_ONLY = {11}, " : "") + 
			"GENERIC_ALL = {5}, GENERIC_EXECUTE = {6}, GENERIC_WRITE = {7}, GENERIC_READ = {8} WHERE IS_DIRECTORY = TRUE AND INHERITED_FROM_FOLDER_ID = {9} AND MEMBER_ID = {10}";
			updateQuery = MessageFormat.format(updateQuery, DBObject.SCHEMA, String.valueOf(newAce.getPermissionId()), flagDirectoryInherit, flagFileInherit,
					newAce.isDeny(), newAce.isGenericAll(), newAce.isGenericExecute(), newAce.isGenericWrite(), newAce.isGenericRead(), newAce.getFileId(), newAce.getMemberId(),
					Boolean.toString(addInheritOnly));
			DBObject.executeStatement(updateQuery);
		}
		
		// update files (no update of the flags)
		if (updateInSubFiles) {
			String updateQuery = "UPDATE {0}.ACLITEM SET PERMISSION_ID = {1}, IS_DENY = {2}, GENERIC_ALL = {3}, GENERIC_EXECUTE = {4}, GENERIC_WRITE = {5}, GENERIC_READ = {6} WHERE IS_DIRECTORY = FALSE AND INHERITED_FROM_FOLDER_ID = {7} AND MEMBER_ID = {8}";
			updateQuery = MessageFormat.format(updateQuery, DBObject.SCHEMA, String.valueOf(newAce.getPermissionId()),
					newAce.isDeny(), newAce.isGenericAll(), newAce.isGenericExecute(), newAce.isGenericWrite(), newAce.isGenericRead(), newAce.getFileId(), newAce.getMemberId());
			DBObject.executeStatement(updateQuery);
		}
		
		//-----------------------------
		// ADD ACTIONS FOR FILES
		//-----------------------------
		String getSubFilesQuery = null;
		if (addToIndirectSubfiles && addToDirectSubfiles) {
			// get all subfiles
			// although we only want the files, we include directories as well because we need to check the flag IS_DACL_PROTECTED 
			getSubFilesQuery = "SELECT ID, PARENT_FILE_IDS, IS_DACL_PROTECTED, IS_DIRECTORY FROM {0}.FILEINFO WHERE PARENT_FILE_IDS LIKE {1}";
			
		} else if (addToIndirectSubfiles || addToDirectSubfiles) {
			// only get direct or indirect subfiles
			String compareOperator = addToIndirectSubfiles ? "<>" : "="; 
			getSubFilesQuery = "SELECT ID, PARENT_FILE_IDS, IS_DACL_PROTECTED, IS_DIRECTORY FROM {0}.FILEINFO WHERE PARENT_FILE_IDS LIKE {1} AND PARENT_ID " + compareOperator + " {2}";
		}
		
		if (getSubFilesQuery != null) {
			getSubFilesQuery = MessageFormat.format(getSubFilesQuery, DBObject.SCHEMA, "'" + currentAce.getFile().getParentFileIds() + ",%'", currentAce.getFileId());
		
			// Build the list with subfiles that will get the ACE added
			final TIntArrayList subfileIds = new TIntArrayList();
			final List<String> blockedInheritanceFiles = new ArrayList<String>();
			DBObject.getUsingCustomProcessor(DBObject.getPreparedStatement(getSubFilesQuery), new CustomProcessor() {
				@Override
				public boolean processResultSet(ResultSet rs) throws SQLException, DBException {
					// is the inheritance for this file already blocked by a previous file?
					boolean inheritanceBlocked = false;
					for (String blockedInheritanceFile : blockedInheritanceFiles) {
						if (rs.getString("PARENT_FILE_IDS").startsWith(blockedInheritanceFile)) {
							inheritanceBlocked = true;
							break;
						}
					}
					
					if (!inheritanceBlocked) {
						if (rs.getBoolean("IS_DACL_PROTECTED")) {
							blockedInheritanceFiles.add(rs.getString("PARENT_FILE_IDS"));
						} else {
							if (!rs.getBoolean("IS_DIRECTORY")) {
								subfileIds.add(rs.getInt("ID"));
							}
						}
					}
					
					return true;
				}
			});
		
			// now we have the list of applicable file id's, add the ACE to all these files (remove inheritance flags because they are not applicable for files) 
			AclItem fileAclItem = newAce.clone();
			fileAclItem.setInheritedFromFolderId(newAce.getFileId());
			// reset flags for files
			fileAclItem.setFlagDirectoryInherit(false);
			fileAclItem.setFlagFileInherit(false);
			fileAclItem.setFlagNoPropagateInherit(false);
			fileAclItem.setFlagInheritedAce(true);
			fileAclItem.setFlagInheritOnly(false); // can only apply to the parent folder
			fileAclItem.setDirectory(false);
			
			for (int counter = 0; counter < subfileIds.size(); counter++) {
				int fileId = subfileIds.get(counter);
				fileAclItem.setId(null); //reset to force an INSERT
				fileAclItem.setFileId(fileId);
				fileAclItem.save();
			}
		}
		
		//-----------------------------
		// ADD ACTIONS FOR DIRECTORIES
		//-----------------------------
		String getSubDirectoriesQuery = null;
		if (addToIndirectSubfolders && addToDirectSubfolders) {
			// add a new ace to all sub folders and files
			getSubDirectoriesQuery = "SELECT ID, PARENT_FILE_IDS, IS_DACL_PROTECTED FROM {0}.FILEINFO WHERE PARENT_FILE_IDS LIKE {1} AND IS_DIRECTORY = TRUE";
			
		} else if (addToIndirectSubfolders || addToDirectSubfolders) {
			// only add a new ace to sub folders or files
			String compareOperator = addToIndirectSubfolders ? "<>" : "="; 
			getSubDirectoriesQuery = "SELECT ID, PARENT_FILE_IDS, IS_DACL_PROTECTED FROM {0}.FILEINFO WHERE PARENT_FILE_IDS LIKE {1} AND IS_DIRECTORY = TRUE AND PARENT_ID " + compareOperator + " {2}";
		}
		
		if (getSubDirectoriesQuery != null) {
			getSubDirectoriesQuery = MessageFormat.format(getSubDirectoriesQuery, DBObject.SCHEMA, "'" + currentAce.getFile().getParentFileIds() + ",%'", currentAce.getFileId());
					
			// Build the list with subfiles that will get the ACE added
			final TIntArrayList subdirectoryIds = new TIntArrayList();
			final List<String> blockedInheritanceDirectories = new ArrayList<String>();
			DBObject.getUsingCustomProcessor(DBObject.getPreparedStatement(getSubDirectoriesQuery), new CustomProcessor() {
				@Override
				public boolean processResultSet(ResultSet rs) throws SQLException, DBException {
					// is the inheritance for this file already blocked by a previous file?
					boolean inheritanceBlocked = false;
					for (String blockedInheritanceDirectory : blockedInheritanceDirectories) {
						if (rs.getString("PARENT_FILE_IDS").startsWith(blockedInheritanceDirectory)) {
							inheritanceBlocked = true;
							break;
						}
					}
					
					if (!inheritanceBlocked) {
						if (rs.getBoolean("IS_DACL_PROTECTED")) {
							blockedInheritanceDirectories.add(rs.getString("PARENT_FILE_IDS"));
						} else {
							subdirectoryIds.add(rs.getInt("ID"));
						}
					}
					
					return true;
				}
				
			});
			
			// now we have the list of applicable directory id's, add the ACE to all these directories 
			AclItem directoryAclItem = newAce.clone();
			directoryAclItem.setInheritedFromFolderId(newAce.getFileId());
			directoryAclItem.setFlagInheritedAce(true);
			directoryAclItem.setFlagNoPropagateInherit(false);
			// when only file inheritance is applied then the ACE is added to directories as well, but with the flag INHERIT_ONLY
			directoryAclItem.setFlagInheritOnly(newAce.isFlagFileInherit() && !newAce.isFlagDirectoryInherit()); 
			directoryAclItem.setDirectory(true);
			for (int counter = 0; counter < subdirectoryIds.size(); counter++) {
				int fileId = subdirectoryIds.get(counter);
				directoryAclItem.setId(null); //reset to force an INSERT
				directoryAclItem.setFileId(fileId);
				directoryAclItem.save();
			}
		}
	}
}
