package com.perdemia.permissionanalyzer.application.handlers.toolbar;

import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchMembersDialog;
import com.perdemia.permissionanalyzer.application.parts.EffectivePermissionsPart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;

public class SeachMembersOnSelectedFileHandler {
	private static final Logger logger = LoggerFactory.getLogger(SeachMembersOnSelectedFileHandler.class.getName());
	
	@Execute
	public void execute(Shell shell, MPart membersMPart){
		if (membersMPart != null && membersMPart.getObject() != null) {
			EffectivePermissionsPart effectivePermissionsPart = (EffectivePermissionsPart) membersMPart.getObject();
			FileInfo selectedFile = effectivePermissionsPart.getSelectedFile();
			SearchMembersDialog searchDialog;
			
			if (FilterHelper.getInstance().getFilterConfig(false).isIncludeGroupMembership()) {
				// use a base query to search for member because of the 'include group membership' we might want to search for other members than the ones attached to the ACL
				try {
					searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), getMembersThatMatchFilterPS(0, selectedFile), FilterHelper.getInstance().getParamCountOfLastFilterQuery());
				} catch (DBException ex) {
					// DBException from the FilterHelper. Log and message is handled in the FilterHelper
					return;
				}
			} else {
				// just search through the list of members attached directly to the selected ACL
				List<Member> members = new ArrayList<Member>();
				for (AclItem aclItem : selectedFile.getAclItems()) {
					Member member = aclItem.getMember(true);
					
					// check if this member is not already in the list
					boolean alreadyInList = false;
					for (Member nextMember : members) {
						if (nextMember.getId().equals(member.getId())) {
							alreadyInList = true;
							break;
						}
					}
					
					if (!alreadyInList) {
						members.add(member);
					}
				}
				searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), members);
			}
			
			if (searchDialog.open() == IDialogConstants.OK_ID) {
				// not using
				//effectivePermissionsPart.selectMember(searchDialog.getSelectedMember());
			}
		}
	}
	
	/**
	 * Builds the query that is required to find members that match the selected filter criteria in the MemberFilterComposite.
	 * This method is only used when the option "Include nested group memberships" is enabled. Otherwise, we just use the direct members 
	 * from the ACE's found on the selected file.
	 * 
	 * The thing with group memberships is that we may find ACE's that don't belong to the users selected by the filter criteria! The dialog
	 * to search a member will only show members that match the filter criteria. So, instead of using the direct members from the 
	 * (filtered) ACE's on the selected file, we use the <b>groups</b> that belong to the members on the ACE's (or the members that match the
	 * filter criteria).
	 * 
	 * @return PreparedStatement with a SELECT member query
	 */
	private PreparedStatement getMembersThatMatchFilterPS(int limit, FileInfo selectedFile) throws DBException {
		StringBuilder additionalMemberFilterBuilder = new StringBuilder("FILTER_MEMBERSHIP.GROUP_ID IN (");
		Object[] additionalMemberFilterValues = new Object[selectedFile.getAclItems().size() + 4];
		Iterator<AclItem> iter = selectedFile.getAclItems().iterator();
		int index = 0;
		while (iter.hasNext()) {
			additionalMemberFilterBuilder.append("?");
			additionalMemberFilterValues[index] = iter.next().getMemberId();
			if (iter.hasNext()) {
				additionalMemberFilterBuilder.append(",");
			}
			index++;
		}
		
		// add a filter on the display_name and is_group, because this query is also used in the SearchMemberDialog
		// the is_group is added twice, when the user selects both Users and Groups to display then we 'disable' this filter by setting
		// member.is_group = true OR member.is_group = false 
		additionalMemberFilterBuilder.append(") AND (FILTER_MEMBER.NAME_LOWERCASE LIKE ? OR FILTER_MEMBER.DISPLAY_NAME_LOWERCASE LIKE ?) AND (FILTER_MEMBER.IS_GROUP = ? OR FILTER_MEMBER.IS_GROUP = ?)");
		additionalMemberFilterValues[additionalMemberFilterValues.length - 4] = "%"; // default value for the NAME_LOWERCASE compare
		additionalMemberFilterValues[additionalMemberFilterValues.length - 3] = "%"; // default value for the DISPLAY_NAME_LOWERCASE compare
		additionalMemberFilterValues[additionalMemberFilterValues.length - 2] = true; // don't filter on is_group, the column is present in the PreparedStatement because the SearchMemberDialog can filter on it
		additionalMemberFilterValues[additionalMemberFilterValues.length - 1] = false;
		
		String additionalMemberFilter = additionalMemberFilterBuilder.toString();
		PreparedStatement membersThatMatchFilterPS = FilterHelper.getInstance().buildFilterQuery(ObjectType.MEMBER, false, false, limit, "NAME_LOWERCASE", additionalMemberFilter, additionalMemberFilterValues);
		
		return membersThatMatchFilterPS;
	}
}
