package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.ReportComparator;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Report;

public class CompareReportsDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(CompareReportsDialog.class.getName());
	
	private Text report1PathText;
	private Text report2PathText;
	private Text outputPathText;
	private Text emailAddressText;
	
	private Button includeChangedItemsNewValueButton;
	private Button includeChangedItemsOldValueButton;
	private Button includeAddedItemsButton;
	private Button includeDeletedItemsButton;
	private Button sendEmailButton;
	
	private String report1Path;
	private String report2Path;
	private String outputPath;
	private String emailAddress;
	private String entity;
	private Report report;
	
	/**
	 * Constructor to initialize the dialog with two report paths. The entity contains the type of report "Report" or "Policy"
	 * 
	 * @param shell
	 * @param report1Path
	 * @param report2Path
	 * @param entity
	 */
	public CompareReportsDialog(Shell shell, String report1Path, String report2Path, String outputPath, String emailAddress, Report report) {
		super(shell);
		this.report1Path = report1Path;
		this.report2Path = report2Path;
		this.outputPath = outputPath;
		this.emailAddress = emailAddress;
		this.report = report;
		
		if (report != null) {
			this.entity = report.isPolicy() ? "Policy" : "Report";
		} else {
			this.entity = "Report";
		}		
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.COMPARE_REPORT_16X16));
		shell.setSize(800, 400);
		shell.setText("Compare the " + entity.toLowerCase());
	}
	
	@Override
	protected Control createDialogArea(final Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Compare the " + entity.toLowerCase() + " with a previous run", ImageFactory.getInstance().getImage(AppImage.COMPARE_REPORT_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		gridLayout.horizontalSpacing = 20;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.WRAP);
		label.setText("Enter two file paths to compare. The application will generate a new report file that only contains the differences between the two given report files on disk:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.heightHint = 40;
		label.setLayoutData(gridData);
		
		Label report1PathLabel = new Label(composite, SWT.NONE);
		report1PathLabel.setText("Path to " + entity.toLowerCase() + " file 1:");
		report1PathText = new Text(composite, SWT.BORDER);
		report1PathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		report1PathText.setText(report1Path);
				
		Label report2PathLabel = new Label(composite, SWT.NONE);
		report2PathLabel.setText("Path to " + entity.toLowerCase() + " file 2:");
		report2PathText = new Text(composite, SWT.BORDER);
		report2PathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		report2PathText.setText(report2Path);
		
		Label outputPathLabel = new Label(composite, SWT.NONE);
		outputPathLabel.setText("Output path: ");
		outputPathText = new Text(composite, SWT.BORDER);
		outputPathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		outputPathText.setText(outputPath);		
		
		Label includeChangedItemsOldValueLabel = new Label(composite, SWT.NONE);
		includeChangedItemsOldValueLabel.setText("Include changed items (old value):");
		includeChangedItemsOldValueButton = new Button(composite, SWT.CHECK);
		includeChangedItemsOldValueButton.setLayoutData(new GridData());
		includeChangedItemsOldValueButton.setSelection(true);
		
		Label includeChangedItemsNewValueLabel = new Label(composite, SWT.NONE);
		includeChangedItemsNewValueLabel.setText("Include changed items (new value):");
		includeChangedItemsNewValueButton = new Button(composite, SWT.CHECK);
		includeChangedItemsNewValueButton.setLayoutData(new GridData());
		includeChangedItemsNewValueButton.setSelection(true);
		
		Label includeAddedItemsLabel = new Label(composite, SWT.NONE);
		includeAddedItemsLabel.setText("Include added items:");
		includeAddedItemsButton = new Button(composite, SWT.CHECK);
		includeAddedItemsButton.setLayoutData(new GridData());
		includeAddedItemsButton.setSelection(true);
		
		Label includeDeletedItemsLabel = new Label(composite, SWT.NONE);
		includeDeletedItemsLabel.setText("Include deleted items:");
		includeDeletedItemsButton = new Button(composite, SWT.CHECK);
		includeDeletedItemsButton.setLayoutData(new GridData());
		includeDeletedItemsButton.setSelection(true);
		
		Label sendEmailLabel = new Label(composite, SWT.NONE);
		sendEmailLabel.setText("Send e-mail:");
		
		Composite sendEmailComposite = new Composite(composite, SWT.NONE);
		gridLayout = new GridLayout(2, false);
		gridLayout.marginLeft = -5;
		sendEmailComposite.setLayout(gridLayout);
		sendEmailComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		sendEmailButton= new Button(sendEmailComposite, SWT.CHECK);
		gridData = new GridData();
		sendEmailButton.setLayoutData(gridData);
		sendEmailButton.setSelection(emailAddress != null && !emailAddress.isEmpty());
				
		emailAddressText = new Text(sendEmailComposite, SWT.BORDER);
		gridData = new GridData();
		gridData.widthHint = 200;
		emailAddressText.setLayoutData(gridData);
		if (emailAddress != null) {
			emailAddressText.setText(emailAddress);
		}
		emailAddressText.setMessage("e-mail address");
				
		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, "Create", true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			// validate fields
			if (report1PathText.getText().length() == 0) {
				MessageDialog.openError(null, "Details not complete", "Please enter the path of the first report to compare.");
				return;
			}
			
			if (report2PathText.getText().length() == 0) {
				MessageDialog.openError(null, "Details not complete", "Please enter the path of the second report to compare.");
				return;
			}
						
			if (outputPathText.getText().length() == 0) {
				MessageDialog.openError(null, "Details not complete", "Please enter the output path for the comparison file.");
				return;
			}
			
			ReportComparator.startReportComparison(report1PathText.getText(), report2PathText.getText(), outputPathText.getText(), report, includeChangedItemsOldValueButton.getSelection(), includeChangedItemsNewValueButton.getSelection(), includeAddedItemsButton.getSelection(), includeDeletedItemsButton.getSelection(), sendEmailButton.getSelection() && !emailAddressText.getText().isEmpty() ? emailAddressText.getText() : null, null);
		}
		
		super.buttonPressed(buttonId);
	}
}
