package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.SmbFile;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.labelproviders.ColoredTableLabelProvider;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo.SourceType;
import com.perdemia.permissionanalyzer.application.parts.ScanFoldersPart;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.NetworkShareHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnectorImpl;

public class ImportSharesDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(ImportSharesDialog.class.getName());
		
	/** UI controls */
	private Text serversText;
	private CheckboxTableViewer networkSharesTableViewer;
	private Button includeAdminShares;
	private Button includeInheritedPermissionsButton;
	private Button scanLocalGroupsButton;
	private Button includeFilesButton;
	private Button followSymbolicLinksButton;
	private Spinner numberSubdirectoriesSpinner;
	
	/** Dialog settings and values */
	private int currentDirectoryListSize;
	private List<DirectoryInfo> directoryList = new ArrayList<DirectoryInfo>();
	
	public ImportSharesDialog(Shell parent, int currentDirectoryListSize) {
		super(parent);
		this.currentDirectoryListSize = currentDirectoryListSize;
	}
	
	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText("Import network shares");
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_16X16));
		shell.setSize(1200, 900);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Import network shares", ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.heightHint = 40;
			
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("The Trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label serversLabel = new Label(composite, SWT.NONE);
		serversLabel.setText("Servers:");
		serversText = new Text(composite, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
		serversText.setText("Enter the servers, one per line");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 6 * serversText.getLineHeight();
		serversText.setLayoutData(gridData);
		
		Button scanServers = new Button(composite, SWT.PUSH);
		scanServers.setText("Scan servers for network shares");
		scanServers.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
				try {
					progressDialog.run(true, true, new IRunnableWithProgress(){
						String[] servers = serversText.getText().split("\r\n");
						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							monitor.beginTask("Scanning servers for network shares....", servers.length);
							
							for (int counter = 0; counter < servers.length; counter++) {
								final String server = servers[counter];
								monitor.subTask(server);
								
								if (monitor.isCanceled()) {
									break;
								}
								
								final List<String> shares = NetworkShareHelper.getNetworkShares(server, null, false);
																	
								Display.getDefault().syncExec(new Runnable(){
									@Override
									public void run() {
										for (String share : shares) {
											DirectoryInfo dirInfo = new DirectoryInfo();
											dirInfo.setPath("\\\\" + server + "\\" + share);
											dirInfo.setEnabled(!share.endsWith("$") || includeAdminShares.getSelection());
											dirInfo.setFollowSymbolicLinks(followSymbolicLinksButton.getSelection());
											dirInfo.setIncludeFiles(includeFilesButton.getSelection());
											dirInfo.setIncludeInheritedPermissions(includeInheritedPermissionsButton.getSelection());
											dirInfo.setNumberOfSubdirectories(numberSubdirectoriesSpinner.getSelection());
											dirInfo.setScanLocalGroups(scanLocalGroupsButton.getSelection());
											dirInfo.setSourceType(SourceType.DIRECTORY);
											directoryList.add(dirInfo);
										}
	
										networkSharesTableViewer.refresh();
									}
								});
								
								monitor.worked(1);								
							}
						}
					});
					
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while scanning the servers.", ex);
					MessageDialog.openError(null, "Error", "Error while scanning the servers: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
				}
			}
		});
		
		Label sharesLabel = new Label(composite, SWT.NONE);
		sharesLabel.setText("Network shares:");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 15;
		sharesLabel.setLayoutData(gridData);
		
		networkSharesTableViewer = CheckboxTableViewer.newCheckList(composite, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = networkSharesTableViewer.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		table.setLayoutData(gridData);

		networkSharesTableViewer.setContentProvider(ArrayContentProvider.getInstance());
		
		TableViewerColumn pathColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		pathColumn.getColumn().setWidth(300);
		pathColumn.getColumn().setText("Path");
		
		TableViewerColumn subDirsColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		subDirsColumn.getColumn().setWidth(75);
		subDirsColumn.getColumn().setText("Depth");
		
		TableViewerColumn includeFilesColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		includeFilesColumn.getColumn().setWidth(120);
		includeFilesColumn.getColumn().setText("Include files");
		
		TableViewerColumn followSymbolicLinksColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		followSymbolicLinksColumn.getColumn().setWidth(170);
		followSymbolicLinksColumn.getColumn().setText("Follow symb. links");

		TableViewerColumn scanLocalGroupsColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		scanLocalGroupsColumn.getColumn().setWidth(170);
		scanLocalGroupsColumn.getColumn().setText("Scan local groups");
		
		TableViewerColumn includeInheritedPermissionsColumn = new TableViewerColumn(networkSharesTableViewer, SWT.NONE);
		includeInheritedPermissionsColumn.getColumn().setWidth(230);
		includeInheritedPermissionsColumn.getColumn().setText("Include Inher. permissions");
		
		networkSharesTableViewer.setLabelProvider(new ColoredTableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				DirectoryInfo info = (DirectoryInfo) element;
				switch (columnIndex) {
					case 0:
						return info.getPath();
					case 1:
						return String.valueOf(info.getNumberOfSubdirectories());
					case 2:
						return info.isIncludeFiles() ? "Yes" : "No";
					case 3:
						return info.isFollowSymbolicLinks() ? "Yes" : "No";
					case 4:
						return info.isScanLocalGroups() ? "Yes" : "No";
					case 5:
						return info.isIncludeInheritedPermissions() ? "Yes" : "No";
					default:
						return "";
				}
			}
			
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					return ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_16X16);
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (((DirectoryInfo) element).isEnabled()) {
					return null;
				} else {
					return ColorFactory.getInstance().getColor(AppColor.GREY);
				}
			}
		});
		
		networkSharesTableViewer.setCheckStateProvider(new ICheckStateProvider() {
			@Override
			public boolean isChecked(Object element) {
				return ((DirectoryInfo) element).isEnabled();
			}

			@Override
			public boolean isGrayed(Object element) {
				return ((DirectoryInfo) element).isEnabled();
			}
		});
		
		networkSharesTableViewer.setInput(directoryList);

		networkSharesTableViewer.addCheckStateListener(new ICheckStateListener() {
			@Override
			public void checkStateChanged(CheckStateChangedEvent event) {
				if (!ScanFoldersPart.checkForMaximumServers(networkSharesTableViewer)) {
					// uncheck again
					int index = directoryList.indexOf(event.getElement());
					networkSharesTableViewer.getTable().getItems()[index].setChecked(false);
				} else {
					((DirectoryInfo) event.getElement()).setEnabled(event.getChecked());
					networkSharesTableViewer.refresh();
				}
			}
		});
		
		Composite optionsComposite = new Composite(composite, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginTop = 15;
		layout.verticalSpacing = 8;
		optionsComposite.setLayout(layout);
		
		Label optionsLabel = new Label(optionsComposite, SWT.NONE);
		optionsLabel.setText("Options:");
		optionsLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		gridData = new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1);
		optionsLabel.setLayoutData(gridData);
		
		includeAdminShares = new Button(optionsComposite, SWT.CHECK);
		includeAdminShares.setText("Include admin shares");
		includeAdminShares.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1));
		includeAdminShares.setSelection(false);
		includeAdminShares.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (DirectoryInfo dirInfo : directoryList) {
					if (dirInfo.getPath().endsWith("$")) {
						dirInfo.setEnabled(includeAdminShares.getSelection());
					}
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		includeFilesButton = new Button(optionsComposite, SWT.CHECK);
		includeFilesButton.setText("Include files");
		includeFilesButton.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1));
		includeFilesButton.setSelection(true);
		includeFilesButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (DirectoryInfo dirInfo : directoryList) {
					dirInfo.setIncludeFiles(includeFilesButton.getSelection());
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		includeInheritedPermissionsButton = new Button(optionsComposite, SWT.CHECK);
		includeInheritedPermissionsButton.setText("Include inherited permissions");
		includeInheritedPermissionsButton.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1));
		includeInheritedPermissionsButton.setSelection(true);
		includeInheritedPermissionsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (DirectoryInfo dirInfo : directoryList) {
					dirInfo.setIncludeInheritedPermissions(includeInheritedPermissionsButton.getSelection());
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		followSymbolicLinksButton = new Button(optionsComposite, SWT.CHECK);
		followSymbolicLinksButton.setText("Follow symbolic links");
		followSymbolicLinksButton.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1));
		followSymbolicLinksButton.setSelection(true);
		followSymbolicLinksButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (DirectoryInfo dirInfo : directoryList) {
					dirInfo.setFollowSymbolicLinks(followSymbolicLinksButton.getSelection());
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		scanLocalGroupsButton = new Button(optionsComposite, SWT.CHECK);
		scanLocalGroupsButton.setText("Scan local groups on this server");
		scanLocalGroupsButton.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, true, true, 2, 1));
		scanLocalGroupsButton.setSelection(true);
		scanLocalGroupsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (DirectoryInfo dirInfo : directoryList) {
					dirInfo.setScanLocalGroups(scanLocalGroupsButton.getSelection());
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		Label numberSubdirectoriesLabel = new Label(optionsComposite, SWT.NONE);
		numberSubdirectoriesLabel.setText("Subdirectories depth:   ");
		
		numberSubdirectoriesSpinner = new Spinner(optionsComposite, SWT.BORDER);
		numberSubdirectoriesSpinner.setLayoutData(new GridData());
		numberSubdirectoriesSpinner.setMinimum(0);
		numberSubdirectoriesSpinner.setTextLimit(2);
		numberSubdirectoriesSpinner.setSelection(LicenseHelper.getEdition().equals(Edition.TRIAL) ? 5 : 0);
		numberSubdirectoriesSpinner.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
					if (numberSubdirectoriesSpinner.getSelection() == 0 || numberSubdirectoriesSpinner.getSelection() > AclScanner.getNumberOfSubDirsForTrial()) {
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Directory limit", "The trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
						numberSubdirectoriesSpinner.setSelection(AclScanner.getNumberOfSubDirsForTrial());
					}
				}
				
				for (DirectoryInfo dirInfo : directoryList) {
					dirInfo.setNumberOfSubdirectories(numberSubdirectoriesSpinner.getSelection());
				}
				networkSharesTableViewer.refresh();
			}
		});
		
		return composite;
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {		
		createButton(parent, IDialogConstants.OK_ID, "Add shares", false);
		createButton(parent, IDialogConstants.CANCEL_ID, "Close", false);
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
				int enabledCount = 0;
				for (DirectoryInfo dirInfo : directoryList) {
					if (dirInfo.isEnabled()) {
						enabledCount++;
					}
				}
				if (enabledCount + currentDirectoryListSize > 2) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Directory limit", "The trial version of Permission Analyzer is limited to 2 directories.");
					return;
				}
			}
		}
		
		super.buttonPressed(buttonId);
	}

	public List<DirectoryInfo> getDirectoryList() {
		return directoryList;
	}
	
	public static void main(String[] args) {
		try {
			NtlmPasswordAuthentication auth = new NtlmPasswordAuthentication("reinier-tp", "reinier", "Rlabelaber1"); 
			SmbFile file = new SmbFile("smb://" + "localhost", auth);
			String[] shares = file.list();
			System.out.println(Arrays.asList(shares));
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}