package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo.Authentication;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class LdapAuthenticationDetailsDialog extends DialogWithTopBar {
	private Authentication authentication;
	
	// UI Fields
	private Button QOPAuthAllowedButton;
	private Button QOPAuthIntAllowedButton;
	private Button QOPAuthConfAllowedButton;
	private Text realmText;
	private Text configPathText;
	private Button useTicketCacheButton;
	private Text KDCAddressText;
	
	// SASL / GSSAPI settings
	private boolean qualityOfProtectionAuthAllowed;
	private boolean qualityOfProtectionAuthIntAllowed;
	private boolean qualityOfProtectionAuthConfAllowed;
	private String realm;
	
	// GSSAPI settings
	private String configPath;
	private boolean useTicketCache;
	private String KDCAddress;
	
	private boolean dirty;
	
	public LdapAuthenticationDetailsDialog(Shell shell, LdapConnectionInfo connectionInfo, Authentication authentication) {
		super(shell);
		this.authentication = authentication;
		qualityOfProtectionAuthAllowed = connectionInfo.isQualityOfProtectionAuthAllowed();
		qualityOfProtectionAuthIntAllowed = connectionInfo.isQualityOfProtectionAuthIntAllowed();
		qualityOfProtectionAuthConfAllowed = connectionInfo.isQualityOfProtectionAuthConfAllowed();
		realm = connectionInfo.getRealm();
		configPath = connectionInfo.getConfigPath();
		useTicketCache = connectionInfo.isUseTicketCache();
		KDCAddress = connectionInfo.getKDCAddress();
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16));
		if (authentication.equals(Authentication.GSSAPI)) {
			shell.setSize(1000, 700);
		} else {
			shell.setSize(1000, 500);
		}
		shell.setText("Authentication details");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Authentication details", ImageFactory.getInstance().getImage(AppImage.CONNECTION_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.marginTop = 15;
		gridLayout.marginLeft = 15;
		gridLayout.marginRight = 15;
		composite.setLayout(gridLayout);
		
		Group qualityOfProtectionGroup = new Group(composite, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		qualityOfProtectionGroup.setLayoutData(gridData);
		qualityOfProtectionGroup.setText("Quality of Protection");
		GridLayout gl_qualityOfProtectionGroup = new GridLayout(1, false);
		gl_qualityOfProtectionGroup.marginHeight = 10;
		gl_qualityOfProtectionGroup.marginRight = 15;
		gl_qualityOfProtectionGroup.marginLeft = 10;
		gl_qualityOfProtectionGroup.marginBottom = 10;
		qualityOfProtectionGroup.setLayout(gl_qualityOfProtectionGroup);
		
		Label QOPDescription = new Label(qualityOfProtectionGroup, SWT.NONE | SWT.WRAP);
		GridData gd_QOPDescription = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_QOPDescription.heightHint = 44;
		gd_QOPDescription.widthHint = 900;
		QOPDescription.setLayoutData(gd_QOPDescription);
		QOPDescription.setText("List of allowed qualities of protection that may be used for communication that occurs on the connection after the authentication has completed. If checked the best protection (confidentiality protection) will be tried first.");
		
		QOPAuthAllowedButton = new Button(qualityOfProtectionGroup, SWT.CHECK | SWT.WRAP);
		GridData gd_QOPAuthAllowedButton = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_QOPAuthAllowedButton.widthHint = 900;
		QOPAuthAllowedButton.setLayoutData(gd_QOPAuthAllowedButton);
		QOPAuthAllowedButton.setText("  Only perform authentication, with no integrity or confidentiality protection for subsequent communication.");
		QOPAuthAllowedButton.setSelection(qualityOfProtectionAuthAllowed);
		QOPAuthAllowedButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				dirty = true;
			}
		});
		
		QOPAuthIntAllowedButton = new Button(qualityOfProtectionGroup, SWT.CHECK | SWT.WRAP);
		GridData gd_QOPAuthIntAllowedButton = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_QOPAuthIntAllowedButton.widthHint = 900;
		QOPAuthIntAllowedButton.setLayoutData(gd_QOPAuthIntAllowedButton);
		QOPAuthIntAllowedButton.setText("  Provide integrity protection for subsequent communication after authentication has completed. While integrity protection does not ensure that third-party observers cannot decipher communication between the client and server, it does ensure that the communication cannot be altered in an undetectable manner.");
		QOPAuthIntAllowedButton.setSelection(qualityOfProtectionAuthIntAllowed);
		QOPAuthIntAllowedButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				dirty = true;
			}
		});
		
		QOPAuthConfAllowedButton = new Button(qualityOfProtectionGroup, SWT.CHECK | SWT.WRAP);
		GridData gd_QOPAuthConfAllowedButton = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_QOPAuthConfAllowedButton.widthHint = 900;
		QOPAuthConfAllowedButton.setLayoutData(gd_QOPAuthConfAllowedButton);
		QOPAuthConfAllowedButton.setText("  Provide confidentiality protection for subsequent communication after authentication has completed. This ensures that third-party observers will not be able to decipher communication between the client and server.");
		QOPAuthConfAllowedButton.setSelection(qualityOfProtectionAuthConfAllowed);
		QOPAuthConfAllowedButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				dirty = true;
			}
		});
		
		if (authentication.equals(Authentication.GSSAPI)) {
			useTicketCacheButton = new Button(composite, SWT.CHECK);
			useTicketCacheButton.setText("  Try to use an existing Kerberos session. If none is available the supplied credentials will be used to setup a new session.");
			useTicketCacheButton.setSelection(useTicketCache);
			GridData gridData_1 = new GridData(GridData.FILL_HORIZONTAL);
			gridData_1.heightHint = 50;
			gridData_1.widthHint = 900;
			gridData_1.horizontalSpan = 3;
			useTicketCacheButton.setLayoutData(gridData_1);
			useTicketCacheButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					dirty = true;
				}
			});
		}
		
		Label realmLabel = new Label(composite, SWT.NONE | SWT.WRAP);
		GridData gd_realmLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_realmLabel.widthHint = 150;
		realmLabel.setLayoutData(gd_realmLabel);
		realmLabel.setText("Realm (domain): ");
		realmText = new Text(composite, SWT.BORDER);
		
		GridData gd_realmText = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_realmText.widthHint = 200;
		realmText.setLayoutData(gd_realmText);
		realmText.setText(notNull(realm));
		realmText.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				dirty = true;
			}
		});
		
		Label realmDescLabel = new Label(composite, SWT.NONE | SWT.WRAP);
		GridData gd_realmDescLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_realmDescLabel.horizontalIndent = 10;
		gd_realmDescLabel.widthHint = 550;
		realmDescLabel.setLayoutData(gd_realmDescLabel);
		realmDescLabel.setText("Optional realm into which the user should authenticate. If this is not provided, an attempt will be made to determine the appropriate value from the system configuration.");
		
		if (authentication.equals(Authentication.GSSAPI)) {
			Label configPathLabel = new Label(composite, SWT.NONE | SWT.WRAP);
			GridData gd_configPathLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_configPathLabel.widthHint = 150;
			configPathLabel.setLayoutData(gd_configPathLabel);
			configPathLabel.setText("Config file path:");
			configPathText = new Text(composite, SWT.BORDER);
			configPathText.addModifyListener(new ModifyListener(){
				@Override
				public void modifyText(ModifyEvent e) {
					dirty = true;
				}
			});
			
			GridData gd_configPathText = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_configPathText.widthHint = 200;
			configPathText.setLayoutData(gd_configPathText);
			configPathText.setText(notNull(configPath));
			Label configPathDescLabel = new Label(composite, SWT.NONE | SWT.WRAP);
			
			GridData gd_configPathDescLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_configPathDescLabel.horizontalIndent = 10;
			gd_configPathDescLabel.widthHint = 550;
			configPathDescLabel.setLayoutData(gd_configPathDescLabel);
			configPathDescLabel.setText("Optional path to a krb5.conf configuration file that should be used when processing the GSSAPI bind request. It may be empty if a configuration file should be automatically constructed for the bind request.");
			
			Label KDCAddressLabel = new Label(composite, SWT.NONE | SWT.WRAP);
			GridData gd_KDCAddressLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_KDCAddressLabel.widthHint = 150;
			KDCAddressLabel.setLayoutData(gd_KDCAddressLabel);
			KDCAddressLabel.setText("KDC Address:");
			KDCAddressText = new Text(composite, SWT.BORDER);
			
			GridData gd_KDCAddressText = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_KDCAddressText.widthHint = 200;
			KDCAddressText.setLayoutData(gd_KDCAddressText);
			KDCAddressText.setText(notNull(KDCAddress));
			KDCAddressText.addModifyListener(new ModifyListener(){
				@Override
				public void modifyText(ModifyEvent e) {
					dirty = true;
				}
			});
			
			Label KDCAddressDescLabel = new Label(composite, SWT.NONE | SWT.WRAP);
			GridData gd_KDCAddressDescLabel = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
			gd_KDCAddressDescLabel.horizontalIndent = 10;
			gd_KDCAddressDescLabel.widthHint = 550;
			KDCAddressDescLabel.setLayoutData(gd_KDCAddressDescLabel);
			KDCAddressDescLabel.setText("Optional IP address or resolvable name for the Kerberos key distribution center. If this is not provided, an attempt will be made to determine the appropriate value from the system configuration.");
		}
		
		dirty = false;
		
		return composite;
	}
		
	@Override
	protected void okPressed() {
		qualityOfProtectionAuthAllowed = QOPAuthAllowedButton.getSelection();
		qualityOfProtectionAuthIntAllowed = QOPAuthIntAllowedButton.getSelection();
		qualityOfProtectionAuthConfAllowed = QOPAuthConfAllowedButton.getSelection();
		realm = realmText.getText();
		configPath = (configPathText != null ? configPathText.getText() : null);
		useTicketCache = (useTicketCacheButton != null ? useTicketCacheButton.getSelection() : true);
		KDCAddress = (KDCAddressText != null ? KDCAddressText.getText() : null);
		
		super.okPressed();
	}
	
	private String notNull(String value) {
		return (value == null ? "" : value);
	}

	public boolean isQualityOfProtectionAuthAllowed() {
		return qualityOfProtectionAuthAllowed;
	}

	public boolean isQualityOfProtectionAuthIntAllowed() {
		return qualityOfProtectionAuthIntAllowed;
	}

	public boolean isQualityOfProtectionAuthConfAllowed() {
		return qualityOfProtectionAuthConfAllowed;
	}

	public String getRealm() {
		return realm;
	}

	public String getConfigPath() {
		return configPath;
	}

	public boolean isUseTicketCache() {
		return useTicketCache;
	}

	public String getKDCAddress() {
		return KDCAddress;
	}

	public boolean isDirty() {
		return dirty;
	}
}
