package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.mihalis.opal.titledSeparator.TitledSeparator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.license4j.ActivationStatus;
import com.license4j.License;
import com.license4j.LicenseValidator;
import com.license4j.ValidationStatus;
import com.license4j.util.FileUtils;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.PerdemiaLicense;

public class LicenseDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(LicenseDialog.class.getName());
	
	private final static int IMPORT_LICENSE_BUTTON = 100;
	private final static int ACTIVATE_BUTTON = 101;
	private final static int DEACTIVATE_BUTTON = 103;
	private Button activateButton;
	private Button deactivateButton;
	
	private Composite licenseDetails;
	private CLabel labelLicenseStatusValue;
	private CLabel labelActivationStatusValue;
	private Label activationIdValueLabel;
	private Label editionValueLabel;
	private Label quantityValueLabel;
	private Label expireValueLabel;
	private Label registrationNameValueLabel;
	private Label userValueLabel;
	private Label companyValueLabel;
	private Label emailValueLabel;
	
	public LicenseDialog(Shell shell) {
		super(shell);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.LICENSE_16X16));
		shell.setText("License information");
		shell.setSize(800, 570);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "License information", ImageFactory.getInstance().getImage(AppImage.LICENSE_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		container.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label labelLicenseStatus = new Label(composite, SWT.NONE);
		labelLicenseStatus.setText("License status:");
		GridData gridData = new GridData();
		gridData.widthHint = 190;
		gridData.horizontalIndent = 5;
		labelLicenseStatus.setLayoutData(gridData);
		labelLicenseStatusValue = new CLabel(composite, SWT.NONE);
		
		Label labelActivationStatus = new Label(composite, SWT.NONE);
		labelActivationStatus.setText("Activation status:");
		gridData = new GridData();
		gridData.widthHint = 190;
		gridData.horizontalIndent = 5;
		labelActivationStatus.setLayoutData(gridData);
		labelActivationStatusValue = new CLabel(composite, SWT.NONE);
		
		licenseDetails = new Composite(composite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		licenseDetails.setLayoutData(gridData);
		gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 8;
		licenseDetails.setLayout(gridLayout);
		
		Label activationIdLabel = new Label(licenseDetails, SWT.NONE);
		activationIdLabel.setText("Activation ID:");
		gridData = new GridData();
		gridData.widthHint = 195;
		activationIdLabel.setLayoutData(gridData);
		activationIdValueLabel = new Label(licenseDetails, SWT.NONE);
		
		Label editionLabel = new Label(licenseDetails, SWT.NONE);
		editionLabel.setText("Edition:");
		gridData = new GridData();
		gridData.widthHint = 195;
		editionLabel.setLayoutData(gridData);
		editionValueLabel = new Label(licenseDetails, SWT.NONE);
					 
		Label quantityLabel = new Label(licenseDetails, SWT.NONE);
		quantityLabel.setText("License quantity:");
		quantityValueLabel = new Label(licenseDetails, SWT.NONE);
					
		Label expireLabel = new Label(licenseDetails, SWT.NONE);
		expireLabel.setText("Expiration date:");
		expireValueLabel = new Label(licenseDetails, SWT.NONE);
		
		TitledSeparator ldapSeparator = new TitledSeparator(licenseDetails, SWT.NONE);
		ldapSeparator.setData(Constants.CSS_CLASS_NAME_KEY, "light");
		ldapSeparator.setText("License registration");
		ldapSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.REGISTRATION_24X24));
		ldapSeparator.setSize(50, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 15;
		ldapSeparator.setLayoutData(gridData);
					
		Group detailsGroup = new Group(licenseDetails, SWT.NONE);
		GridData gd_detailsGroup = new GridData(GridData.FILL_HORIZONTAL);
		gd_detailsGroup.horizontalSpan = 2;
		detailsGroup.setLayoutData(gd_detailsGroup);
		detailsGroup.setText("Details");
		gridLayout = new GridLayout(2, false);
		detailsGroup.setLayout(gridLayout);

		Label registrationNameLabel = new Label(detailsGroup, SWT.NONE);
		registrationNameLabel.setText("Registration name:");
		gridData = new GridData();
		gridData.widthHint = 190;
		registrationNameLabel.setLayoutData(gridData);
		registrationNameValueLabel = new Label(detailsGroup, SWT.NONE);
					
		Label userNameLabel = new Label(detailsGroup, SWT.NONE);
		userNameLabel.setText("Full name:");
		userValueLabel = new Label(detailsGroup, SWT.NONE);
					
		Label companyLabel = new Label(detailsGroup, SWT.NONE);
		companyLabel.setText("Company:");
		companyValueLabel = new Label(detailsGroup, SWT.NONE);
					
		Label emailLabel = new Label(detailsGroup, SWT.NONE);
		emailLabel.setText("E-mail:");
		emailValueLabel = new Label(detailsGroup, SWT.NONE);
			
		return composite;
	}
	
	private void loadLicenseInfo() {
		License license = LicenseHelper.getLicense();
		
		licenseDetails.setVisible(license != null);
		
		labelLicenseStatusValue.setText(LicenseHelper.getLicenseStatusString(license));
		if (license != null && license.getValidationStatus().equals(ValidationStatus.LICENSE_VALID)) {
			labelLicenseStatusValue.setImage(ImageFactory.getInstance().getImage(AppImage.VALID_16X16));
		} else {
			labelLicenseStatusValue.setImage(ImageFactory.getInstance().getImage(AppImage.STATUS_ERROR_16X16));
		}
		labelLicenseStatusValue.pack(true);
		
		labelActivationStatusValue.setText(LicenseHelper.getActivateStatusString(license));
		if (license != null && (license.isActivationCompleted() || !license.isActivationRequired())) {
			labelActivationStatusValue.setImage(ImageFactory.getInstance().getImage(AppImage.VALID_16X16));
		} else {
			labelActivationStatusValue.setImage(ImageFactory.getInstance().getImage(AppImage.STATUS_ERROR_16X16));
		}
		labelActivationStatusValue.pack(true);
		
		if (license != null) {
			activationIdValueLabel.setText(String.valueOf(license.getLicenseText().getActivationID()));
			activationIdValueLabel.pack(true);
			editionValueLabel.setText(LicenseHelper.getEditionString());
			editionValueLabel.pack(true);
			quantityValueLabel.setText(String.valueOf(license.getLicenseText().getLicenseQuantity()));
			quantityValueLabel.pack(true);
			Date expirationDate = license.getLicenseText().getLicenseExpireDate();
			SimpleDateFormat dateFormat = new SimpleDateFormat("dd MMMMM yyyy HH:mm");
			expireValueLabel.setText(expirationDate != null ? dateFormat.format(expirationDate) : "None");
			expireValueLabel.pack(true);
			registrationNameValueLabel.setText(license.getLicenseText().getUserRegisteredTo());
			registrationNameValueLabel.pack(true);
			userValueLabel.setText(license.getLicenseText().getUserFullName());
			userValueLabel.pack(true);
			companyValueLabel.setText(license.getLicenseText().getUserCompany() != null ? license.getLicenseText().getUserCompany() : "");
			companyValueLabel.pack(true);
			emailValueLabel.setText(license.getLicenseText().getUserEMail() != null ? license.getLicenseText().getUserEMail() : "");
			emailValueLabel.pack(true);
		}
		
		activateButton.setVisible(license != null && license.getValidationStatus().equals(ValidationStatus.LICENSE_VALID) && !license.isActivationCompleted() && license.isActivationRequired());
		deactivateButton.setVisible(license != null && license.getValidationStatus().equals(ValidationStatus.LICENSE_VALID) && license.isActivationCompleted());
	}
	
	/**
	 * Open the File Dialog and read the content of a new license file. It validates the license and if valid it writes the content to disk 
	 */
	private void importLicense(final boolean forActivation) {
		ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(getShell());
		
		FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
	    dialog.setFilterNames(new String[] { "License Files (*.lic | *.l4j | *.txt)" });
	    dialog.setFilterExtensions(new String[] { "*.lic;*.l4j;*.txt" }); 
	    final String filePath = dialog.open();
	    
	    if (filePath != null) {
	    	try {
				progressDialog.run(true, false, new IRunnableWithProgress(){

					@Override
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						monitor.beginTask("Importing license...", IProgressMonitor.UNKNOWN);
						
						// read license string from file.
			            String licenseTextString = ApplicationUtil.getAbsoluteFileContent(filePath);
				        				        
				        // First validate license and get a temporary license object to check for validation status later
			            License temporaryLicenseObject = LicenseHelper.getLicense(licenseTextString);

			            // If given license key is valid, then save it on disk, and update GUI fields
			            if (temporaryLicenseObject.getValidationStatus() == ValidationStatus.LICENSE_VALID &&
			            		(!forActivation || temporaryLicenseObject.isActivationCompleted() || !temporaryLicenseObject.isActivationRequired())) {
			                try {
			                    // We use Apache commons-io (FileUtils class) to easily save string to file
			                    ApplicationUtil.writeAbsoluteFileContent(LicenseHelper.LICENSE_PATH, licenseTextString);
			                } catch (IOException ex) {
			                	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while writing license file: " + ex.getMessage());
			    	            return;
			                }		                
			                			                			                			                
			                if (forActivation || temporaryLicenseObject.isActivationCompleted() || !temporaryLicenseObject.isActivationRequired()) {
			                	SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "License activated", "The license has been activated and is ready to use.");
			                } else {
//			                	if (SyncMessageDialog.showMessage(MessageDialog.CONFIRM, "License import", "The license has been imported and can now be activated. Do you want to activate the license?")) {
			                		LicenseHelper.refreshLicenseInfo(false);
			                		Display.getDefault().syncExec(new Runnable(){
			                			public void run() {
			                				activateLicense();
			                			}
			                		});			                		
//			                	} else {
//			                		// refresh GUI
//					                Display.getDefault().syncExec(new Runnable(){
//										@Override
//										public void run() {
//											loadLicenseInfo();
//										}			                	
//					                });
//			                	}			                	
			                }
			            } else {
			            	if (temporaryLicenseObject.getActivationStatus() == null) {
			            		SyncMessageDialog.showMessage(MessageDialog.ERROR, "Validation error", "The license is not valid:\r\n\r\n" + LicenseHelper.getLicenseStatusString(temporaryLicenseObject));
			            	} else {
			            		SyncMessageDialog.showMessage(MessageDialog.ERROR, "Validation error", "The license activation is not valid:\r\n\r\n" + 
			            				"License status: " + LicenseHelper.getLicenseStatusString(temporaryLicenseObject) + "\r\n\r\n" + 
			            				"Activation status: " + LicenseHelper.getActivateStatusString(temporaryLicenseObject));
			            	}
			            }
			            
			            LicenseHelper.refreshLicenseInfo(true);
						
						// refresh GUI
		                Display.getDefault().syncExec(new Runnable(){
							@Override
							public void run() {
								loadLicenseInfo();
							}			                	
		                });
					}
					
				});
			} catch (InvocationTargetException | InterruptedException e) {
				MessageDialog.openError(getShell(), "Error", "Error while importing the license: " + e.getMessage());
				logger.error("Error while importing the license", e);
			}
	    }
	}
	
	/**
	 * Asks the user to activate online or manually using a web form. In the first case it will show a progress bar and tries to activate the license online using {@link LicenseValidator#autoActivate(License)}.
	 * In the second case (manual activation) it will display a {@link TextboxDialog} with the activation request which can be used by the user in the web form. This web form will generate a download to a new (activated) license file.
	 */
	private void activateLicense() {
		ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(getShell());
				
		// Perdemia license?
		if (LicenseHelper.getLicense() instanceof PerdemiaLicense) {			
			try {
				LicenseHelper.activatePerdamiaLicenseFile();
            } catch (IOException ex) {
            	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while activating license file: " + ex.getMessage());
	            return;
            }
			
			SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "License activated", "The license has been activated and is ready to use.");
			LicenseHelper.refreshLicenseInfo(true);
			
			// refresh GUI
            Display.getDefault().syncExec(new Runnable(){
				@Override
				public void run() {
					loadLicenseInfo();
				}			                	
            });
			
		} else {
			// auto or manually?
			int result = SyncMessageDialog.showQuestionWithCancelMessage("License activation", "The license can be activated using an Internet connection or manually using a web form. \r\n\r\nDo you want to activate the license using your Internet connection? (click No to start the manual activation process)");
			
			if (result == IDialogConstants.YES_ID) {
				
				// activate using Internet
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){
	
						@Override
						public void run(IProgressMonitor monitor)
								throws InvocationTargetException, InterruptedException {
							monitor.beginTask("Activating license online...", IProgressMonitor.UNKNOWN);
							License license = LicenseValidator.autoActivate(LicenseHelper.getLicense());
							
							// display result
							if (license.getActivationStatus().equals(ActivationStatus.ACTIVATION_COMPLETED)) {
								SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "License activated", "The license has been activated and is ready to use.");
								
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Validation error", "The license activation is not valid:\r\n\r\n" + 
					    				"License status: " + LicenseHelper.getLicenseStatusString(license) + "\r\n\r\n" + 
					    				"Activation status: " + LicenseHelper.getActivateStatusString(license));
							}
							
							try {
			                    ApplicationUtil.writeAbsoluteFileContent(LicenseHelper.LICENSE_PATH, license.getLicenseString());
			                } catch (IOException ex) {
			                	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while writing license file: " + ex.getMessage());
			    	            return;
			                }
							
							LicenseHelper.refreshLicenseInfo(true);
							
							// refresh GUI
			                Display.getDefault().syncExec(new Runnable(){
								@Override
								public void run() {
									loadLicenseInfo();
								}			                	
			                });
						}				
					});
				} catch (InvocationTargetException | InterruptedException e) {
					MessageDialog.openError(getShell(), "Error", "Error while activating the license: " + e.getMessage());
					logger.error("Error while activating the license", e);
				}
				
			} else if (result == IDialogConstants.NO_ID) {
				// show activation token
				TextboxDialog textDialog = new TextboxDialog(getShell(), "Activation request", "Below you find your activation request. Copy this request and paste it in the activation web form on:\r\n<a>http://www.permissionanalyzer.com/activate-license.html</a>.\r\n\r\nThis web form will give you a new (activated) license file.", LicenseHelper.getLicense().getManualActivationRequestString(), "Next", null);
				textDialog.open();
				
				if (MessageDialog.openConfirm(getShell(), "Import activation file", "Import the activated license file that you received from the activation web form based on your activation request.")) {
					importLicense(true);
				}
			}
		}
	}
	
	/**
	 * Asks the user to deactivate online or manually using a web form. In the first case it will show a progress bar and tries to deactivate the license online using {@link LicenseValidator#autoDeactivate(License)}.
	 * In the second case (manual activation) it will display a {@link TextboxDialog} with the deactivation request which can be used by the user in the web form. This web form will generate a deactivation token.
	 */
	private void deactivateLicense() {
		ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(getShell());
		
		// Perdemia license?
		if (LicenseHelper.getLicense() instanceof PerdemiaLicense) {			
			try {
				LicenseHelper.deactivateLicenseFile();
            } catch (IOException ex) {
            	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while deactivating license file: " + ex.getMessage());
	            return;
            }
			
			SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "License deactivated", "The license has been deactivated.");
			LicenseHelper.refreshLicenseInfo(true);
			
			// refresh GUI
            Display.getDefault().syncExec(new Runnable(){
				@Override
				public void run() {
					loadLicenseInfo();
				}			                	
            });
			
		} else {
			// auto or manually?
			int result = SyncMessageDialog.showQuestionWithCancelMessage("License deactivation", "The license can be activated using an Internet connection or manually using a web form. \r\n\r\nDo you want to activate the license using your Internet connection?");
			
			if (result == IDialogConstants.YES_ID) {
				
				// activate using Internet
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){
	
						@Override
						public void run(IProgressMonitor monitor)
								throws InvocationTargetException, InterruptedException {
							monitor.beginTask("Deactivating license online...", IProgressMonitor.UNKNOWN);
							License license = LicenseValidator.autoDeactivate(LicenseHelper.getLicense());
							
							// display result
							if (license.getActivationStatus().equals(ActivationStatus.DEACTIVATION_COMPLETED)) {
								
								try {
									LicenseHelper.deactivateLicenseFile();
				                } catch (IOException ex) {
				                	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while deactivating license file: " + ex.getMessage());
				    	            return;
				                }
								
								SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "License deactivated", "The license has been deactivated.");
								
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Validation error", "The license deactivation could not be processed:\r\n\r\n" + 
					    				"License status: " + LicenseHelper.getLicenseStatusString(license) + "\r\n\r\n" + 
					    				"Activation status: " + LicenseHelper.getActivateStatusString(license));
							}
							
							LicenseHelper.refreshLicenseInfo(true);
							
							// refresh GUI
			                Display.getDefault().syncExec(new Runnable(){
								@Override
								public void run() {
									loadLicenseInfo();
								}			                	
			                });
						}				
					});
				} catch (InvocationTargetException | InterruptedException e) {
					MessageDialog.openError(getShell(), "Error", "Error while deactivating the license: " + e.getMessage());
					logger.error("Error while deactivating the license", e);
				}
				
			} else if (result == IDialogConstants.NO_ID) {
				// deactivate the file
				try {
					LicenseHelper.deactivateLicenseFile();
	            } catch (IOException ex) {
	            	SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while deactivating license file: " + ex.getMessage());
		            return;
	            }
				
				MessageDialog.openInformation(getShell(), "License deactivate", "Your license has been deactivated, you will now receive a dactivation token which can be used on our web form to finish the deactivation.");
				
				// show deactivation token
				TextboxDialog textDialog = new TextboxDialog(getShell(), "Deactivation request", "Below you find your deactivation request. Copy this request and paste it in the deactivation web form on:\r\n<a>http://www.permissionanalyzer.com/activate-license.html</a>.", LicenseHelper.getLicense().getManualDeactivationRequestString());
				textDialog.open();
				
				LicenseHelper.refreshLicenseInfo(true);
				
				// refresh GUI
				loadLicenseInfo();
			}
		}
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
				
		deactivateButton = createButton(parent, DEACTIVATE_BUTTON, "Deactivate license", false);
		((GridData) deactivateButton.getLayoutData()).widthHint = 150;
				
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		// license, but not yet activated?
		activateButton = createButton(parent, ACTIVATE_BUTTON, "Activate license", false);
		((GridData) activateButton.getLayoutData()).widthHint = 150;
		
		Button button = createButton(parent, IMPORT_LICENSE_BUTTON, "Import License", false);
		((GridData) button.getLayoutData()).widthHint = 150;
		
		createButton(parent, IDialogConstants.CLOSE_ID, "Close", true);
		
		// now the buttons have been created initialize the license info fields
		loadLicenseInfo();
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IMPORT_LICENSE_BUTTON) {
			importLicense(false);
		}		
		
		if (buttonId == IDialogConstants.CLOSE_ID) {
			close();
		}
		
		if (buttonId == ACTIVATE_BUTTON) {
			activateLicense();
		}
		
		if (buttonId == DEACTIVATE_BUTTON) {
			deactivateLicense();
		}
	}
}
