package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FontFactory;
import com.perdemia.permissionanalyzer.application.util.FontFactory.AppFont;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.MembershipCache;

public class MemberInfoDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(MemberInfoDialog.class.getName());
		
	/** UI controls */
	private TabFolder tabFolder;
	private TableViewer memberOfTableViewer;
	private TableViewer membersTableViewer;
	private Button includeNestedGroupMembershipForMemberOf;
	private Button includeNestedGroupMembershipForMembers;
	private Text searchMemberOfText;
	private Text searchMembersText;
	private Button includeUsers;
	private Button includeGroups;
	
	/** Dialog settings and values */
	private Member member;
	
	/** Option used by the context menu item "Show matching members" for wilcard search on account name in the {@link MemberFilterComposite} */
	private String baseQuery;
	private Object baseQueryParameter;
	private boolean usedForFilterResults;
	private String filterName;
	private String filterValue;
	private String orderBy;	
	
	/** Option used by the context menu item "Show matching members" for members found in a report, in the {@link MemberFilterComposite} */
	List<Member> memberList;
	
	public MemberInfoDialog(Shell shell, Member member) {
		super(shell);
		this.member = member;
	}
	
	public MemberInfoDialog(Shell shell, Integer memberId) {
		super(shell);
		try {
			this.member = Member.getById(memberId);
		} catch (DBException e) {
			logger.error("Error while loading member details", e);
			MessageDialog.openError(null, "Error", "Error while loading member details: " + GUILogHandler.getNestedMessage(e));
		}
	}
	
	/**
	 * Constructor used in the {@link MemberFilterComposite} when the context menu item "Show matching members" is selected for a wild card filter
	 * 
	 * @param shell
	 * @param baseQuery query used to select the members. The filter on name and/or is_group are added to this query.
	 */
	public MemberInfoDialog(Shell shell, String baseQuery, Object baseQueryParameter, String filterName, String filterValue, String orderBy) {
		super(shell);
		usedForFilterResults = true;
		this.baseQuery = baseQuery;
		this.baseQueryParameter = baseQueryParameter;
		this.orderBy = orderBy;
		this.filterName = filterName;
		this.filterValue = filterValue;
	}
	
	/**
	 * Constructor used in the {@link MemberFilterComposite} when the context menu item "Show matching members" is selected for a report filter
	 * 
	 * @param shell
	 * @param memberList the list of members to display
	 */
	public MemberInfoDialog(Shell shell, List<Member> memberList, String filterName, String filterValue) {
		super(shell);
		usedForFilterResults = true;
		this.memberList = memberList;
		this.filterName = filterName;
		this.filterValue = filterValue;
	}

	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText("Member info");
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.USER_INFO_16X16));
		shell.setSize(850, 700);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		ApplicationUtil.setBusyCursor();
		Composite composite = (Composite) super.createDialogArea(container, "Member info", ImageFactory.getInstance().getImage(AppImage.USER_INFO_48X48));
		//container.getShell().setImage(ImageFactory.getInstance().getImage(AppImage.USER_INFO_16X16));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		//container.getShell().setText("Member info");
		//GridData dialogLayoutData = (GridData) container.getLayoutData();
		//dialogLayoutData.widthHint = 850;
		//dialogLayoutData.heightHint = 700;
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		/*
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.heightHint = 40;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("The Trial version of Permission Analyzer has a limit of 50 users and 25 groups.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}*/
		
		//Member info fields
		Composite memberInfoComposite = new Composite(composite, SWT.NONE);
		memberInfoComposite.setLayout(new GridLayout(2, false));
		memberInfoComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		if (!usedForFilterResults) {
			// account name
			Label accountNameLabel = new Label(memberInfoComposite, SWT.NONE);
			accountNameLabel.setText("Account name: ");
			accountNameLabel.setLayoutData(new GridData());
			Label accountNameValueLabel = new Label(memberInfoComposite, SWT.NONE);
			accountNameValueLabel.setText(member.getName(false));
			accountNameValueLabel.setLayoutData(new GridData());
			
			// display name
			Label displayNameLabel = new Label(memberInfoComposite, SWT.NONE);
			displayNameLabel.setText("Display name: ");
			displayNameLabel.setLayoutData(new GridData());
			Label displayNameValueLabel = new Label(memberInfoComposite, SWT.NONE);
			displayNameValueLabel.setText(member.getDisplayName(false, true));
			displayNameValueLabel.setLayoutData(new GridData());
			
			// Domain
			Label domainLabel = new Label(memberInfoComposite, SWT.NONE);
			domainLabel.setText("Domain: ");
			domainLabel.setLayoutData(new GridData());
			Label domainValueLabel = new Label(memberInfoComposite, SWT.NONE);
			domainValueLabel.setText(member.getDomain() == null ? "None" : member.getDomain().getDomain());
			domainValueLabel.setLayoutData(new GridData());
					
			// DN
			Label dnLabel = new Label(memberInfoComposite, SWT.NONE);
			dnLabel.setText("LDAP DN: ");
			dnLabel.setLayoutData(new GridData());
			Label dnValueLabel = new Label(memberInfoComposite, SWT.NONE);
			dnValueLabel.setText((member.getCn() != null && member.getRdn() != null) ? "CN=" + member.getCn() + "," + member.getRdn().getRdn() : "N/A");
			dnValueLabel.setLayoutData(new GridData());
			
			// Built-in status
			Label builtinLabel = new Label(memberInfoComposite, SWT.NONE);
			builtinLabel.setText("Built-in account: ");
			builtinLabel.setLayoutData(new GridData());
			Label builtinValueLabel = new Label(memberInfoComposite, SWT.NONE);
			builtinValueLabel.setText(member.isBuiltIn() ? "Yes" : "No");
			builtinValueLabel.setLayoutData(new GridData());
						
			// Enabled status
			Label enabledLabel = new Label(memberInfoComposite, SWT.NONE);
			enabledLabel.setText("Enabled: ");
			enabledLabel.setLayoutData(new GridData());
			Label enabledValueLabel = new Label(memberInfoComposite, SWT.NONE);
			enabledValueLabel.setText(member.isEnabled() ? "Yes" : "No");
			enabledValueLabel.setLayoutData(new GridData());
			
		} else {
			// Just display the filter name and value
			Label filterNameLabel = new Label(memberInfoComposite, SWT.NONE);
			filterNameLabel.setText(filterName + ": ");
			filterNameLabel.setLayoutData(new GridData());
			filterNameLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_TITLE));
			
			Label fitlerValueLabel = new Label(memberInfoComposite, SWT.NONE);
			fitlerValueLabel.setText(filterValue);
			fitlerValueLabel.setLayoutData(new GridData());
			fitlerValueLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_TITLE));
		}
		
		// tabs
		tabFolder = new TabFolder (composite, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		tabFolder.setLayoutData(gridData);
		
		//======================================
		//Tab with memberOf items
		//======================================
		if (!usedForFilterResults) {
			TabItem membersOfTab = new TabItem (tabFolder, SWT.NONE);
			membersOfTab.setText ("Member of");
			Composite membersOfComposite = new Composite(tabFolder, SWT.NONE);
			membersOfComposite.setLayout(new GridLayout(2, false));
			membersOfComposite.setLayoutData(gridData);
			
			// search controls
			Label searchLabel = new Label(membersOfComposite, SWT.NONE);
			searchLabel.setText("Search groups (use * as wildcard):");
			gridData = new GridData();
			gridData.verticalAlignment = SWT.BOTTOM;
			gridData.horizontalAlignment = SWT.LEFT;
			gridData.horizontalSpan = 2;
			searchLabel.setLayoutData(gridData);
	
			// Search textbox
			searchMemberOfText = DelayedKeyListenerText.create(membersOfComposite, SWT.BORDER, new KeyAdapter() {
				@Override
				public void keyReleased(KeyEvent e) {
					// because of the delayed keypress search, it might happen that the dialog has been closed!
					if (!searchMemberOfText.isDisposed()) {
					if (e.keyCode == SWT.ARROW_DOWN) {
						memberOfTableViewer.getTable().setFocus();
					} else {
						searchMemberOf(searchMemberOfText.getText(), includeNestedGroupMembershipForMemberOf.getSelection(), e.keyCode == 13);
					}					
				}			
				}			
			});
			
			searchMemberOfText.setMessage("search");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			searchMemberOfText.setLayoutData(gridData);
			
			// Search button
			Button searchMembersButton = new Button(membersOfComposite, SWT.PUSH);
			searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_20X20));
			searchMembersButton.setToolTipText("Search LDAP");
			searchMembersButton.setLayoutData(new GridData());
			
			// Nested group membership checkbox
			includeNestedGroupMembershipForMemberOf = new Button(membersOfComposite, SWT.CHECK);
			includeNestedGroupMembershipForMemberOf.setText("Include nested group membership");
			includeNestedGroupMembershipForMemberOf.setSelection(true);
			gridData = new GridData();
			gridData.horizontalSpan = 2;
			includeNestedGroupMembershipForMemberOf.setLayoutData(gridData);
						
			// add the listeners
			searchMemberOfText.addTraverseListener(new TraverseListener() {
				@Override
				public void keyTraversed(TraverseEvent e) {
					if (e.detail == SWT.TRAVERSE_RETURN) {
						searchMemberOf(searchMemberOfText.getText(), includeNestedGroupMembershipForMemberOf.getSelection(), true);
					}
				}
			});
			
			searchMembersButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMemberOf(searchMemberOfText.getText(), includeNestedGroupMembershipForMemberOf.getSelection(), true);
				}
			});
			includeNestedGroupMembershipForMemberOf.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMemberOf(searchMemberOfText.getText(), includeNestedGroupMembershipForMemberOf.getSelection(), true);
				}
			});
	
			
			// Search result list	
			memberOfTableViewer = new TableViewer(membersOfComposite, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION);
			memberOfTableViewer.getTable().setHeaderVisible(true);
			memberOfTableViewer.getTable().setLinesVisible(true);
			gridData = new GridData(GridData.FILL_BOTH);
			gridData.horizontalSpan = 2;
			memberOfTableViewer.getTable().setLayoutData(gridData);
	
			// Group name column
			TableViewerColumn column = new TableViewerColumn(memberOfTableViewer, SWT.NONE);
			column.getColumn().setText("Group");
			column.getColumn().setWidth(300);
			column.getColumn().setResizable(true);
			
			// Display name column (just a placeholder to use the same LabelProvider, group has no display name)
			column = new TableViewerColumn(memberOfTableViewer, SWT.NONE);
			column.getColumn().setText("Description");
			column.getColumn().setWidth(200);
			column.getColumn().setResizable(true);
						
			// Via group column
			column = new TableViewerColumn(memberOfTableViewer, SWT.NONE);
			column.getColumn().setText("Via group");
			column.getColumn().setWidth(200);
			column.getColumn().setResizable(true);
					
			memberOfTableViewer.setContentProvider(ArrayContentProvider.getInstance());
			memberOfTableViewer.setLabelProvider(new MemberLabelProvider());
			
			membersOfTab.setControl(membersOfComposite);
			
			// Result list buttons
			Composite searchResultButtons = new Composite(composite, SWT.NONE);
			searchResultButtons.setLayout(new GridLayout(1, true));
		}
		
		//=====================================================
		//Tab with members items (only applicable for groups)
		//=====================================================
		if (usedForFilterResults || member.isGroup()) {
			TabItem membersTab = new TabItem (tabFolder, SWT.NONE);
			membersTab.setText ("Members");
			Composite membersComposite = new Composite(tabFolder, SWT.NONE);
			membersComposite.setLayout(new GridLayout(2, false));
			membersComposite.setLayoutData(gridData);
			
			// search controls
			Label searchLabel = new Label(membersComposite, SWT.NONE);
			searchLabel.setText("Search members (use * as wildcard):");
			gridData = new GridData();
			gridData.verticalAlignment = SWT.BOTTOM;
			gridData.horizontalAlignment = SWT.LEFT;
			gridData.horizontalSpan = 2;
			searchLabel.setLayoutData(gridData);
	
			// Search textbox
			searchMembersText = DelayedKeyListenerText.create(membersComposite, SWT.BORDER, new KeyAdapter() {
				@Override
				public void keyReleased(KeyEvent e) {
					// because of the delayed keypress search, it might happen that the dialog has been closed!
					if (!searchMembersText.isDisposed()) {
					if (e.keyCode == SWT.ARROW_DOWN) {
						membersTableViewer.getTable().setFocus();
					} else {
						searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), e.keyCode == 13);
					}					
				}			
				}			
			});
			searchMembersText.setMessage("search");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			searchMembersText.setLayoutData(gridData);
			
			// Search button
			Button searchMembersButton = new Button(membersComposite, SWT.PUSH);
			searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_20X20));
			searchMembersButton.setToolTipText("Search LDAP");
			searchMembersButton.setLayoutData(new GridData());
			
			// Checkboxes
			Composite checkboxesComposite = new Composite(membersComposite, SWT.NONE);
			checkboxesComposite.setLayout(new GridLayout(3, false));
			gridData = new GridData();
			gridData.horizontalSpan = 2;
			checkboxesComposite.setLayoutData(gridData);
			
			// Include users checkbox
			includeUsers = new Button(checkboxesComposite, SWT.CHECK);
			includeUsers.setText("Show users");
			includeUsers.setLayoutData(new GridData());
			includeUsers.setSelection(true);
						
			// Include groups checkbox
			includeGroups = new Button(checkboxesComposite, SWT.CHECK);
			includeGroups.setText("Show groups");
			includeGroups.setLayoutData(new GridData());
			includeGroups.setSelection(true);
			
			// Nested group membership checkbox
			includeNestedGroupMembershipForMembers = new Button(checkboxesComposite, SWT.CHECK);
			includeNestedGroupMembershipForMembers.setText("Include nested group membership");
			includeNestedGroupMembershipForMembers.setLayoutData(new GridData());
			if (usedForFilterResults) {
				includeNestedGroupMembershipForMembers.setVisible(false);
			}
			
			// add the listeners
			searchMembersText.addTraverseListener(new TraverseListener() {
				@Override
				public void keyTraversed(TraverseEvent e) {
					if (e.detail == SWT.TRAVERSE_RETURN) {
						searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), true);
					}
				}
			});
						
			searchMembersButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), true);
				}
			});
			
			includeUsers.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), true);
				}
			});
			
			includeGroups.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), true);
				}
			});
			
			includeNestedGroupMembershipForMembers.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchMembers(searchMembersText.getText(), includeUsers.getSelection(), includeGroups.getSelection(), includeNestedGroupMembershipForMembers.getSelection(), true);
				}
			});
			
			// Search result list	
			membersTableViewer = new TableViewer(membersComposite, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION);
			membersTableViewer.getTable().setHeaderVisible(true);
			membersTableViewer.getTable().setLinesVisible(true);
			gridData = new GridData(GridData.FILL_BOTH);
			gridData.horizontalSpan = 2;
			membersTableViewer.getTable().setLayoutData(gridData);
	
			// Member name column
			TableViewerColumn column = new TableViewerColumn(membersTableViewer, SWT.NONE);
			column.getColumn().setText("Account name");
			column.getColumn().setWidth(200);
			column.getColumn().setResizable(true);
			
			// Display name column
			column = new TableViewerColumn(membersTableViewer, SWT.NONE);
			column.getColumn().setText("Display name");
			column.getColumn().setWidth(300);
			column.getColumn().setResizable(true);
			
			// Via group column
			column = new TableViewerColumn(membersTableViewer, SWT.NONE);
			column.getColumn().setText("Via group");
			column.getColumn().setWidth(300);
			column.getColumn().setResizable(true);
			
			membersTableViewer.setContentProvider(ArrayContentProvider.getInstance());
			membersTableViewer.setLabelProvider(new MemberLabelProvider());
			
			membersTab.setControl(membersComposite);
			
			// Result list buttons
			Composite searchResultButtons = new Composite(composite, SWT.NONE);
			searchResultButtons.setLayout(new GridLayout(1, true));
		}

		//initialize the lists
		try {
			ApplicationUtil.setBusyCursor();
			if (usedForFilterResults || member.isGroup()) {
				if (baseQuery != null) {
					List<Member> members = Member.findByName(null, true, true, -1, orderBy, 0, baseQuery, baseQueryParameter);
					membersTableViewer.setInput(members);
				} else if (memberList != null) {
					membersTableViewer.setInput(memberList);
				} else {
					membersTableViewer.setInput(MembershipCache.getGroupMembers(member, true, true, false));
				}
			}
			
			if (!usedForFilterResults) {
				// don't use ORDER BY member.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
				memberOfTableViewer.setInput(Member.getMemberOfList(member, null, true, 0, "3", 0));
			}
			
		} catch (DBException ex) {
			MessageDialog.openError(null, "Error", "Database error while loading the list of members: " + GUILogHandler.getNestedMessage(ex));
		} finally {
			ApplicationUtil.setDefaultCursor();
		}
		
		ApplicationUtil.setDefaultCursor();
		return composite;
	}
	
	/**
	 * Refresh the table with members 
	 */
	private void searchMembers(String searchText, boolean includeUsers, boolean includeGroups, boolean includeNestedGroupMembership, boolean explicitUserSearchAction) {
		if (!includeUsers && !includeGroups) {
			if (explicitUserSearchAction) {
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "Invalid selection", "Select at least one of the checkboxes to show users and/or groups.");
			}
			return;
		}
		
		searchText = searchText.replace('*', '%');
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with %
		if (explicitUserSearchAction ||	(searchText.length() > 2 && !searchText.startsWith("%")) || baseQuery != null || memberList != null) {
			try {
				ApplicationUtil.setBusyCursor(); 
				
				if (!searchText.endsWith("%")) {
					searchText += "%";
				}
				
				// determine the right 'source' to filter on
				if (baseQuery != null) {
					List<Member> members = Member.findByName(searchText, includeUsers, includeGroups, -1, orderBy, 0, baseQuery, baseQueryParameter);
					membersTableViewer.setInput(members);
					
				} else if (memberList != null) {
					searchText = searchText.replace("%", "").toLowerCase();
					
					List<Member> filteredMembers = new ArrayList<Member>();
					for (Member member : memberList) {
						if (member.getDisplayName(true, true).toLowerCase().contains(searchText) && ((includeUsers && includeGroups) || member.isGroup() == includeGroups)) {
							filteredMembers.add(member);
						}
					}
					
					membersTableViewer.setInput(filteredMembers);
					
				} else {
					if (searchText.equals("%")) {
						membersTableViewer.setInput(MembershipCache.getGroupMembers(member, includeUsers, includeGroups, includeNestedGroupMembership));
					} else {
						// don't use ORDER BY member.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
						membersTableViewer.setInput(Member.getMembers(member, searchText, includeUsers, includeGroups, includeNestedGroupMembership, 0, "3", 0));
					}
				}
				
				membersTableViewer.refresh();
			} catch (Exception e) {
				MessageDialog.openError(null, "Error", "Error while searching members.\r\n\r\n" + GUILogHandler.getNestedMessage(e));
				logger.error("Error while searching members", e);
			} finally {
				ApplicationUtil.setDefaultCursor();
			}
		}
	}
	
	/**
	 * Refresh the table with groups 
	 */
	private void searchMemberOf(String searchText, boolean includeNestedGroupMembership, boolean explicitUserSearchAction) {
		searchText = searchText.replace('*', '%');
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with %
		if (explicitUserSearchAction ||	(searchText.length() > 2 && !searchText.startsWith("%"))) {
			try {
				ApplicationUtil.setBusyCursor(); 
				
				if (!searchText.endsWith("%")) {
					searchText += "%";
				}
				
				// don't use ORDER BY member.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
				memberOfTableViewer.setInput(Member.getMemberOfList(member, searchText, includeNestedGroupMembership, 0, "3", 0));
				memberOfTableViewer.refresh();
			} catch (Exception e) {
				MessageDialog.openError(null, "Error", "Error while searching groups.\r\n\r\n" + GUILogHandler.getNestedMessage(e));
				logger.error("Error while searching groups", e);
			} finally {
				ApplicationUtil.setDefaultCursor();
			}
		}
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {		
		createButton(parent, IDialogConstants.SELECT_ALL_ID, "Copy to clipboard", false);
		createButton(parent, IDialogConstants.CANCEL_ID, "Close", false);
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.SELECT_ALL_ID) {
			StringBuilder membersText = new StringBuilder("Account name,Display name,Via group\r\n");
			
			TableItem[] items;
			if (tabFolder.getSelectionIndex() == 0) {
				items = memberOfTableViewer.getTable().getItems();
			} else {
				items = membersTableViewer.getTable().getItems();
			}
			
			for (TableItem tableItem : items) {
				membersText.append(tableItem.getText(0));
				membersText.append(",");
				membersText.append(tableItem.getText(1));
				membersText.append(",");
				membersText.append(tableItem.getText(2));
				membersText.append("\r\n");
			}
			Clipboard clipboard = new Clipboard(Display.getDefault());
			TextTransfer textTransfer = TextTransfer.getInstance();
	        clipboard.setContents(new String[]{membersText.toString()}, new Transfer[]{textTransfer});
	        clipboard.dispose();
	        SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "Copied " + items.length + " members to the clipboard.");
		} else {
			close();
		}
	}
}

class MemberLabelProvider extends TableLabelProvider {
	
	@Override
	public Image getColumnImage(Object element, int columnIndex) {
		if (columnIndex == 0) {
			if (((Member) element).isGroup()) {
				return ImageFactory.getInstance().getImage(AppImage.GROUP_20X20);
			} else {
				return ImageFactory.getInstance().getImage(AppImage.USER_20X20);
			}
		} else {
			return null;
		}
	}

	@Override
	public String getColumnText(Object element, int columnIndex) {
		Member member = (Member) element;
		switch (columnIndex) {
			case 0: return member.getName(true);
			case 1: return member.getDisplayName(false, true);
			case 2: return member.getViaGroupName();
		}
		return null;
	}	
}