package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class NetworkSharePasswordDialog extends DialogWithTopBar {
	private Text domainText;
	private Text usernameText;
	private Text passwordText;
	
	private String domain;
	private String user;
	private String password;
		
	public NetworkSharePasswordDialog(Shell shell, String defaultDomain, String defaultUsername) {
		super(shell);
		domain = defaultDomain;
		user = defaultUsername;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_16X16));
		shell.setSize(600, 500);
		shell.setText("Network Share Login");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Network Share Login", ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.WRAP);
		label.setText("Please enter your credentials to retrieve the network shares for this server. These credentials are stored encrypted in the application settings and only used to retrieve all available shares for a server. The actual ACL's in each share info are scanned using the current logged in Windows user!");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 150;
		gridData.horizontalSpan = 2;
		label.setLayoutData(gridData);
		
		Label domainLabel = new Label(composite, SWT.NONE);
		domainLabel.setText("Domain: ");
		gridData = new GridData();
		domainLabel.setLayoutData(gridData);
		domainText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		domainText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		domainText.setText(domain);
		
		Label usernameLabel = new Label(composite, SWT.NONE);
		usernameLabel.setText("Username: ");
		usernameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		usernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		usernameText.setText(user);

		Label passwordLabel = new Label(composite, SWT.NONE);
		passwordLabel.setText("Password: ");
		passwordText = new Text(composite, SWT.SINGLE | SWT.BORDER | SWT.PASSWORD);
		passwordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
		return composite;
	}
	
	protected void buttonPressed(int buttonId) {
		domain = domainText.getText();
		password = passwordText.getText();
		user = usernameText.getText();
			
		super.buttonPressed(buttonId);
	}

	public String getPassword() {
		return password;
	}

	public String getDomain() {
		return domain;
	}
	
	public String getUser() {
		return user;
	}
}
