package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import org.apache.commons.mail.EmailException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.mihalis.opal.titledSeparator.TitledSeparator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo.Authentication;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo.Encryption;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.EmailHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.util.DBUtil;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnector;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnectorImpl;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapFactory;

public class PreferenceDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(PreferenceDialog.class.getName());
	
	/** LDAP UI fields */
	private Text connectionNameText;
	private Combo hostNameCombo;
	private Combo hostPortCombo;
	private Text usernameText;
	private Text passwordText;
	private Combo encryptionCombo;
	private Combo authenticationCombo;
	private TableViewer tableViewer;
	private Button updateButton;
	private Button authenticationDetailsButton;
	private LdapAuthenticationDetailsDialog authDetailsDialog;
	private boolean isConnectionChanged;
	
	/** Internet tab UI controls */
	private Text proxyHostText;
	private Text proxyPortText;
	private Text proxyUsernameText;
	private Text proxyPasswordText;
	private Button useSystemProxyButton;
	private Button useNoProxyButton;
	//private Button checkForUpdatesButton;
	
	/** E-mail tab UI controls */
	private Text smtpHostText;
	private Combo smtpPortCombo;
	private Text smtpUsernameText;
	private Text smtpPasswordText;
	private Text smtpFromText;
	private Combo smtpTypeCombo;
	private Button smtpAttachReportsButton;
	private Text reportEmailTemplateText;
	private Text policyEmailTemplateText;
	private Text reportEmailSubjectText;
	private Text policyEmailSubjectText;
	private Text csvSeparatorText;
	private Text maxFileExportMegaBytesText;
	
	/** Security tab UI controls */
	private Text applicationPasswordText;
	private Button encryptDatabaseButton;
	
	/** Database tab UI controls */
	private Combo dbConnectionTypeCombo;
	private Text dbConnectionStringText;
	private Text dbConnectionDriverText;
	private Text dbConnectionUsernameText;
	private Text dbConnectionPasswordText;
	private Text batchSizeFilesText;
	private Text batchSizeAcesText;
	private Text batchSizeMembersText;
	private Text batchSizeMembershipsText;	
	
	/** Advanced tab UI controls */
	private Text ldapAccountNameAttributeText;
	private Text ldapUserDisplayNameAttributeText;
	private Text ldapGroupDisplayNameAttributeText;
	private Text ldapGroupTypeAttributeText;
	private Text ldapDNAttributeText;
	private Text ldapPrimaryGroupIdAttributeText;
	private Text ldapGroupMemberAttributeText;
	private Text ldapMemberOfAttributeText;
	private Button debugLdapButton;
	private Button followReferralsButton;
	private Text logLevelsText;
	private Button calculateOwnershipRatioInOverviewButton;
	private Button calculateOwnershipRatioInEffectivePermissionsTabButton;
	private Label logoFileNameLabel;
	private Text logoWidthText;
	private Text logoHeightText;
	private Text logoLocationTopText;
	private Text logoLocationRightText;
	private Button useUserDisplayNameInReportButton;
	
	/** values */
	private List<LdapConnectionInfo> ldapConnectionsList = new ArrayList<LdapConnectionInfo>();
	private LdapConnectionInfo selectedConnection = new LdapConnectionInfo();
	private String proxyHost;
	private int proxyPort;
	private String proxyUsername;
	private String proxyPassword;
	private boolean useSystemProxy;
	private String smtpHost;
	private int smtpPort;
	private int smtpType;
	private String smtpUsername;
	private String smtpPassword;
	private String smtpFrom;
	private boolean smtpAttachReports;
	private String reportEmailTemplate;
	private String policyEmailTemplate;
	private String reportEmailSubject;
	private String policyEmailSubject;
	private boolean checkForUpdates;
	private String applicationPassword;
	private boolean encryptDatabase;
	private String logLevels;
	private String ldapAccountNameAttribute; //"sAMAccountName"
	private String ldapUserDisplayNameAttribute; //"displayName"
	private String ldapGroupDisplayNameAttribute; //"description"
	private String ldapGroupTypeAttribute; //"groupType"
	private String ldapDNAttribute; //"distinguishedName"
	private String ldapPrimaryGroupIdAttribute; //"primaryGroupID"
	private String ldapGroupMemberAttribute; //"member"
	private String ldapMemberOfAttribute; //"memberOf"
	private boolean debugLdap;
	private boolean followReferrals;	
	private int dbConnectionType;
	private String dbConnectionString;
	private String dbConnectionDriver;
	private String dbConnectionUsername;
	private String dbConnectionPassword;
	private int batchSizeFiles;
	private int batchSizeAces;
	private int batchSizeMembers;
	private int batchSizeMemberships;
	private boolean calculateOwnershipRatioInOverview;
	private boolean calculateOwnershipRatioInEffectivePermissionsTab;
	private String logoFileName;
	private String logoBase64;
	private int logoWidth;
	private int logoHeight;
	private int logoLocationTop;
	private int logoLocationRight;
	private double logoSizeRatio; //helper field
	private String csvSeparator;
	private int maxFileExportMegaBytes;
	private boolean useUserDisplayNameInReport;
	
	public PreferenceDialog(Shell shell) {
		super(shell);
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
		
		//initialize values
		PreferenceHelper prefs = PreferenceHelper.getInstance();
    	
		// put a COPY of the connection ArrayList in the Dialog, otherwise action in the dialog will be immediately applied in the preferences
    	List<LdapConnectionInfo> ldapConnections = new ArrayList<LdapConnectionInfo>();
    	ldapConnections.addAll(prefs.getLdapConnectionsList());
    	
		this.ldapConnectionsList = ldapConnections;
		this.proxyHost = notNull(prefs.getProxyHost());
		this.proxyPort = prefs.getProxyPort();
		this.proxyUsername = notNull(prefs.getProxyUsername());
		this.proxyPassword = notNull(prefs.getProxyPassword());
		this.useSystemProxy = prefs.isUseSystemProxy();
		this.smtpHost = notNull(prefs.getSmtpHost());
		this.smtpPort = prefs.getSmtpPort();
		this.smtpType = prefs.getSmtpType();
		this.smtpUsername = notNull(prefs.getSmtpUsername());
		this.smtpPassword = notNull(prefs.getSmtpPassword());
		this.smtpFrom = notNull(prefs.getSmtpFrom());
		this.smtpAttachReports = prefs.isSmtpAttachReports();
		this.reportEmailTemplate = notNull(prefs.getReportEmailTemplate());
		this.policyEmailTemplate = notNull(prefs.getPolicyEmailTemplate());
		this.reportEmailSubject = notNull(prefs.getReportEmailSubject());
		this.policyEmailSubject = notNull(prefs.getPolicyEmailSubject());
		this.checkForUpdates = prefs.isCheckForUpdates();
		this.applicationPassword = notNull(prefs.getMasterPassword());
		this.encryptDatabase = prefs.isEncryptDatabase();
		this.logLevels = notNull(prefs.getLogLevels());
		this.ldapAccountNameAttribute = notNull(prefs.getLdapAccountNameAttribute());
		this.ldapUserDisplayNameAttribute = notNull(prefs.getLdapUserDisplayNameAttribute());
		this.ldapGroupDisplayNameAttribute = notNull(prefs.getLdapGroupDisplayNameAttribute());
		this.ldapGroupTypeAttribute = notNull(prefs.getLdapGroupTypeAttribute());
		this.ldapDNAttribute = notNull(prefs.getLdapDNAttribute());
		this.ldapPrimaryGroupIdAttribute = notNull(prefs.getLdapPrimaryGroupIdAttribute());
		this.ldapGroupMemberAttribute = notNull(prefs.getLdapGroupMemberAttribute());
		this.ldapMemberOfAttribute = notNull(prefs.getLdapMemberOfAttribute());
		this.debugLdap = prefs.isDebugLdap();
		this.followReferrals = prefs.isFollowReferrals();
		this.dbConnectionType = prefs.getDbConnectionType();
		this.dbConnectionString = prefs.getDbConnectionString();
		this.dbConnectionDriver = prefs.getDbConnectionDriver();
		this.dbConnectionUsername = prefs.getDbConnectionUsername();
		this.dbConnectionPassword = prefs.getDbConnectionPassword();
		this.batchSizeFiles = prefs.getBatchSizeFiles();
		this.batchSizeAces = prefs.getBatchSizeAces();
		this.batchSizeMembers = prefs.getBatchSizeMembers();
		this.batchSizeMemberships = prefs.getBatchSizeMemberships();
		this.calculateOwnershipRatioInOverview = prefs.isCalculateOwnershipRatioInOverview();
		this.calculateOwnershipRatioInEffectivePermissionsTab = prefs.isCalculateOwnershipRatioInEffectivePermissionsTab();
		this.logoFileName = notNull(prefs.getLogoFileName());
		this.logoBase64 = prefs.getLogoBase64();	
		this.logoWidth = prefs.getLogoWidth();
		this.logoHeight = prefs.getLogoHeight();
		this.logoLocationTop = prefs.getLogoLocationTop();
		this.logoLocationRight = prefs.getLogoLocationRight();
		this.logoSizeRatio = (double) logoWidth / (double) logoHeight;
		this.csvSeparator = prefs.getCsvSeparator();
		this.maxFileExportMegaBytes = prefs.getMaxFileExportMegaBytes();
		this.useUserDisplayNameInReport = prefs.isUseUserDisplayNameInReport();
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText("Preferences");
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.PREFERENCES_16X16));
		shell.setSize(1100, 900);
	}
	
	private String notNull(String value) {
		return (value == null ? "" : value);
	}

	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Preferences", ImageFactory.getInstance().getImage(AppImage.PREFERENCES_48X48));

		// container.setLayoutData(new GridData(GridData.FILL_BOTH));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 15;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		final CTabFolder tabFolder = new CTabFolder(composite, SWT.NONE);
		tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));
				
		CTabItem networkTab = new CTabItem(tabFolder, SWT.NONE);
		networkTab.setText("Network");
		networkTab.setImage(ImageFactory.getInstance().getImage(AppImage.NETWORK_32X32));
		networkTab.setControl(createNetworkTabContent(tabFolder));

		CTabItem internetTab = new CTabItem(tabFolder, SWT.NONE);
		internetTab.setText("Internet");
		internetTab.setImage(ImageFactory.getInstance().getImage(AppImage.INTERNET_32X32));
		internetTab.setControl(createInternetTabContent(tabFolder));
		
		CTabItem emailTab = new CTabItem(tabFolder, SWT.NONE);
		emailTab.setText("E-mail");
		emailTab.setImage(ImageFactory.getInstance().getImage(AppImage.EMAIL_32X32));
		emailTab.setControl(createEmailTabContent(tabFolder));
				
		CTabItem securityTab = new CTabItem(tabFolder, SWT.NONE);
		securityTab.setText("Security");
		securityTab.setImage(ImageFactory.getInstance().getImage(AppImage.SECURITY_32X32));
		securityTab.setControl(createSecurityTabContent(tabFolder));

		CTabItem databaseTab = new CTabItem(tabFolder, SWT.NONE);
		databaseTab.setText("Database");
		databaseTab.setImage(ImageFactory.getInstance().getImage(AppImage.DATABASE_32X32));
		databaseTab.setControl(createDatabaseTabContent(tabFolder));
		
		CTabItem advancedTab = new CTabItem(tabFolder, SWT.NONE);
		advancedTab.setText("Advanced");
		advancedTab.setImage(ImageFactory.getInstance().getImage(AppImage.ADVANCED_32X32));
		advancedTab.setControl(createAdvancedTabContent(tabFolder));
		
		return composite;
	}

	private Composite createNetworkTabContent(Composite tabFolder) {
		Composite networkTabContent = new Composite(tabFolder, SWT.NONE);
		networkTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		networkTabContent.setLayout(new GridLayout(1, false));
		networkTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		networkTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator ldapSeparator = new TitledSeparator(networkTabContent, SWT.NONE);
		ldapSeparator.setText("LDAP connections");
		ldapSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.CONNECTION_24X24));
		ldapSeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 10;
		ldapSeparator.setLayoutData(gridData);
		//ldapSeparator.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		Group detailsGroup = new Group(networkTabContent, SWT.NONE);
		GridData gd_detailsGroup = new GridData(GridData.FILL_HORIZONTAL);
		// gd_detailsGroup.heightHint = 271;
		gd_detailsGroup.horizontalIndent = 30;
		detailsGroup.setLayoutData(gd_detailsGroup);
		detailsGroup.setText("LDAP connection details");
		detailsGroup.setLayout(new GridLayout(2, false));

		Label connectionNameLabel = new Label(detailsGroup, SWT.NONE);
		connectionNameLabel.setText("Connection name:");
		connectionNameText = new Text(detailsGroup, SWT.BORDER);
		connectionNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		connectionNameText.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
				
		Label hostNameLabel = new Label(detailsGroup, SWT.NONE);
		hostNameLabel.setText("Host name:");
		hostNameCombo = new Combo(detailsGroup, SWT.BORDER);
		hostNameCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		hostNameCombo.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		loadDefaultHostnames();
		
		Label hostPortLabel = new Label(detailsGroup, SWT.NONE);
		hostPortLabel.setText("Host port:");
		hostPortLabel.setToolTipText("Default ports: LDAP: 389, LDAP SSL: 636, Global Catalog: 3268, , Global Catalog SSL: 3269");
		hostPortCombo = new Combo(detailsGroup, SWT.BORDER);
		hostPortCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		hostPortCombo.setToolTipText("Default ports: LDAP: 389, LDAP SSL: 636, Global Catalog: 3268");
		hostPortCombo.add("389 (default ldap)");
		hostPortCombo.add("636 (default SSL ldap)");
		hostPortCombo.add("3268 (default global catalog)");
		hostPortCombo.add("3269 (default SSL global catalog)");
		hostPortCombo.setText("389");
		hostPortCombo.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		hostPortCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switch (hostPortCombo.getSelectionIndex()) {
					case 0: hostPortCombo.setText("389"); break;
					case 1: hostPortCombo.setText("636"); break;
					case 2: hostPortCombo.setText("3268"); break;
					case 3: hostPortCombo.setText("3269"); break;
				}
			}
		});
		
		Label usernameLabel = new Label(detailsGroup, SWT.NONE);
		usernameLabel.setText("Username DN (optional for GSSAPI / Kerberos):  ");
		usernameLabel.setToolTipText("Use domain\\myuser or myuser@domain.com or a full DN like \"CN=Admin User,CN=Users,DC=domain,DC=com\"");
		usernameText = new Text(detailsGroup, SWT.BORDER);
		usernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		usernameText.setToolTipText("Use domain\\myuser or myuser@domain.com or a full DN like \"CN=Admin User,CN=Users,DC=domain,DC=com\"");
		usernameText.addModifyListener(new ModifyListener(){ 
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		
		Label passwordLabel = new Label(detailsGroup, SWT.NONE);
		passwordLabel.setText("Password (optional, asked when required):");
		passwordText = new Text(detailsGroup, SWT.BORDER | SWT.PASSWORD);
		passwordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		passwordText.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		
		Label encryptionLabel = new Label(detailsGroup, SWT.NONE);
		encryptionLabel.setText("Encryption:");
		encryptionCombo = new Combo(detailsGroup, SWT.BORDER | SWT.READ_ONLY);
		encryptionCombo.add("Plain");
		encryptionCombo.add("SSL/TLS");
		encryptionCombo.add("STARTTLS");
		encryptionCombo.select(0);
		encryptionCombo.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		
		Composite authenticationComposite = new Composite(detailsGroup, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		authenticationComposite.setLayoutData(gridData);
		authenticationComposite.setLayout(new GridLayout(3, false));
		
		Label authenticationLabel = new Label(authenticationComposite, SWT.NONE);
		authenticationLabel.setText("Authentication:");
		gridData = new GridData();
		gridData.widthHint = 320;
		gridData.horizontalIndent = -5;
		authenticationLabel.setLayoutData(gridData);
		authenticationCombo = new Combo(authenticationComposite, SWT.BORDER | SWT.READ_ONLY);
		authenticationCombo.add("Simple");
		authenticationCombo.add("DIGEST-MD5 (SASL)");
		authenticationCombo.add("CRAM-MD5 (SASL)");
		authenticationCombo.add("GSSAPI (Kerberos)");
		authenticationCombo.select(0);
		authenticationCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				authenticationDetailsButton.setVisible(authenticationCombo.getSelectionIndex() > 1);
				authDetailsDialog = null;
			}
		});
		authenticationCombo.addModifyListener(new ModifyListener(){
			@Override
			public void modifyText(ModifyEvent e) {
				isConnectionChanged = true;				
			}			
		});
		
		authenticationDetailsButton = new Button(authenticationComposite, SWT.PUSH);
		authenticationDetailsButton.setImage(ImageFactory.getInstance().getImage(AppImage.EDIT_16X16));
		authenticationDetailsButton.setToolTipText("Enter authentication details");
		authenticationDetailsButton.setVisible(false);
		authenticationDetailsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (authDetailsDialog == null) {
					authDetailsDialog = new LdapAuthenticationDetailsDialog(null, selectedConnection, Authentication.values()[authenticationCombo.getSelectionIndex()]);
				}
				authDetailsDialog.open();
			}
		});
			
		Composite detailsButtonBarComposite = new Composite(detailsGroup, SWT.NONE);
		detailsButtonBarComposite.setLayout(new GridLayout(2, false));
		GridData gd_composite = new GridData(GridData.FILL_HORIZONTAL);
		gd_composite.horizontalSpan = 2;
		detailsButtonBarComposite.setLayoutData(gd_composite);

		Button addButton = new Button(detailsButtonBarComposite, SWT.NONE);
		addButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		addButton.setText("Add");
		addButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_16X16));
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				LdapConnectionInfo info = new LdapConnectionInfo();
				try {
					getLdapInfoFromUI(info);
				} catch (NumberFormatException ex) {
					MessageDialog.openError(null, "Invalid number", "Please enter a valid port number.");
					return;
				}
				
				//validate fields
				if (connectionNameText.getText().length() == 0 || hostNameCombo.getText().length() == 0 || hostPortCombo.getText().length() == 0 || (usernameText.getText().length() == 0 && !info.getAuthentication().equals(Authentication.GSSAPI))) {
					MessageDialog.openError(null, "Details not complete", "Please enter all required fields.");
					return;
				}
				
				//check if the name already exists
				for (LdapConnectionInfo connection : ldapConnectionsList) {
					if (connection.getConnectionName().equalsIgnoreCase(info.getConnectionName())) {
						// connection name already exists
						MessageDialog.openError(null, "Name already exists", "The connection name " + info.getConnectionName() + " already exists in the list.");
						return;
					}
				}
				
				if (usernameText.getText().indexOf(",") == -1 && usernameText.getText().indexOf("@") == -1 && usernameText.getText().indexOf("\\") == -1) {
					MessageDialog.openError(null, "Invalid username", "Please modify your username, either use a full DN or a syntax like myadmin@mydomain or mydomain\\myadmin");
					return;
				}
				
				//determine the current max id
				int maxId = -1;
				for (LdapConnectionInfo connection : ldapConnectionsList) {
					maxId = Math.max(maxId, connection.getConnectionId());
				}
				info.setConnectionId(maxId + 1);
				
				ldapConnectionsList.add(info);
				clearLdapUIFields();
				tableViewer.refresh();
			}
		});

		updateButton = new Button(detailsButtonBarComposite, SWT.NONE);
		updateButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		updateButton.setText("Update");
		updateButton.setEnabled(false);
		updateButton.setImage(ImageFactory.getInstance().getImage(AppImage.APPLY_16X16));
		updateButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				updateLdapConnection();
			}
		});

		tableViewer = new TableViewer(networkTabContent, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = tableViewer.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_table.horizontalIndent = 30;
		table.setLayoutData(gd_table);

		TableViewerColumn nameColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		nameColumn.getColumn().setWidth(150);
		nameColumn.getColumn().setText("Connection name");

		TableViewerColumn hostNameColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		hostNameColumn.getColumn().setWidth(152);
		hostNameColumn.getColumn().setText("Host name");
		
		TableViewerColumn hostPortColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		hostPortColumn.getColumn().setWidth(100);
		hostPortColumn.getColumn().setText("Host port");
		
		TableViewerColumn usernameColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		usernameColumn.getColumn().setWidth(182);
		usernameColumn.getColumn().setText("Username");
	
		TableViewerColumn passwordColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		passwordColumn.getColumn().setWidth(142);
		passwordColumn.getColumn().setText("Password");
				
		TableViewerColumn encryptionColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		encryptionColumn.getColumn().setWidth(100);
		encryptionColumn.getColumn().setText("Encryption");
		
		TableViewerColumn authenticationColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		authenticationColumn.getColumn().setWidth(150);
		authenticationColumn.getColumn().setText("Authentication");
		
		tableViewer.setContentProvider(ArrayContentProvider.getInstance());
		tableViewer.setLabelProvider(new TableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				LdapConnectionInfo info = (LdapConnectionInfo) element;
				switch (columnIndex) {
					case 0: return info.getConnectionName();
					case 1: return info.getHostName();
					case 2: return String.valueOf(info.getHostPort());
					case 3: return info.getUsername();
					case 4: return info.getPassword().length() > 0 && !info.isOnlyStorePasswordInMemory() ? "********" : "";
					case 5: 
						switch (info.getEncryption()) {
							case PLAIN: return "Plain";
							case SSL: return "SSL/TLS";
							case STARTTLS: return "STARTTLS";
						}
					case 6: 
						switch (info.getAuthentication()) {
							case SIMPLE: return "Simple";
							case CRAMMD5: return "CRAM-MD5 (SASL)";
							case DIGESTMD5: return "DIGEST-MD5 (SASL)";
							case GSSAPI: return "GSSAPI (Kerberos)";
						}
					default: return "";
				}
			}
			
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0){
					return ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16);
				} else {
					return null;
				}
			}
		});

		tableViewer.setInput(ldapConnectionsList);

		Composite buttonBarComposite = new Composite(networkTabContent, SWT.NONE);
		buttonBarComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonBarComposite.setLayout(new GridLayout(3, false));

		final Button testConnectionButton = new Button(buttonBarComposite, SWT.PUSH);
		testConnectionButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		testConnectionButton.setText("Test connection");
		testConnectionButton.setEnabled(false);
		testConnectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.TEST_16X16));
		testConnectionButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ApplicationUtil.setBusyCursor();
				  
				selectedConnection = ldapConnectionsList.get(tableViewer.getTable().getSelectionIndex());
				LdapConnector connector = LdapFactory.getLdapConnector();
				boolean success = LdapFactory.openConnection(selectedConnection, connector);
				if (success) {
					connector.closeConnection();
					MessageDialog.openInformation(null, "Connection OK", "The connection to the LDAP server has been tested succesfully!");
				}
				
				ApplicationUtil.setDefaultCursor();
			}
		});
		
		final Button deleteButton = new Button(buttonBarComposite, SWT.NONE);
		deleteButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		deleteButton.setText("Delete");
		deleteButton.setEnabled(false);
		deleteButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		new Label(buttonBarComposite, SWT.NONE);
		deleteButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// check if this LDAP connection is still in use
				LdapConnectionInfo selectedConnection = ldapConnectionsList.get(tableViewer.getTable().getSelectionIndex());
				for (LdapOUInfo ouInfo : PreferenceHelper.getInstance().getLdapOUList()) {
					if (ouInfo.getConnection().getConnectionId() == selectedConnection.getConnectionId()) {
						MessageDialog.openError(null, "Connection in use", "This connection is still being used by a LDAP container in the Scan configuration.");
						return;
					}
				}
				
				ldapConnectionsList.remove(tableViewer.getTable().getSelectionIndex());
				clearLdapUIFields();
				tableViewer.refresh();
				deleteButton.setEnabled(false);
				testConnectionButton.setEnabled(false);
			}
		});
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener(){
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				// unsaved changes?
				if (isConnectionChanged || (authDetailsDialog != null && authDetailsDialog.isDirty())) {
					if (MessageDialog.openConfirm(getShell(), "Unsaved changes", "A LDAP connection is being edited but is not updated yet. Do you want to update the connection information?")) {
						// update the connection using the UI fields. Return if a validation error occurred.
						if (!updateLdapConnection()) {
							return;
						}
					}					
				}
								
				if (ldapConnectionsList.size() > 0 && tableViewer.getTable().getSelectionIndex() != -1) {
					deleteButton.setEnabled(true);
					testConnectionButton.setEnabled(true);
					
					selectedConnection = ldapConnectionsList.get(tableViewer.getTable().getSelectionIndex());
					updateButton.setEnabled(true);

					// update UI fields
					connectionNameText.setText(selectedConnection.getConnectionName());
					hostNameCombo.setText(selectedConnection.getHostName());
					hostPortCombo.setText(String.valueOf(selectedConnection.getHostPort()));
					if (selectedConnection.isOnlyStorePasswordInMemory()) {
						passwordText.setText("");
					} else {
						passwordText.setText(selectedConnection.getPassword());
					}
					encryptionCombo.select(selectedConnection.getEncryption().ordinal());
					authenticationCombo.select(selectedConnection.getAuthentication().ordinal());
					authenticationCombo.notifyListeners(SWT.Selection, new Event());
					usernameText.setText(selectedConnection.getUsername());
					isConnectionChanged = false;
				}
			}
		});
		
		isConnectionChanged = false;

		return networkTabContent;
	}
	
	private Composite createEmailTabContent(Composite tabFolder) {
		Composite emailTabContent = new Composite(tabFolder, SWT.NONE);
		emailTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		emailTabContent.setLayout(new GridLayout(2, false));
		//internetTabContent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		emailTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		emailTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator smtpInfoSeparator = new TitledSeparator(emailTabContent, SWT.NONE);
		smtpInfoSeparator.setText("SMTP server");
		smtpInfoSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.SERVER_24X24));
		smtpInfoSeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		smtpInfoSeparator.setLayoutData(gridData);
		
		Label smtpHostLabel = new Label(emailTabContent, SWT.NONE);
		smtpHostLabel.setText("SMTP host:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpHostLabel.setLayoutData(gridData);
		smtpHostText = new Text(emailTabContent, SWT.BORDER);
		smtpHostText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		smtpHostText.setText(smtpHost);
		
		Label smtpPortLabel = new Label(emailTabContent, SWT.NONE);
		smtpPortLabel.setText("SMTP port:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpPortLabel.setLayoutData(gridData);
		smtpPortCombo = new Combo(emailTabContent, SWT.BORDER);
		smtpPortCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		smtpPortCombo.add("25  (default)");
		smtpPortCombo.add("587 (default StartTLS)");
		smtpPortCombo.add("465 (default SSL)");
		smtpPortCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switch (smtpPortCombo.getSelectionIndex()) {
					case 0: smtpPortCombo.setText("25"); break;
					case 1: smtpPortCombo.setText("587"); break;
					case 2: smtpPortCombo.setText("465"); break;
				}
			}
		});		
		smtpPortCombo.setText(smtpPort == 0 ? "" : String.valueOf(smtpPort));
	
		
		Label smtpTypeLabel = new Label(emailTabContent, SWT.NONE);
		smtpTypeLabel.setText("SMTP connection type:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpTypeLabel.setLayoutData(gridData);
		smtpTypeCombo = new Combo(emailTabContent, SWT.BORDER | SWT.READ_ONLY);
		smtpTypeCombo.add("Plain");
		smtpTypeCombo.add("SSL/TLS");
		smtpTypeCombo.add("STARTTLS");
		smtpTypeCombo.select(smtpType);
		
		Label smtpUsernameLabel = new Label(emailTabContent, SWT.NONE);
		smtpUsernameLabel.setText("SMTP username:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpUsernameLabel.setLayoutData(gridData);
		smtpUsernameText = new Text(emailTabContent, SWT.BORDER);
		smtpUsernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		smtpUsernameText.setText(smtpUsername);
		
		Label smtpPasswordLabel = new Label(emailTabContent, SWT.NONE);
		smtpPasswordLabel.setText("SMTP password:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpPasswordLabel.setLayoutData(gridData);
		smtpPasswordText = new Text(emailTabContent, SWT.BORDER | SWT.PASSWORD);
		smtpPasswordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		smtpPasswordText.setText(smtpPassword);
		
		Label smtpFromLabel = new Label(emailTabContent, SWT.NONE);
		smtpFromLabel.setText("Sender e-mail address:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		smtpFromLabel.setLayoutData(gridData);
		smtpFromText = new Text(emailTabContent, SWT.BORDER);
		smtpFromText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		smtpFromText.setText(smtpFrom);
				
		Composite buttonBarComposite = new Composite(emailTabContent, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 220;
		buttonBarComposite.setLayoutData(gridData);
		buttonBarComposite.setLayout(new GridLayout(2, false));
		
		smtpAttachReportsButton = new Button(buttonBarComposite, SWT.CHECK);
		smtpAttachReportsButton.setText("Send reports as zipped attachment in the e-mails");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		smtpAttachReportsButton.setLayoutData(gridData);
		smtpAttachReportsButton.setSelection(smtpAttachReports);
		
		final Button testEmailButton = new Button(buttonBarComposite, SWT.PUSH);
		testEmailButton.setLayoutData(new GridData());
		testEmailButton.setText("Send test e-mail");
		testEmailButton.setImage(ImageFactory.getInstance().getImage(AppImage.TEST_16X16));
		testEmailButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				InputDialog inputDialog = new InputDialog(null, "E-mail address", "Please enter an e-mail address as recipient for the test e-mail:", "", null);
				
				if (inputDialog.open() == InputDialog.OK && inputDialog.getValue().length() > 0) {			
					ApplicationUtil.setBusyCursor();
					try {
						String msg = "Dear Sir/Madam,\r\n\r\nThis is a test e-mail sent from the application Permission Analyzer.";
						EmailHelper.sendMail(smtpHostText.getText(), Integer.parseInt(smtpPortCombo.getText()), smtpUsernameText.getText(), smtpPasswordText.getText(), 
								smtpTypeCombo.getSelectionIndex() == 1, smtpTypeCombo.getSelectionIndex() == 2, smtpFromText.getText(), 
								"Test e-mail from Permission Analyzer", msg, msg, inputDialog.getValue(), null, true);
						
					} catch (NumberFormatException ex) {
						MessageDialog.openError(null, "Invalid number", "Please enter a valid proxy port number.");
						return;
						
					} catch (EmailException ex) {
						logger.error("Error while sending test e-mail.", ex);
						MessageDialog.openError(null, "Could not send e-mail", "Error while sending the test e-mail, please see the log for more details.\r\n\r\n" + GUILogHandler.getNestedMessage(ex));
						return;
						
					} finally {
						ApplicationUtil.setDefaultCursor();
					}
					
					MessageDialog.openInformation(null, "SMTP server OK", "The test e-mail has been sent succesfully!");
				}
			}
		});
		
		TitledSeparator emailTemplatesSeparator = new TitledSeparator(emailTabContent, SWT.NONE);
		emailTemplatesSeparator.setText("E-mail templates");
		emailTemplatesSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.EMAIL_TEMPLATE_24X24));
		emailTemplatesSeparator.setSize(60, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		emailTemplatesSeparator.setLayoutData(gridData);
		
		// report subject and template
		Label reportEmailSubjectLabel = new Label(emailTabContent, SWT.NONE);
		reportEmailSubjectLabel.setText("E-mail subject for reports:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		reportEmailSubjectLabel.setLayoutData(gridData);
		reportEmailSubjectText = new Text(emailTabContent, SWT.BORDER);
		reportEmailSubjectText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		reportEmailSubjectText.setText(reportEmailSubject);
		
		Label templateReportLabel = new Label(emailTabContent, SWT.NONE);
		templateReportLabel.setText("E-mail template for reports:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		templateReportLabel.setLayoutData(gridData);
		reportEmailTemplateText = new Text(emailTabContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		reportEmailTemplateText.setText(reportEmailTemplate);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.heightHint = 6 * reportEmailTemplateText.getLineHeight();
		reportEmailTemplateText.setLayoutData(gridData);
		
		// policy subject and template
		Label policyEmailSubjectLabel = new Label(emailTabContent, SWT.NONE);
		policyEmailSubjectLabel.setText("E-mail subject for policies:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		policyEmailSubjectLabel.setLayoutData(gridData);
		policyEmailSubjectText = new Text(emailTabContent, SWT.BORDER);
		policyEmailSubjectText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		policyEmailSubjectText.setText(policyEmailSubject);
		
		Label templatePolicyLabel = new Label(emailTabContent, SWT.NONE);
		templatePolicyLabel.setText("E-mail template for policies:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		templatePolicyLabel.setLayoutData(gridData);
		policyEmailTemplateText = new Text(emailTabContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		policyEmailTemplateText.setText(policyEmailTemplate);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.heightHint = 6 * policyEmailTemplateText.getLineHeight();
		policyEmailTemplateText.setLayoutData(gridData);
		
		return emailTabContent;
	}
	
	private Composite createInternetTabContent(Composite tabFolder) {
		Composite internetTabContent = new Composite(tabFolder, SWT.NONE);
		internetTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		internetTabContent.setLayout(new GridLayout(2, false));
		//internetTabContent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		internetTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		internetTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator updateServiceSeparator = new TitledSeparator(internetTabContent, SWT.NONE);
		updateServiceSeparator.setText("Update service");
		updateServiceSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.UPDATE_SERVICE_24X24));
		updateServiceSeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		updateServiceSeparator.setLayoutData(gridData);
		
		useSystemProxyButton = new Button(internetTabContent, SWT.RADIO);
		useSystemProxyButton.setText("Use system proxy");
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 30;
		useSystemProxyButton.setLayoutData(gridData);
		useSystemProxyButton.setSelection(useSystemProxy);
		
		useNoProxyButton = new Button(internetTabContent, SWT.RADIO);
		useNoProxyButton.setText("Use no proxy");
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 30;
		useNoProxyButton.setLayoutData(gridData);
		useNoProxyButton.setSelection(!useSystemProxy && (proxyHost == null || proxyHost.isEmpty()));
		
		final Button useCustomProxyButton = new Button(internetTabContent, SWT.RADIO);
		useCustomProxyButton.setText("Use custom proxy:");
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 30;
		useCustomProxyButton.setLayoutData(gridData);
		useCustomProxyButton.setSelection(proxyHost != null && !proxyHost.isEmpty());
		
		Label proxyHostLabel = new Label(internetTabContent, SWT.NONE);
		proxyHostLabel.setText("Proxy host:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 15;
		proxyHostLabel.setLayoutData(gridData);
		proxyHostText = new Text(internetTabContent, SWT.BORDER);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 15;
		proxyHostText.setLayoutData(gridData);
		proxyHostText.setText(proxyHost);
		proxyHostText.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(final KeyEvent e) {
				if (e.keyCode != 13 && e.keyCode != SWT.ARROW_RIGHT && e.keyCode != SWT.ARROW_LEFT && e.keyCode != SWT.HOME && e.keyCode != SWT.END) {
					useCustomProxyButton.setSelection(true);
					useNoProxyButton.setSelection(false);
					useSystemProxyButton.setSelection(false);
				}
			}
		});
		
		Label proxyPortLabel = new Label(internetTabContent, SWT.NONE);
		proxyPortLabel.setText("Proxy port:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		proxyPortLabel.setLayoutData(gridData);
		proxyPortText = new Text(internetTabContent, SWT.BORDER);
		proxyPortText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		proxyPortText.setText(proxyPort == 0 ? "" : String.valueOf(proxyPort));
		
		Label proxyUsernameLabel = new Label(internetTabContent, SWT.NONE);
		proxyUsernameLabel.setText("Proxy username:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		proxyUsernameLabel.setLayoutData(gridData);
		proxyUsernameText = new Text(internetTabContent, SWT.BORDER);
		proxyUsernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		proxyUsernameText.setText(proxyUsername);
		
		Label proxyPasswordLabel = new Label(internetTabContent, SWT.NONE);
		proxyPasswordLabel.setText("Proxy password:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		proxyPasswordLabel.setLayoutData(gridData);
		proxyPasswordText = new Text(internetTabContent, SWT.BORDER | SWT.PASSWORD);
		proxyPasswordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		proxyPasswordText.setText(proxyPassword);
		
		/*
		checkForUpdatesButton = new Button(internetTabContent, SWT.CHECK);
		checkForUpdatesButton.setText("Check for new updates on startup");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 50;
		gridData.horizontalIndent = 30;
		gridData.horizontalSpan = 2;
		checkForUpdatesButton.setLayoutData(gridData);
		checkForUpdatesButton.setSelection(checkForUpdates);*/
		
		return internetTabContent;
	}
	
	private Composite createSecurityTabContent(Composite tabFolder) {
		Composite securityTabContent = new Composite(tabFolder, SWT.NONE);
		securityTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		securityTabContent.setLayout(new GridLayout(2, false));
		//securityTabContent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		securityTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		securityTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator securitySeparator = new TitledSeparator(securityTabContent, SWT.NONE);
		securitySeparator.setText("Data protection");
		securitySeparator.setImage(ImageFactory.getInstance().getImage(AppImage.ACCESS_CONTROL_24X24));
		securitySeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		securitySeparator.setLayoutData(gridData);
		
		Label applicationPasswordLabel = new Label(securityTabContent, SWT.NONE);
		applicationPasswordLabel.setText("Application password:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		applicationPasswordLabel.setLayoutData(gridData);
		applicationPasswordText = new Text(securityTabContent, SWT.BORDER | SWT.PASSWORD);
		applicationPasswordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		applicationPasswordText.setText(applicationPassword);
		applicationPasswordText.addKeyListener(new KeyListener(){
			@Override
			public void keyPressed(KeyEvent e) {}

			@Override
			public void keyReleased(KeyEvent e) {
				if (applicationPasswordText.getText().length() == 0) {
					encryptDatabaseButton.setSelection(false);
				}
				encryptDatabaseButton.setEnabled(applicationPasswordText.getText().length() > 0  && DBUtil.isLocalDB() && !(LicenseHelper.getEdition().equals(Edition.TRIAL) || LicenseHelper.getEdition().equals(Edition.BASIC)));
			}
		});
		
		if (LicenseHelper.getEdition().equals(Edition.TRIAL) || LicenseHelper.getEdition().equals(Edition.BASIC)) {
			Composite trialComposite = new Composite(securityTabContent, SWT.NONE);
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 3;
			gridData.heightHint = 40;
			gridData.verticalIndent = 20;
			trialComposite.setLayoutData(gridData);
			GridLayout gridLayout = new GridLayout(1, false);		
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("Database encryption is not available in the Trial or Basic edition.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		encryptDatabaseButton = new Button(securityTabContent, SWT.CHECK);
		if (DBUtil.isLocalDB()) {
			encryptDatabaseButton.setText("Encrypt database");
		} else {
			encryptDatabaseButton.setText("Encrypt database (encryption is not supported for external databases)");
		}
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 50;
		gridData.horizontalIndent = 30;
		gridData.horizontalSpan = 2;
		encryptDatabaseButton.setLayoutData(gridData);
		encryptDatabaseButton.setSelection(encryptDatabase && DBUtil.isLocalDB());
		encryptDatabaseButton.setEnabled(applicationPassword != null && applicationPassword.length() > 0 && DBUtil.isLocalDB() && !(LicenseHelper.getEdition().equals(Edition.TRIAL) || LicenseHelper.getEdition().equals(Edition.BASIC)));
				
		return securityTabContent;
	}
	
	private Composite createDatabaseTabContent(Composite tabFolder) {
		Composite databaseTabContent = new Composite(tabFolder, SWT.NONE);
		databaseTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 20;
		databaseTabContent.setLayout(gridLayout);
		//databaseTabContent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		databaseTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		databaseTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator databaseSeparator = new TitledSeparator(databaseTabContent, SWT.NONE);
		databaseSeparator.setText("Data storage");
		databaseSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.DATABASE_24X24));
		databaseSeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		databaseSeparator.setLayoutData(gridData);
		
		if (!LicenseHelper.getEdition().equals(Edition.ENTERPRISE)  && !LicenseHelper.getEdition().equals(Edition.CONSULTANT)
				&& !LicenseHelper.getEdition().equals(Edition.ADVANCED) && !LicenseHelper.getEdition().equals(Edition.SCAN_AGENT)) {
			Composite trialComposite = new Composite(databaseTabContent, SWT.NONE);
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 3;
			gridData.heightHint = 40;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("External database support is only available in the Advanced, Enterprise and Scan Agent edition.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label dbConnectionTypeLabel = new Label(databaseTabContent, SWT.NONE);
		dbConnectionTypeLabel.setText("Database type:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dbConnectionTypeLabel.setLayoutData(gridData);
		dbConnectionTypeCombo = new Combo(databaseTabContent, SWT.BORDER | SWT.READ_ONLY);
		dbConnectionTypeCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dbConnectionTypeCombo.add("Local H2 database");
		dbConnectionTypeCombo.add("Oracle");
		dbConnectionTypeCombo.add("DB2");
		dbConnectionTypeCombo.add("MS SQL Server");
		dbConnectionTypeCombo.add("MySQL");
		dbConnectionTypeCombo.add("PostgreSQL");
		dbConnectionTypeCombo.add("Derby");
		dbConnectionTypeCombo.add("Other");
		
		dbConnectionTypeCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switch (dbConnectionTypeCombo.getSelectionIndex()) {
					case 0:
						dbConnectionStringText.setText(PreferenceHelper.DEFAULT_CONNECTION_STRING);
						dbConnectionDriverText.setText("org.h2.Driver");
						break;
					case 1:
						dbConnectionStringText.setText("jdbc:oracle:thin:@yourServer:1521:yourDBName");
						dbConnectionDriverText.setText("oracle.jdbc.driver.OracleDriver");
						break;
					case 2:
						dbConnectionStringText.setText("jdbc:db2://yourServer:50000/yourDBName");
						dbConnectionDriverText.setText("com.ibm.db2.jcc.DB2Driver");
						break;
					case 3:
						dbConnectionStringText.setText("jdbc:sqlserver://yourServer\\optional-instance-name:1433;databaseName=yourDBName");
						dbConnectionDriverText.setText("com.microsoft.sqlserver.jdbc.SQLServerDriver");
						break;
					case 4:
						dbConnectionStringText.setText("jdbc:mysql://yourServer/yourDBName?rewriteBatchedStatements=true");
						dbConnectionDriverText.setText("com.mysql.jdbc.Driver");
						break;
					case 5:
						dbConnectionStringText.setText("jdbc:postgresql://yourServer:5432/yourDBName");
						dbConnectionDriverText.setText("org.postgresql.Driver");
						break;
					case 6:
						dbConnectionStringText.setText("jdbc:derby:[app_dir]yourDBName");
						dbConnectionDriverText.setText("org.apache.derby.jdbc.EmbeddedDriver");
						break;
					case 7:
						dbConnectionStringText.setText("jdbc:dbtype://yourServer:port/yourDBName");
						dbConnectionDriverText.setText("");
						break;	
				}
				
				if (dbConnectionTypeCombo.getSelectionIndex() == 0) {
					dbConnectionUsernameText.setText("");
					dbConnectionPasswordText.setText("");
					dbConnectionUsernameText.setEnabled(false);
					dbConnectionPasswordText.setEnabled(false);
				} else {
					dbConnectionUsernameText.setEnabled(true);
					dbConnectionPasswordText.setEnabled(true);
				}
			}
		});
		dbConnectionTypeCombo.select(dbConnectionType);
						
		Label dbConnectionLabel = new Label(databaseTabContent, SWT.NONE);
		dbConnectionLabel.setText("Database connection:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dbConnectionLabel.setLayoutData(gridData);
		dbConnectionStringText = new Text(databaseTabContent, SWT.BORDER);
		dbConnectionStringText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dbConnectionStringText.setText(dbConnectionString);
		
		Label dbConnectionDriverLabel = new Label(databaseTabContent, SWT.NONE);
		dbConnectionDriverLabel.setText("Database driver:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dbConnectionDriverLabel.setLayoutData(gridData);
		dbConnectionDriverText = new Text(databaseTabContent, SWT.BORDER);
		dbConnectionDriverText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dbConnectionDriverText.setText(dbConnectionDriver);
		
		Label dbConnectionUsernameLabel = new Label(databaseTabContent, SWT.NONE);
		dbConnectionUsernameLabel.setText("Database username:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dbConnectionUsernameLabel.setLayoutData(gridData);
		dbConnectionUsernameText = new Text(databaseTabContent, SWT.BORDER);
		dbConnectionUsernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dbConnectionUsernameText.setText(dbConnectionUsername);
		dbConnectionUsernameText.setEnabled(dbConnectionTypeCombo.getSelectionIndex() > 0);
		
		Label dbConnectionPasswordLabel = new Label(databaseTabContent, SWT.NONE);
		dbConnectionPasswordLabel.setText("Database password:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dbConnectionPasswordLabel.setLayoutData(gridData);
		dbConnectionPasswordText = new Text(databaseTabContent, SWT.BORDER | SWT.PASSWORD);
		dbConnectionPasswordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dbConnectionPasswordText.setText(dbConnectionPassword);
		dbConnectionPasswordText.setEnabled(dbConnectionTypeCombo.getSelectionIndex() > 0);
		
		Composite buttonComposite = new Composite(databaseTabContent, SWT.NONE);
		buttonComposite.setLayout(new GridLayout(4, false));
		gridData = new GridData();
		gridData.horizontalIndent = 25;
		gridData.horizontalSpan = 2;
		buttonComposite.setLayoutData(gridData);
		
		Button testConnectionButton = new Button(buttonComposite, SWT.PUSH);
		testConnectionButton.setText("Test database connection");
		testConnectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.TEST_16X16));
		testConnectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ApplicationUtil.setBusyCursor();
				try {
					DBUtil.testDbConnection(dbConnectionTypeCombo.getSelectionIndex(), dbConnectionStringText.getText(), dbConnectionDriverText.getText(), dbConnectionUsernameText.getText(), dbConnectionPasswordText.getText(), encryptDatabase);
					MessageDialog.openInformation(null, "Connection successful", "Connection successful");
				} catch (Exception ex) {
					logger.error("Error while testing the database connection.", ex);
					
					String message = "Connection failed: " + DBUtil.getNestedMessage(ex);
					if (message.toLowerCase().contains("store header")) {
						message += "\r\n\r\nThis probably means that the database has been encrypted and that the given application password to decrypt is not correct.";
					}
					
					if (ex instanceof ClassNotFoundException) {
						message += "\r\n\r\nMake sure you have downloaded the driver package from the website and that you have placed the External_DB_Drivers.jar in the plugins folder.";
					}
					message += "\r\n\r\nSee the log file for more details.";
					
					MessageDialog.openError(null, "Error", message);
				} finally {
					ApplicationUtil.setDefaultCursor();
				}
			}
			
		});
				
		Button defragDatabaseButton = new Button(buttonComposite, SWT.PUSH);
		defragDatabaseButton.setText("Defrag local database");
		defragDatabaseButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){

						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							monitor.beginTask("Defragmentation of the database in progress (this may take a few minutes).....", IProgressMonitor.UNKNOWN);
							
							try {
								String connectionString = PreferenceHelper.getInstance().getDbConnectionString(); // "jdbc:h2:" + databasePath + ";MAX_COMPACT_TIME=2000;ALIAS_COLUMN_NAME=TRUE;CACHE_SIZE=524288;MV_STORE=FALSE;MVCC=FALSE;"; //512 MB
								String path = connectionString.substring(8, connectionString.indexOf(";"));
								path = path.replace("[app_dir]", Platform.getLocation().toFile().getParent() + File.separator);
								
								File dbFile = new File(path + ".h2.db");
								double initialSize = 0;
								if (dbFile != null) {
									initialSize = dbFile.length();
								}
								DBObject.executeStatement("SHUTDOWN DEFRAG");
													
								String message = "Defragmentation completed.";
								if (initialSize != 0) {
									double sizeAfterCompress = dbFile.length();
									message += "\r\n\r\nOld database size: " + (int) (initialSize / 1024 / 1024) + "MB";
									message += "\r\nNew database size: " + (int) (sizeAfterCompress / 1024 / 1024) + "MB";
								}
								SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", message);
								
							} catch (Exception ex) {
								logger.error("Error during the defragmentation.", ex);
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error during the defragmentation: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
								
							} finally {
								DBObject.clearPreparedStatementCache(); //cached preparedStatements don't have a connection anymore
								FilterHelper.getInstance().applyNewFilter(false);
							}			
						}
						
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error during the defragmentation.", ex);
					MessageDialog.openError(null, "Error", "Error during the defragmentation: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
				}
			}
			
		});
		defragDatabaseButton.setEnabled(DBUtil.isLocalDB());
		
		Button compressDatabaseButton = new Button(buttonComposite, SWT.PUSH);
		compressDatabaseButton.setText("Compress local database");
		compressDatabaseButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){

						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							monitor.beginTask("Compressing the database (this may take a few minutes).....", IProgressMonitor.UNKNOWN);
							
							try {
								String connectionString = PreferenceHelper.getInstance().getDbConnectionString(); // "jdbc:h2:" + databasePath + ";MAX_COMPACT_TIME=2000;ALIAS_COLUMN_NAME=TRUE;CACHE_SIZE=524288;MV_STORE=FALSE;MVCC=FALSE;"; //512 MB
								String path = connectionString.substring(8, connectionString.indexOf(";"));
								path = path.replace("[app_dir]", Platform.getLocation().toFile().getParent() + File.separator);
								
								File dbFile = new File(path + ".h2.db");
								double initialSize = 0;
								if (dbFile != null) {
									initialSize = dbFile.length();
								}
								DBObject.executeStatement("SHUTDOWN COMPACT");
													
								String message = "Compressing database completed.";
								if (initialSize != 0) {
									double sizeAfterCompress = dbFile.length();
									double compressed = ((initialSize - sizeAfterCompress) / 1024 / 1024);
									double percent = 100 - ((sizeAfterCompress / initialSize) * 100);
									message += "\r\n\r\nOld database size: " + (int) (initialSize / 1024 / 1024) + "MB";
									message += "\r\nNew database size: " + (int) (sizeAfterCompress / 1024 / 1024) + "MB";
									message += "\r\nCompressed " + (int)compressed + "MB (" + (int)percent + "%)";
								}
								SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", message);
								
							} catch (Exception ex) {
								logger.error("Error while compressing database.", ex);
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while compressing the database: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
								
							} finally {
								DBObject.clearPreparedStatementCache(); //cached preparedStatements don't have a connection anymore
								FilterHelper.getInstance().applyNewFilter(false);
							}			
						}
						
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while compressing database.", ex);
					MessageDialog.openError(null, "Error", "Error while compressing the database: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
				}
			}
			
		});
		compressDatabaseButton.setEnabled(DBUtil.isLocalDB());
		
		Button runStatisticsDatabaseButton = new Button(buttonComposite, SWT.PUSH);
		runStatisticsDatabaseButton.setText("Run database statistics");
		runStatisticsDatabaseButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){

						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							try {
								DBUtil.updateStatistics();
								
								SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "Database statistics have been updated.");
								
							} catch (Exception ex) {
								logger.error("Error while running the database statistics.", ex);
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while running the database statistics: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
							}
						}
						
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while running the database statistics.", ex);
					MessageDialog.openError(null, "Error", "Error while running the database statistics: " + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.");
				}
				progressDialog.getProgressMonitor().beginTask("Updating database statistics...", IProgressMonitor.UNKNOWN);
			}
			
		});
		
		if (!LicenseHelper.getEdition().equals(Edition.ADVANCED) && !LicenseHelper.getEdition().equals(Edition.ENTERPRISE)  
				&& !LicenseHelper.getEdition().equals(Edition.CONSULTANT) && !LicenseHelper.getEdition().equals(Edition.SCAN_AGENT)) {
			dbConnectionTypeCombo.setEnabled(false);
			dbConnectionDriverText.setEnabled(false);
			dbConnectionUsernameText.setEnabled(false);
			dbConnectionPasswordText.setEnabled(false);
		}
		
		// Advanced database settings
		TitledSeparator advancedDatabaseSeparator = new TitledSeparator(databaseTabContent, SWT.NONE);
		advancedDatabaseSeparator.setText("Advanced database settings");
		advancedDatabaseSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.DATABASE_24X24));
		advancedDatabaseSeparator.setSize(60, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		advancedDatabaseSeparator.setLayoutData(gridData);
		
		Composite batchSizeWithInfoComposite = new Composite(databaseTabContent, SWT.NONE);
		batchSizeWithInfoComposite.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		batchSizeWithInfoComposite.setLayoutData(gridData);
		
		Label batchSizeFilesLabel = new Label(batchSizeWithInfoComposite, SWT.NONE);
		batchSizeFilesLabel.setText("Batch size for files:");
		gridData = new GridData();
		gridData .widthHint = 190;
		gridData.horizontalIndent = 25;
		batchSizeFilesLabel.setLayoutData(gridData);
		batchSizeFilesText = new Text(batchSizeWithInfoComposite, SWT.BORDER);
		batchSizeFilesText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		batchSizeFilesText.setText(Integer.toString(batchSizeFiles));
		Button infoButton = new Button(batchSizeWithInfoComposite, SWT.PUSH);
		infoButton.setImage(ImageFactory.getInstance().getImage(AppImage.INFO_16X16));
		infoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				MessageDialog.openInformation(getShell(), "Batch sizes", "The batch size indicates the number of records that is being sent to the database at once. It is a tradeoff between speed, memory usage and "
						+ "the size of transaction logs in the database. The larger the batch size, the more memory and required space for the transaction log it will need. ");
			}
		});
		
		Label batchSizeAcesLabel = new Label(databaseTabContent, SWT.NONE);
		batchSizeAcesLabel.setText("Batch size for ACE's:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		gridData .widthHint = 190;
		batchSizeAcesLabel.setLayoutData(gridData);
		batchSizeAcesText = new Text(databaseTabContent, SWT.BORDER);
		batchSizeAcesText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		batchSizeAcesText.setText(Integer.toString(batchSizeAces));
		
		Label batchSizeMembersLabel = new Label(databaseTabContent, SWT.NONE);
		batchSizeMembersLabel.setText("Batch size for members:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		gridData .widthHint = 190;
		batchSizeMembersLabel.setLayoutData(gridData);
		batchSizeMembersText = new Text(databaseTabContent, SWT.BORDER);
		batchSizeMembersText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		batchSizeMembersText.setText(Integer.toString(batchSizeMembers));
		
		Label batchSizeMembershipsLabel = new Label(databaseTabContent, SWT.NONE);
		batchSizeMembershipsLabel.setText("Batch size for group memberships:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		gridData .widthHint = 190;
		batchSizeMembershipsLabel.setLayoutData(gridData);
		batchSizeMembershipsText = new Text(databaseTabContent, SWT.BORDER);
		batchSizeMembershipsText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		batchSizeMembershipsText.setText(Integer.toString(batchSizeMemberships));
		
		return databaseTabContent;
	}

	private Composite createAdvancedTabContent(final Composite tabFolder) {
		Composite advancedTabContent = new Composite(tabFolder, SWT.NONE);
		advancedTabContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		advancedTabContent.setLayout(new GridLayout(2, false));
		//securityTabContent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		advancedTabContent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		advancedTabContent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TitledSeparator ldapSeparator = new TitledSeparator(advancedTabContent, SWT.NONE);
		ldapSeparator.setText("Generic LDAP settings");
		ldapSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.CONNECTION_24X24));
		ldapSeparator.setSize(60, 800);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 10;
		ldapSeparator.setLayoutData(gridData);
			
		Label accountNameLabel = new Label(advancedTabContent, SWT.NONE);
		accountNameLabel.setText("Account name attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		accountNameLabel.setLayoutData(gridData);
		ldapAccountNameAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapAccountNameAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapAccountNameAttributeText.setText(ldapAccountNameAttribute);
		
		Label userDisplayNameLabel = new Label(advancedTabContent, SWT.NONE);
		userDisplayNameLabel.setText("User display name attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		userDisplayNameLabel.setLayoutData(gridData);
		ldapUserDisplayNameAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapUserDisplayNameAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapUserDisplayNameAttributeText.setText(ldapUserDisplayNameAttribute);
		
		Label groupDisplayNameLabel = new Label(advancedTabContent, SWT.NONE);
		groupDisplayNameLabel.setText("Group display name attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		groupDisplayNameLabel.setLayoutData(gridData);
		ldapGroupDisplayNameAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapGroupDisplayNameAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapGroupDisplayNameAttributeText.setText(ldapGroupDisplayNameAttribute);
		
		Label groupTypeLabel = new Label(advancedTabContent, SWT.NONE);
		groupTypeLabel.setText("Group type attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		groupTypeLabel.setLayoutData(gridData);
		ldapGroupTypeAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapGroupTypeAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapGroupTypeAttributeText.setText(ldapGroupTypeAttribute);
		
		Label dnLabel = new Label(advancedTabContent, SWT.NONE);
		dnLabel.setText("DN attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		dnLabel.setLayoutData(gridData);
		ldapDNAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapDNAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapDNAttributeText.setText(ldapDNAttribute);
		
		Label primaryGroupIdLabel = new Label(advancedTabContent, SWT.NONE);
		primaryGroupIdLabel.setText("Primary Group ID attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		primaryGroupIdLabel.setLayoutData(gridData);
		ldapPrimaryGroupIdAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapPrimaryGroupIdAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapPrimaryGroupIdAttributeText.setText(ldapPrimaryGroupIdAttribute);
		
		Label groupMemberLabel = new Label(advancedTabContent, SWT.NONE);
		groupMemberLabel.setText("Group member attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		groupMemberLabel.setLayoutData(gridData);
		ldapGroupMemberAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapGroupMemberAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapGroupMemberAttributeText.setText(ldapGroupMemberAttribute);
		
		Label memberOfLabel = new Label(advancedTabContent, SWT.NONE);
		memberOfLabel.setText("MemberOf attribute:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		memberOfLabel.setLayoutData(gridData);
		ldapMemberOfAttributeText = new Text(advancedTabContent, SWT.BORDER);
		ldapMemberOfAttributeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapMemberOfAttributeText.setText(ldapMemberOfAttribute);
		
		followReferralsButton = new Button(advancedTabContent, SWT.CHECK);
		followReferralsButton.setText("Follow referrals");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 10;
		gridData.horizontalSpan = 2;
		followReferralsButton.setLayoutData(gridData);
		followReferralsButton.setSelection(followReferrals);
		
		debugLdapButton = new Button(advancedTabContent, SWT.CHECK);
		debugLdapButton.setText("Log LDAP debug information");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 5;
		gridData.horizontalSpan = 2;
		debugLdapButton.setLayoutData(gridData);
		debugLdapButton.setSelection(debugLdap);
				
		TitledSeparator overviewSeparator = new TitledSeparator(advancedTabContent, SWT.NONE);
		overviewSeparator.setText("Overview settings");
		overviewSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
		overviewSeparator.setSize(60, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 15;
		overviewSeparator.setLayoutData(gridData);
		
		calculateOwnershipRatioInOverviewButton = new Button(advancedTabContent, SWT.CHECK);
		calculateOwnershipRatioInOverviewButton.setText("Calculate ownership ratio's in the folder tree/table and export reports");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 10;
		gridData.horizontalSpan = 2;
		calculateOwnershipRatioInOverviewButton.setLayoutData(gridData);
		calculateOwnershipRatioInOverviewButton.setSelection(calculateOwnershipRatioInOverview);		
		calculateOwnershipRatioInOverviewButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (calculateOwnershipRatioInOverviewButton.getSelection()) {
					calculateOwnershipRatioInEffectivePermissionsTabButton.setSelection(true);
				}
			}
		});
		
		calculateOwnershipRatioInEffectivePermissionsTabButton = new Button(advancedTabContent, SWT.CHECK);
		calculateOwnershipRatioInEffectivePermissionsTabButton.setText("Calculate ownership ratio's in the effective permissions tab");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 5;
		gridData.horizontalSpan = 2;
		calculateOwnershipRatioInEffectivePermissionsTabButton.setLayoutData(gridData);
		calculateOwnershipRatioInEffectivePermissionsTabButton.setSelection(calculateOwnershipRatioInEffectivePermissionsTab);
				
		TitledSeparator applicationSeparator = new TitledSeparator(advancedTabContent, SWT.NONE);
		applicationSeparator.setText("Application debugging");
		applicationSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.ANALYZE_24X24));
		applicationSeparator.setSize(60, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 15;
		applicationSeparator.setLayoutData(gridData);
		
		Label logLevelsLabel = new Label(advancedTabContent, SWT.NONE);
		logLevelsLabel.setText("Log levels:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		logLevelsLabel.setLayoutData(gridData);
		logLevelsText = new Text(advancedTabContent	, SWT.BORDER);
		logLevelsText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		logLevelsText.setText(logLevels);
		
		//TODO "folder tracing" pref opnemen, puntkomma gescheiden, * voor directe subfolders, ** voor alle subfolders
		
		TitledSeparator reportingSeparator = new TitledSeparator(advancedTabContent, SWT.NONE);
		reportingSeparator.setText("Reporting");
		reportingSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.REPORT_24X24));
		reportingSeparator.setSize(60, 800);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 15;
		reportingSeparator.setLayoutData(gridData);
		
		Composite logoComposite = new Composite(advancedTabContent, SWT.NONE);
		logoComposite.setLayout(new GridLayout(12, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		logoComposite.setLayoutData(gridData);
				
		Label logoLabel = new Label(logoComposite, SWT.NONE);
		logoLabel.setText("Custom logo:");
		gridData = new GridData();
		gridData.widthHint = 160;
		gridData.horizontalIndent = 25;
		logoLabel.setLayoutData(gridData);
		Button browseLogoButton = new Button(logoComposite, SWT.PUSH);
		browseLogoButton.setText("Select logo");
		browseLogoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				FileDialog dialog = new FileDialog(tabFolder.getShell(), SWT.OPEN);
			    dialog.setFilterNames(new String[] { "Image Files", "All Files (*.*)" });
			    dialog.setFilterExtensions(new String[] { "*.png;*.jpg;*.jpeg;*.gif", "*.*" }); 
			    String filePath = dialog.open();
			    if (filePath != null) {
			    	// convert the image to a Base64 string
			    	FileInputStream fileInputStreamReader = null;
			    	ByteArrayInputStream bis = null;
			    	
			    	try {
				    	File imageFile =  new File(filePath);
				    	fileInputStreamReader = new FileInputStream(imageFile);
		                byte[] bytes = new byte[(int) imageFile.length()];
		                fileInputStreamReader.read(bytes);
				    	String encodedfile = new String(Base64.getEncoder().encode(bytes), "UTF-8");
				    	String fileExtension = imageFile.getName().substring(imageFile.getName().lastIndexOf(".") + 1);
				    	
				    	System.out.println("fileExtension: " + fileExtension);
				    	
				    	logoBase64 = "data:image/" + fileExtension + ";base64," + encodedfile;
				    					    	
				    	byte[] imageByte = Base64.getDecoder().decode(encodedfile);
				    	bis = new ByteArrayInputStream(imageByte);
				    	ImageData imageData = new ImageData(bis);
				    	Image logoImage = new Image(Display.getDefault(), imageData);
				    	
				    	logoFileNameLabel.setText(imageFile.getName() + " (" + logoImage.getBounds().width + " x " + logoImage.getBounds().height + " )");
				    	logoFileNameLabel.pack(true);
				    	logoFileNameLabel.getParent().pack(true);
			           
				    	logoSizeRatio = (double)logoImage.getBounds().width / (double)logoImage.getBounds().height;
				    	
				    	logoWidth = (int) ((double)logoHeight * logoSizeRatio);
				    	logoWidthText.setText(String.valueOf(logoWidth));
				    	
			    	} catch (Exception ex) {
			    		logger.error("Error while getting logo as base64", ex);
						MessageDialog.openError(getShell(), "Error", "Error while storing the logo: " + ex.getMessage());
			    	} finally {
			    		try {
				    		if (fileInputStreamReader != null) {
				    			fileInputStreamReader.close();
				    		}
				    		if (bis != null) {
				    			bis.close();
				    		}
			    		} catch (Exception ex){}
			    	}
			    }
			}
		});
		
		Button removeLogoButton = new Button(logoComposite, SWT.PUSH);
		removeLogoButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeLogoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				logoFileNameLabel.setText("(no logo)");
		    	logoFileNameLabel.pack(true);
		    	logoFileNameLabel.getParent().pack(true);
		    	logoBase64 = "";
			}
		});
				
		Label logoWidthLabel = new Label(logoComposite, SWT.NONE);
		logoWidthLabel.setText("Width (px): ");
		logoWidthText = new Text(logoComposite, SWT.BORDER);
		logoWidthText.setText(String.valueOf(logoWidth));
		gridData = new GridData();
		gridData.widthHint = 30;
		logoWidthText.setLayoutData(gridData);
		logoWidthText.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				try {					
					logoWidth = Integer.parseInt(logoWidthText.getText());
					logoHeight = (int) ((double)logoWidth / logoSizeRatio);
					logoHeightText.setText(Integer.toString(logoHeight));
					
				} catch (NumberFormatException ex) {
					MessageDialog.openWarning(null, "Not a number", "Please enter a number for the image width.");
					logoWidthText.setText(Integer.toString(logoWidth));
				}
			}
		});
		
		Label logoHeightLabel = new Label(logoComposite, SWT.NONE);
		logoHeightLabel.setText("Height (px): ");
		logoHeightText = new Text(logoComposite, SWT.BORDER);
		logoHeightText.setText(String.valueOf(logoHeight));
		gridData = new GridData();
		gridData.widthHint = 30;
		logoHeightText.setLayoutData(gridData);
		logoHeightText.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				try {					
					logoHeight = Integer.parseInt(logoHeightText.getText());
					logoWidth = (int) ((double)logoHeight * logoSizeRatio);
					logoWidthText.setText(Integer.toString(logoWidth));
					
				} catch (NumberFormatException ex) {
					MessageDialog.openWarning(null, "Not a number", "Please enter a number for the image height.");
					logoHeightText.setText(Integer.toString(logoHeight));
				}
			}
		});
		
		Label logoLocationRightLabel = new Label(logoComposite, SWT.NONE);
		logoLocationRightLabel.setText("Location from the right (px): ");
		logoLocationTopText = new Text(logoComposite, SWT.BORDER);
		logoLocationTopText.setText(String.valueOf(logoLocationTop));
		gridData = new GridData();
		gridData.widthHint = 30;
		logoLocationTopText.setLayoutData(gridData);
		
		Label logoLocationTopLabel = new Label(logoComposite, SWT.NONE);
		logoLocationTopLabel.setText("Location from the top (px): ");
		logoLocationRightText = new Text(logoComposite, SWT.BORDER);
		logoLocationRightText.setText(String.valueOf(logoLocationRight));
		gridData = new GridData();
		gridData.widthHint = 30;
		logoLocationRightText.setLayoutData(gridData);
		
		Button logoInfoButton = new Button(logoComposite, SWT.PUSH);
		logoInfoButton.setImage(ImageFactory.getInstance().getImage(AppImage.INFO_16X16));
		logoInfoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				MessageDialog.openInformation(getShell(), "Report logo", "A custom logo can be selected which will be embedded in the reports as Base64 text. You can specify how the logo will appear in the HTML report by modifying the width, height and x/y position.");
			}
		});
		
		logoFileNameLabel = new Label(logoComposite, SWT.NONE);
		logoFileNameLabel.setText(logoFileName);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 190;
		gridData.horizontalSpan = 12;
		logoFileNameLabel.setLayoutData(gridData);
		
		Composite csvReportComposite = new Composite(advancedTabContent, SWT.NONE);
		csvReportComposite.setLayout(new GridLayout(6, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		csvReportComposite.setLayoutData(gridData);
		
		Label csvSeparatorLabel = new Label(csvReportComposite, SWT.NONE);
		csvSeparatorLabel.setText("CSV separator:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		csvSeparatorLabel.setLayoutData(gridData);
		csvSeparatorText = new Text(csvReportComposite, SWT.BORDER);
		gridData = new GridData();
		gridData.widthHint = 30;
		csvSeparatorText.setLayoutData(gridData);
		csvSeparatorText.setText(csvSeparator);
		
		Label maxFileExportMegaBytesLabel = new Label(csvReportComposite, SWT.NONE);
		maxFileExportMegaBytesLabel.setText("Split file export after x MegaBytes:");
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		maxFileExportMegaBytesLabel.setLayoutData(gridData);
		maxFileExportMegaBytesText = new Text(csvReportComposite, SWT.BORDER);
		gridData = new GridData();
		gridData.widthHint = 30;
		maxFileExportMegaBytesText.setLayoutData(gridData);
		maxFileExportMegaBytesText.setText(String.valueOf(maxFileExportMegaBytes));
		
		Button maxFileExportMegaBytesInfoButton = new Button(csvReportComposite, SWT.PUSH);
		maxFileExportMegaBytesInfoButton.setImage(ImageFactory.getInstance().getImage(AppImage.INFO_16X16));
		maxFileExportMegaBytesInfoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				MessageDialog.openInformation(getShell(), "Split file export", "If you enter a value greater than zero the report export will create a new file after the maximum number of MB has been reached. The new file name with get the suffix _2.csv, _3.csv, etc.");
			}
		});
		
		useUserDisplayNameInReportButton = new Button(csvReportComposite, SWT.CHECK);
		useUserDisplayNameInReportButton.setText("Use user display name instead of account name in the reports");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 30;
		gridData.verticalIndent = 5;
		useUserDisplayNameInReportButton.setLayoutData(gridData);
		useUserDisplayNameInReportButton.setSelection(useUserDisplayNameInReport);
		
		return advancedTabContent;
	}
	
	@Override
	protected Control createButtonBar(Composite parent) {
		return super.createButtonBar(parent);
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			// unsaved changes?
			if (isConnectionChanged || (authDetailsDialog != null && authDetailsDialog.isDirty())) {
				if (MessageDialog.openConfirm(getShell(), "Unsaved changes", "A LDAP connection is being edited but is not updated yet. Do you want to update the connection information?")) {
					// update the connection using the UI fields. Return if a validation error occurred.
					if (!updateLdapConnection()) {
						return;
					}
				}
			}
			
			//validate UI input and copy them to variables
			proxyPort = validateNumber(proxyPortText.getText(), "proxy port", 0);
			if (proxyPort < 0) return;
			
			smtpPort = validateNumber(smtpPortCombo.getText(), "smtp port", 0);
			if (smtpPort < 0) return;
			
			batchSizeFiles = validateNumber(batchSizeFilesText.getText(), "batch size of files", 1000);
			if (batchSizeFiles < 0) return;
			
			batchSizeAces = validateNumber(batchSizeAcesText.getText(), "batch size of ACE's", 5000);
			if (batchSizeAces < 0) return;
			
			batchSizeMembers = validateNumber(batchSizeMembersText.getText(), "batch size of members", 1000);
			if (batchSizeMembers < 0) return;
			
			batchSizeMemberships = validateNumber(batchSizeMembershipsText.getText(), "batch size of group memberships", 1000);
			if (batchSizeMemberships < 0) return;
			
			logoWidth = validateNumber(logoWidthText.getText(), "logo width", 0);
			if (logoWidth < 0) return;
			
			logoHeight = validateNumber(logoHeightText.getText(), "logo height", 0);
			if (logoHeight < 0) return;
			
			logoLocationTop = validateNumber(logoLocationTopText.getText(), "logo location top", 0);
			if (logoLocationTop < 0) return;
			
			logoLocationRight = validateNumber(logoLocationRightText.getText(), "logo location right", 0);
			if (logoLocationRight < 0) return;
			
			maxFileExportMegaBytes = validateNumber(maxFileExportMegaBytesText.getText(), "number of lines for CSV split files", 0);
			if (maxFileExportMegaBytes < 0) return;
			
			if (useNoProxyButton.getSelection() || useSystemProxyButton.getSelection()) {
				proxyHostText.setText("");
			}
			
			proxyHost = proxyHostText.getText();
			proxyUsername = proxyUsernameText.getText();
			proxyPassword = proxyPasswordText.getText();
			useSystemProxy = useSystemProxyButton.getSelection();
			smtpHost = smtpHostText.getText();
			smtpType = smtpTypeCombo.getSelectionIndex();
			smtpUsername = smtpUsernameText.getText();
			smtpPassword = smtpPasswordText.getText();
			smtpFrom = smtpFromText.getText();
			smtpAttachReports = smtpAttachReportsButton.getSelection();
			reportEmailTemplate = reportEmailTemplateText.getText();
			policyEmailTemplate = policyEmailTemplateText.getText();
			reportEmailSubject = reportEmailSubjectText.getText();
			policyEmailSubject = policyEmailSubjectText.getText();
			//checkForUpdates = checkForUpdatesButton.getSelection();
			applicationPassword = applicationPasswordText.getText();
			encryptDatabase = encryptDatabaseButton.getSelection();			
			logLevels = logLevelsText.getText();
			ldapAccountNameAttribute = ldapAccountNameAttributeText.getText();
			ldapUserDisplayNameAttribute = ldapUserDisplayNameAttributeText.getText();
			ldapGroupDisplayNameAttribute = ldapGroupDisplayNameAttributeText.getText();
			ldapGroupTypeAttribute = ldapGroupTypeAttributeText.getText();
			ldapDNAttribute = ldapDNAttributeText.getText();
			ldapPrimaryGroupIdAttribute = ldapPrimaryGroupIdAttributeText.getText();
			ldapGroupMemberAttribute = ldapGroupMemberAttributeText.getText();
			ldapMemberOfAttribute = ldapMemberOfAttributeText.getText();
			debugLdap = debugLdapButton.getSelection();
			followReferrals = followReferralsButton.getSelection();
			dbConnectionType = dbConnectionTypeCombo.getSelectionIndex();
			dbConnectionString = dbConnectionStringText.getText();
			dbConnectionDriver = dbConnectionDriverText.getText();
			dbConnectionUsername = dbConnectionUsernameText.getText();
			dbConnectionPassword = dbConnectionPasswordText.getText();
			calculateOwnershipRatioInOverview = calculateOwnershipRatioInOverviewButton.getSelection();
			calculateOwnershipRatioInEffectivePermissionsTab = calculateOwnershipRatioInEffectivePermissionsTabButton.getSelection();
			logoFileName = logoFileNameLabel.getText();
			csvSeparator = csvSeparatorText.getText();
			useUserDisplayNameInReport = useUserDisplayNameInReportButton.getSelection();
			
			super.buttonPressed(buttonId);
		} else {
			super.buttonPressed(buttonId);
		}
	}

	/**
	 * Extracts information from the UI fields and updates the given {@link LdapConnectionInfo} object.
	 * 
	 */
	private void getLdapInfoFromUI(LdapConnectionInfo info) throws NumberFormatException {
		info.setHostPort(Integer.parseInt(hostPortCombo.getText()));
		info.setConnectionName(connectionNameText.getText());
		info.setHostName(hostNameCombo.getText());
		info.setPassword(passwordText.getText());
		info.setOnlyStorePasswordInMemory(passwordText.getText().isEmpty());
		info.setEncryption(Encryption.values()[encryptionCombo.getSelectionIndex()]);
		info.setAuthentication(Authentication.values()[authenticationCombo.getSelectionIndex()]);
		info.setUsername(usernameText.getText());
		
		// has the authentication dialog been opened?
		if (authDetailsDialog != null) {
			selectedConnection.setQualityOfProtectionAuthAllowed(authDetailsDialog.isQualityOfProtectionAuthAllowed());
			selectedConnection.setQualityOfProtectionAuthIntAllowed(authDetailsDialog.isQualityOfProtectionAuthIntAllowed());
			selectedConnection.setQualityOfProtectionAuthConfAllowed(authDetailsDialog.isQualityOfProtectionAuthConfAllowed());
			selectedConnection.setRealm(authDetailsDialog.getRealm());
			selectedConnection.setUseTicketCache(authDetailsDialog.isUseTicketCache());
			selectedConnection.setConfigPath(authDetailsDialog.getConfigPath());
			selectedConnection.setKDCAddress(authDetailsDialog.getKDCAddress());
		}
	}

	/**
	 * Clears the input fields for the LDAP connection details
	 */
	private void clearLdapUIFields() {
		connectionNameText.setText("");
		hostNameCombo.setText("");
		hostPortCombo.setText("");
		passwordText.setText("");
		encryptionCombo.select(0);
		authenticationCombo.select(0);
		authenticationCombo.notifyListeners(SWT.Selection, new Event());
		usernameText.setText("");
		selectedConnection = new LdapConnectionInfo();
		updateButton.setEnabled(false);
		authDetailsDialog = null;
		isConnectionChanged = false;
	}
	
	/**
	 * Method triggered by the update button and the OK button of this dialog in case the selectedConnection.getName() is not empty.
	 * @return
	 */
	public boolean updateLdapConnection() {
		// check if the name already exists
		for (LdapConnectionInfo connection : ldapConnectionsList) {
			if (connection.getConnectionName().equalsIgnoreCase(connectionNameText.getText()) && connection != selectedConnection) {
				// connection name already exists
				MessageDialog.openError(null, "Name already exists", "The connection name " + connectionNameText.getText() + " already exists in the list.");
				return false;
			}
		}
		
		if (usernameText.getText().indexOf(",") == -1 && usernameText.getText().indexOf("@") == -1 && usernameText.getText().indexOf("\\") == -1) {
			MessageDialog.openError(null, "Invalid username", "Please modify your username, either use a full DN or a syntax like myadmin@mydomain or mydomain\\myadmin");
			return false;
		}
		
		try {
			getLdapInfoFromUI(selectedConnection);
		} catch (NumberFormatException ex) {
			MessageDialog.openError(null, "Invalid number", "Please enter a valid port number.");
			return false;
		}
		
		//validate fields
		if (connectionNameText.getText().length() == 0 || hostNameCombo.getText().length() == 0 || hostPortCombo.getText().length() == 0 || (usernameText.getText().length() == 0 && !selectedConnection.getAuthentication().equals(Authentication.GSSAPI))) {
			MessageDialog.openError(null, "Details not complete", "Please enter all required fields.");
			return false;
		}
		
		clearLdapUIFields();
		tableViewer.refresh();
		return true;
	}
	
	private void loadDefaultHostnames() {
		try {
			ApplicationUtil.setBusyCursor();
			List<String> hostnames = LdapConnectorImpl.findLDAPServersInWindowsDomain();
			hostNameCombo.removeAll();
			for (String hostname : hostnames) {
				hostNameCombo.add(hostname);
			}
			
			if (hostNameCombo.getItemCount() > 0) {
				hostNameCombo.select(0);
			}
		} catch (Throwable ex) {
			logger.error("Error while looking up domain controllers", ex);
			
		} finally {
			ApplicationUtil.setDefaultCursor();
		}
	}
	
	private int validateNumber(String value, String fieldName, int defaultValue) {
		try {
			if (value.length() > 0) {
				return Integer.parseInt(value);
			} else {
				return defaultValue;
			}
		} catch (NumberFormatException ex) {
			MessageDialog.openError(getShell(), "Invalid number", "Please enter a valid number for the " + fieldName + ".");
			return -1;
		}
	}
	
	public List<LdapConnectionInfo> getLdapConnectionsList() {
		return ldapConnectionsList;
	}

	public String getProxyHost() {
		return proxyHost;
	}

	public int getProxyPort() {
		return proxyPort;
	}

	public boolean isCheckForUpdates() {
		return checkForUpdates;
	}

	public String getApplicationPassword() {
		return applicationPassword;
	}

	public boolean isEncryptDatabase() {
		return encryptDatabase;
	}

	public String getProxyUsername() {
		return proxyUsername;
	}

	public void setProxyUsername(String proxyUsername) {
		this.proxyUsername = proxyUsername;
	}

	public String getProxyPassword() {
		return proxyPassword;
	}

	public void setProxyPassword(String proxyPassword) {
		this.proxyPassword = proxyPassword;
	}

	public String getLogLevels() {
		return logLevels;
	}

	public void setLogLevels(String logLevels) {
		this.logLevels = logLevels;
	}

	public String getLdapAccountNameAttribute() {
		return ldapAccountNameAttribute;
	}

	public String getLdapUserDisplayNameAttribute() {
		return ldapUserDisplayNameAttribute;
	}

	public String getLdapGroupDisplayNameAttribute() {
		return ldapGroupDisplayNameAttribute;
	}

	public String getLdapGroupTypeAttribute() {
		return ldapGroupTypeAttribute;
	}

	public String getLdapDNAttribute() {
		return ldapDNAttribute;
	}

	public String getLdapPrimaryGroupIdAttribute() {
		return ldapPrimaryGroupIdAttribute;
	}

	public String getLdapGroupMemberAttribute() {
		return ldapGroupMemberAttribute;
	}

	public boolean isDebugLdap() {
		return debugLdap;
	}

	public boolean isFollowReferrals() {
		return followReferrals;
	}

	public String getDbConnectionString() {
		return dbConnectionString;
	}

	public String getDbConnectionUsername() {
		return dbConnectionUsername;
	}

	public String getDbConnectionPassword() {
		return dbConnectionPassword;
	}

	public String getDbConnectionDriver() {
		return dbConnectionDriver;
	}

	public int getDbConnectionType() {
		return dbConnectionType;
	}

	public String getSmtpHost() {
		return smtpHost;
	}

	public int getSmtpPort() {
		return smtpPort;
	}

	public int getSmtpType() {
		return smtpType;
	}

	public String getSmtpUsername() {
		return smtpUsername;
	}

	public String getSmtpPassword() {
		return smtpPassword;
	}

	public String getReportEmailTemplate() {
		return reportEmailTemplate;
	}

	public String getPolicyEmailTemplate() {
		return policyEmailTemplate;
	}

	public String getSmtpFrom() {
		return smtpFrom;
	}

	public boolean isSmtpAttachReports() {
		return smtpAttachReports;
	}

	public String getReportEmailSubject() {
		return reportEmailSubject;
	}

	public String getPolicyEmailSubject() {
		return policyEmailSubject;
	}

	public String getLdapMemberOfAttribute() {
		return ldapMemberOfAttribute;
	}

	public int getBatchSizeFiles() {
		return batchSizeFiles;
	}

	public void setBatchSizeFiles(int batchSizeFiles) {
		this.batchSizeFiles = batchSizeFiles;
	}

	public int getBatchSizeAces() {
		return batchSizeAces;
	}

	public void setBatchSizeAces(int batchSizeAces) {
		this.batchSizeAces = batchSizeAces;
	}

	public int getBatchSizeMembers() {
		return batchSizeMembers;
	}

	public int getBatchSizeMemberships() {
		return batchSizeMemberships;
	}

	public boolean isUseSystemProxy() {
		return useSystemProxy;
	}

	public boolean isCalculateOwnershipRatioInOverview() {
		return calculateOwnershipRatioInOverview;
	}

	public boolean isCalculateOwnershipRatioInEffectivePermissionsTab() {
		return calculateOwnershipRatioInEffectivePermissionsTab;
	}

	public String getLogoFileName() {
		return logoFileName;
	}

	public String getLogoBase64() {
		return logoBase64;
	}

	public int getLogoWidth() {
		return logoWidth;
	}

	public int getLogoHeight() {
		return logoHeight;
	}

	public int getLogoLocationTop() {
		return logoLocationTop;
	}

	public int getLogoLocationRight() {
		return logoLocationRight;
	}

	public String getCsvSeparator() {
		return csvSeparator;
	}

	public int getMaxFileExportMegaBytes() {
		return maxFileExportMegaBytes;
	}

	public boolean isUseUserDisplayNameInReport() {
		return useUserDisplayNameInReport;
	}
}
