package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.File;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.export.ReportComparator;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.ReportHistory;
import com.perdemia.permissionanalyzer.data.util.ReportCache;

/** This dialog is used to display the history of a report or policy.
 */
public class ReportHistoryDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(ReportHistoryDialog.class.getName());
	
	/** Filter data */
	private java.util.List<ReportHistory> reportHistoryList = new ArrayList<ReportHistory>();
	
	/** UI controls */
	private Text searchText;
	private TableViewer searchResultTableViewer;
	private ComboViewer reportCombo;
	private Button showOnlyFailedReportsCheckbox;
	private Button hideComparisonReportsCheckbox;
	private TableViewerColumn nameColumn;
	
	/** Date format in the search result list, for each scan log record */
	private DateFormat dateFormatter = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.getDefault());
	
	private Report report;
	
	/**
	 * The entity for the history, "Report" or "Policy"
	 */
	private String entity;
	private String entities;
	
	
	/**
	 * Constructor
	 * 
	 * @param shell
	 */
	public ReportHistoryDialog(Shell shell, Report report){
		super(shell);
		
		this.report = report;
		
		if (report.isPolicy()) {
			this.entity = "Policy";
			this.entities = "Policies";
		} else {
			this.entity = "Report";
			this.entities = "Reports";
		}		
		
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);		
	}
		
	@Override
	protected boolean isResizable()  {
	  return true;
	}
	
	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.REPORT_HISTORY_16X16));
		shell.setSize(1200, 900);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {		
		ApplicationUtil.setBusyCursor();
		
		Composite composite = (Composite) super.createDialogArea(container, entity + " history", ImageFactory.getInstance().getImage(AppImage.REPORT_HISTORY_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		container.getShell().setText(entity + " history");
				
		// set width, height and layout
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);			
		
		// Search label
		CLabel searchLabel = new CLabel(composite, SWT.NONE);
		searchLabel.setText("Search the " + entity.toLowerCase() + " history:");
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.BOTTOM;
		gridData.horizontalAlignment = SWT.LEFT;
		gridData.horizontalSpan = 3;
		searchLabel.setLayoutData(gridData);
					
		// Search textbox
		searchText = DelayedKeyListenerText.create(composite, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.ARROW_DOWN) {
					searchResultTableViewer.getTable().setFocus();
				} else {
					searchHistoryRecords(false);
				}					
			}			
		});
		searchText.setMessage("search");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		searchText.setLayoutData(gridData);
		searchText.addTraverseListener(new TraverseListener() {
			@Override
			public void keyTraversed(TraverseEvent e) {
				if (e.detail == SWT.TRAVERSE_RETURN) {
					searchHistoryRecords(true);
				}
			}
		});
		
		// server dropdown
		reportCombo = new ComboViewer(composite, SWT.BORDER | SWT.READ_ONLY);
		reportCombo.setContentProvider(ArrayContentProvider.getInstance());
		reportCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return ((Report) element).getName();
			}
			@Override
			public Image getImage(Object element) {
				if (((Report) element).isPolicy()) {
					return ImageFactory.getInstance().getImage(AppImage.POLICY_16X16);
				} else {
					return ImageFactory.getInstance().getImage(AppImage.REPORT_16X16);
				}
			}
		});
		gridData = new GridData();
		reportCombo.getCombo().setLayoutData(gridData);
		
		List<Report> reports = new ArrayList<Report>();
		Report dummyReport = new Report();
		dummyReport.setName("All " + entities.toLowerCase());
		dummyReport.setId(-1);
		reports.add(dummyReport);
		
		try {
			reports.addAll(Report.getAll());			
		} catch (Exception e) {
			logger.error("Error while retrieving the list of reports from the report history table", e);
		}
		
		reportCombo.setInput(reports);		
		
		// select the right report
		for (Report nextReport : reports) {
			if (nextReport.getId() == report.getId()) {
				reportCombo.setSelection(new StructuredSelection(nextReport), true);
			}
		}
		
		reportCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				searchHistoryRecords(true);						
			}					
		});
		
		
		// Search button
		Button searchMembersButton = new Button(composite, SWT.PUSH);
		searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_20X20));
		searchMembersButton.setToolTipText("Search records");
		searchMembersButton.setLayoutData(new GridData());
		searchMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchHistoryRecords(true);
			}
		});
		
		// filter checkboxes
		Composite filterComposite = new Composite(composite, SWT.NONE);
		filterComposite.setLayout(new GridLayout(2, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		filterComposite.setLayoutData(gridData);
		
		showOnlyFailedReportsCheckbox = new Button(filterComposite, SWT.CHECK);
		showOnlyFailedReportsCheckbox.setText("Only show " + entities.toLowerCase() + " that have failed");
		showOnlyFailedReportsCheckbox.setSelection(false);
		showOnlyFailedReportsCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.STATUS_WARNING_16X16));
		showOnlyFailedReportsCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchHistoryRecords(true);
			}
		});
		
		if (!report.isPolicy()) {
			hideComparisonReportsCheckbox = new Button(filterComposite, SWT.CHECK);
			hideComparisonReportsCheckbox.setText("Hide comparison reports");
			hideComparisonReportsCheckbox.setSelection(false);
			hideComparisonReportsCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.COMPARE_REPORT_16X16));
			hideComparisonReportsCheckbox.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					searchHistoryRecords(true);
				}
			});
		}
							
		searchResultTableViewer = new TableViewer(composite, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.VIRTUAL | SWT.MULTI);	
		searchResultTableViewer.setUseHashlookup(true);
		searchResultTableViewer.getTable().setHeaderVisible(true);
		searchResultTableViewer.getTable().setLinesVisible(true);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 3;
		searchResultTableViewer.getTable().setLayoutData(gridData);
		
		TableViewerColumn timeColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		timeColumn.getColumn().setWidth(200);
		timeColumn.getColumn().setText("Run date");
		
		nameColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		nameColumn.getColumn().setText(entity + " name");

		TableViewerColumn policyStatusColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		policyStatusColumn.getColumn().setWidth(150);
		policyStatusColumn.getColumn().setText("Policy status");
		
		TableViewerColumn itemCountColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		itemCountColumn.getColumn().setWidth(80);
		itemCountColumn.getColumn().setText("Result count");
		
		TableViewerColumn policyThresholdColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		policyThresholdColumn.getColumn().setWidth(70);
		policyThresholdColumn.getColumn().setText("Threshold");
				
		TableViewerColumn reportTypeColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		if (report.isPolicy()) {
			reportTypeColumn.getColumn().setWidth(180);
		} else {
			reportTypeColumn.getColumn().setWidth(250);
		}
		reportTypeColumn.getColumn().setText("Report type");
		
		TableViewerColumn lastRunMessageColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		lastRunMessageColumn.getColumn().setWidth(120);
		lastRunMessageColumn.getColumn().setText("Last run message");
		
		TableViewerColumn fileTypeColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		fileTypeColumn.getColumn().setWidth(150);
		fileTypeColumn.getColumn().setText("Email address");
		
		TableViewerColumn targetPathColumn = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		targetPathColumn.getColumn().setWidth(120);
		targetPathColumn.getColumn().setText("File path");					
		
		if (!report.isPolicy()) {
			// hide the policy columns
			policyStatusColumn.getColumn().setWidth(0);
			policyStatusColumn.getColumn().setResizable(false);
			policyThresholdColumn.getColumn().setWidth(0);
			policyThresholdColumn.getColumn().setResizable(false);
		}
		
		searchResultTableViewer.setContentProvider(ArrayContentProvider.getInstance());
		searchResultTableViewer.setLabelProvider(new TableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				ReportHistory reportHistory = (ReportHistory) element;
				switch (columnIndex) {
				case 0:
					return dateFormatter.format(reportHistory.getRunDate());
				case 1:
					try {
						return String.valueOf(ReportCache.getReport(reportHistory.getReportId()).getName());
					} catch (DBException e) {
						// exception already logged, this should never happen
						return "Unknown report name for ID " + reportHistory.getReportId();
					}
				case 2:
					if (reportHistory.getThreshold() == 0) {
						return "";
					} else if (reportHistory.getStatusMessage() == null || !reportHistory.getStatusMessage().equalsIgnoreCase("ok")) {
						return "Policy ran with errors";
					} else if (reportHistory.getCount() >= reportHistory.getThreshold()) {
						return "Found permissions";
					} else {
						return "Passed";
					}
				case 3:
					return String.valueOf(reportHistory.getCount());
				case 4:
					return String.valueOf(reportHistory.getThreshold());
				case 5:
					if (reportHistory.isWasComparison()) {
						return "Comparison";
					} else {
						return Report.reportTypeToString(reportHistory.getReportType());
					}
				case 6:
					return reportHistory.getStatusMessage();					
				case 7:
					return reportHistory.getEmailAddress();
				case 8:
					return reportHistory.getFilePath();
				default:
					return "";
				}
			}

			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0 || columnIndex == 2){
					ReportHistory reportHistory = (ReportHistory) element;
					if (reportHistory.getStatusMessage() == null || !reportHistory.getStatusMessage().equalsIgnoreCase("ok")) {
						return ImageFactory.getInstance().getImage(AppImage.STATUS_ERROR_16X16);
					} else if (reportHistory.getCount() >= reportHistory.getThreshold() && reportHistory.getThreshold() > 0) {
						return ImageFactory.getInstance().getImage(AppImage.STATUS_WARNING_16X16);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.VALID_16X16);
					}
				} else {
					return null;
				}
			}
		});
				
		searchResultTableViewer.setItemCount(reportHistoryList.size());
		searchResultTableViewer.setInput(reportHistoryList);
		
		//----------------------------------------------
		// context menu for the table
		//----------------------------------------------
		MenuManager memberMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		memberMenuMgr.setRemoveAllWhenShown(true);
		memberMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final ReportHistory reportHistory = ((ReportHistory) selection.getFirstElement());
						
					// open report file
					Action action = new Action() {
						public void run() {
							String filePath = reportHistory.getFilePath();
							if (filePath != null && !filePath.isEmpty()) {
								Program.launch(filePath);
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "File not present", "The target file does not exist on the system.");
							}
						}
					};
					action.setText("Open report file");
					if (reportHistory.isPolicy()) {
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.POLICY_16X16));
					} else {
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.REPORT_16X16));
					}
					manager.add(action);
										
					// open folder					
					action = new Action() {
						public void run() {
							if (reportHistory.getFilePath() != null && !reportHistory.getFilePath().isEmpty()) {
								String filePath = new File(reportHistory.getFilePath()).getParent();
								Program.launch(filePath);
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "File not present", "The target file does not exist on the system.");
							}
						}
					};
					action.setText("Open report folder in Explorer");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
					manager.add(action);
					
					// compare report					
					action = new Action() {
						public void run() {
							IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
							if (selection.size() != 2) {
								SyncMessageDialog.showMessage(MessageDialog.WARNING, "Report comparison", "Please select two history records to compare.");
								return;
							}
							List<?> selectedRecords = selection.toList();
							ReportHistory record1 = (ReportHistory) selectedRecords.get(0);
							ReportHistory record2 = (ReportHistory) selectedRecords.get(1);
							
							try {
								if (record1.getRunDate().before(record2.getRunDate())) {
									ReportComparator.compareTwoReportRuns(record1, record2, ReportCache.getReport(record1.getReportId()), true, true, true, true, true, true, null);
								} else {
									ReportComparator.compareTwoReportRuns(record2, record1, ReportCache.getReport(record1.getReportId()), true, true, true, true, true, true, null);
								}
							} catch (DBException ex) {
								logger.error("Error while looking up the report in the database", ex);
								MessageDialog.openError(null, "Error", "Error while looking up the report in the database: " + ex.getMessage());
							}
						}
					};
					action.setText("Compare two selected reports");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.COMPARE_REPORT_16X16));
					manager.add(action);
										
					// delete action
					action = new Action() {
						public void run() {
							try {
								IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
								if (SyncMessageDialog.showMessage(MessageDialog.CONFIRM, "Delete report", "Are you sure you want to delete the " + selection.size() + " selected history record(s)?")) {
									Iterator<?> selectionIter = selection.iterator();
									while (selectionIter.hasNext()) {
										ReportHistory historyRecord = (ReportHistory) selectionIter.next();
										historyRecord.delete();
										reportHistoryList.remove(historyRecord);
									}
									
									searchResultTableViewer.refresh();
								}
							} catch (DBException ex) {
								logger.error("Error while deleting the report history record(s) in the database", ex);
								MessageDialog.openError(null, "Error", "Error while deleting the report history record(s) in the database: " + ex.getMessage());
							}
						}
					};
					action.setText("Delete history record(s)");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.DELETE_16X16));
					manager.add(action);
				}
				
			}
		});
		Menu memberMenu = memberMenuMgr.createContextMenu(searchResultTableViewer.getTable());
		searchResultTableViewer.getTable().setMenu(memberMenu);
				
		// initialize the search
		searchHistoryRecords(true);
		
		ApplicationUtil.setDefaultCursor();
		return composite;
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
		
		CLabel explainLabel = new CLabel(parent, SWT.NONE);
		explainLabel.setText("Select two records and compare them in the context menu");
		explainLabel.setImage(ImageFactory.getInstance().getImage(AppImage.COMPARE_REPORT_16X16));
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
			
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		createButton(parent, IDialogConstants.CANCEL_ID, "Close", false);
	}
	
	/**
	 * Collects the search parameters from the UI, build a query for the report history records, and refreshes the table.
	 * 
	 * @param explicitUserSearchAction true in case the user pressed the search button or ENTER in the search text box
	 */
	private void searchHistoryRecords(boolean explicitUserSearchAction) {
		String search = searchText.getText().toLowerCase().replace('*', '%');
		
		// only search when the user explicit requested the search OR when the length is greater than 2
		if (explicitUserSearchAction ||	search.length() > 2) {
			
			reportHistoryList.clear();
			
			if (!search.endsWith("%")) {
				search += "%";
			}
			
			if (!search.startsWith("%")) {
				search = "%" + search;
			}
			
			try {
				ApplicationUtil.setBusyCursor();
				
				StringBuilder whereClause = new StringBuilder();
				List<Object> whereParameters = new ArrayList<Object>();
				if (reportCombo.getCombo().getSelectionIndex() > 0) {
					// hide the report name column, since where showing one particular report
					nameColumn.getColumn().setResizable(false);
					nameColumn.getColumn().setWidth(0);
					
					// always remove the ' and ', we'll remove it at the end
					Report selectedReport = ((Report)((StructuredSelection)reportCombo.getSelection()).getFirstElement());
					whereClause.append(" and report_id = ?");
					whereParameters.add(selectedReport.getId());
				} else {
					// enable the report name column, since where showing all the reports
					nameColumn.getColumn().setWidth(285);
					nameColumn.getColumn().setResizable(true);
				}
								
				if (showOnlyFailedReportsCheckbox.getSelection()) {
					whereClause.append(" and (STATUS_MESSAGE != 'OK' or (THRESHOLD > 0 and THRESHOLD <= RESULT_COUNT)) ");
				}
				
				if (!report.isPolicy()) {
					if (hideComparisonReportsCheckbox.getSelection()) {
						whereClause.append(" and WAS_COMPARISON = false ");
					}
				}
								
				if (search.length() > 2) {
					whereClause.append(" and (lower(FILE_PATH) like ? or lower(EMAIL_ADDRESS) like ? or lower(STATUS_MESSAGE) like ?) ");
					whereParameters.add(search);
					whereParameters.add(search);
					whereParameters.add(search);
				}
				
				if (report.isPolicy()) {
					whereClause.append(" and THRESHOLD > 0");
				} else {
					whereClause.append(" and THRESHOLD = 0");
				}
				
				if (whereClause.length() > 0) {
					reportHistoryList = ReportHistory.where(whereClause.toString().substring(4), whereParameters.toArray(), "RUN_DATE DESC, REPORT_ID");
				} else {
					reportHistoryList = ReportHistory.getAll();
				}
				
			} catch (DBException ex) {
				logger.error("Error while searching for report history records: " + ex.getMessage(), ex);
				MessageDialog.openError(null, "Error", "Error while searching for report history records: " + ex.getMessage());
			}
			
			searchResultTableViewer.setInput(reportHistoryList);
			searchResultTableViewer.setItemCount(reportHistoryList.size());
			searchResultTableViewer.refresh();
			
			if (reportHistoryList.isEmpty() && explicitUserSearchAction){
				MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "No results", "No report history records found.");
			}
			
			ApplicationUtil.setDefaultCursor();
		}		
	}
}
