package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.ArrayList;
import java.util.List;

import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.FilterSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class SaveFilterDialog extends DialogWithTopBar {
	private TableViewer tableViewer;
	private int defaultSelection;
	private List<FilterSet> filterSets;
	private FilterSet selectedFilterSet;
	private String title;
	
	/** Creat button ID */
	public final static int CREATE_ID = 3;
		
	public SaveFilterDialog(Shell shell, List<FilterSet> filterSets, int defaultSelection, String title) {
		super(shell);
		this.defaultSelection = defaultSelection;
		this.filterSets = filterSets;
		this.title = title;
		
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.SAVE_16X16));
		shell.setSize(800, 600);
		shell.setText(title);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, title, ImageFactory.getInstance().getImage(AppImage.SELECTION_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.NONE);
		label.setText("Please select a selection to overwrite or create a new one:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 30;
		label.setLayoutData(gridData);
		
		tableViewer = new TableViewer(composite, SWT.BORDER);
		tableViewer.getTable().setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// label provider
		tableViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				return ((FilterSet) element).getName();
			}

			public Image getImage(Object element) {
				return ImageFactory.getInstance().getImage(AppImage.SELECTION_16X16);
			}
		});
		
		// content provider and input
		tableViewer.setContentProvider(ArrayContentProvider.getInstance());
		tableViewer.setInput(filterSets);
		
		if (defaultSelection != -1) {
			tableViewer.getTable().select(defaultSelection);
		}
				
		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		Button selectionButton = createButton(parent, CREATE_ID, "Create new selection", false);
		selectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.SELECTION_16X16));
		((GridData) selectionButton.getLayoutData()).widthHint = 200;
		selectionButton.pack();
		super.createButtonsForButtonBar(parent);
		
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == CREATE_ID) {
			// create a new filter set
			InputDialog dialog = new InputDialog(Display.getCurrent().getActiveShell(), "New filter set", "Please enter a name for the new filter set:", "", new IInputValidator() {
				@Override
				public String isValid(String newText) {
					if (newText.length() > 255) {
						return "Please enter a name with a maximum length of 255 characters.";
					}
					for (FilterSet filterSet : filterSets) {
						if (filterSet.getName().equals(newText)) {
							return "Name already exists, please enter another name.";
						}
					}
					return null;
				}
			});
			if (dialog.open() == IDialogConstants.OK_ID) {
				FilterSet filterSet = new FilterSet();
				filterSet.setName(dialog.getValue());
				filterSet.setFilters(new ArrayList<Filter>());
				selectedFilterSet = filterSet;
				setReturnCode(CREATE_ID);
				close();
			}
		} else if (buttonId == IDialogConstants.OK_ID) {
			if (((IStructuredSelection) tableViewer.getSelection()).size() == 0) {
				MessageDialog.openError(getParentShell(), "No selection selected", "Please select an existing selection or create a new one.");
			} else {
				selectedFilterSet = (FilterSet) ((IStructuredSelection)tableViewer.getSelection()).getFirstElement();
				super.buttonPressed(buttonId);
			}
		} else {
			super.buttonPressed(buttonId);
		}
	}

	public FilterSet getSelectedFilterSet() {
		return selectedFilterSet;
	}
}
