package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.IOException;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.model.DirectoryInfo.SourceType;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

public class SelectImportFileDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(SelectImportFileDialog.class.getName());
	
	private Text filePathText;
	private Combo fileTypeCombo;
	
	private String filePath;
	private SourceType sourceType = SourceType.POWER_SHELL;
	
	public SelectImportFileDialog(Shell shell) {
		super(shell);
	}
	
	public SelectImportFileDialog(Shell shell, SourceType sourceType) {
		super(shell);
		this.sourceType = sourceType;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
		shell.setSize(900, LicenseHelper.getEdition().equals(Edition.TRIAL) ? 470 : 450);
		shell.setText("Import ACL file");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Import ACL file", ImageFactory.getInstance().getImage(AppImage.IMPORT_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.heightHint = 40;
			gridData.horizontalSpan = 3;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("The Trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label label = new Label(composite, SWT.WRAP);
		label.setText("Instead of scanning a local directory or network share, you can import a text file that contains all the directories and files with their Access Control List information. We currently support a text file exported by PowerShell or the EMC Isilon NAS system. If you have another format, then please contact support to see if we can add your format to the list of file types.");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		gridData.heightHint = 130;
		label.setLayoutData(gridData);
		
		
		label = new Label(composite, SWT.WRAP);
		label.setText("File type:");
		
		fileTypeCombo = new Combo(composite, SWT.READ_ONLY);
		fileTypeCombo.add("PowerShell export file");
		fileTypeCombo.add("EMC Isilon NAS");
		
		switch (sourceType) {
			case EMC_ISILON:
				fileTypeCombo.select(1); break;
			case POWER_SHELL: default:
				fileTypeCombo.select(0);
		}
		
		Button infoButton = new Button(composite, SWT.PUSH);
		infoButton.setLayoutData(new GridData());
		infoButton.setImage(ImageFactory.getInstance().getImage(AppImage.INFO_16X16));
		infoButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					TextboxDialog infoDialog;
					if (fileTypeCombo.getSelectionIndex() == 0) {
						infoDialog = new TextboxDialog(getShell(), "File format details", "", ApplicationUtil.getFileContent("PowerShell_example.txt"));
					} else {
						infoDialog = new TextboxDialog(getShell(), "File format details", "", ApplicationUtil.getFileContent("EMC_Isilonacl_example.txt"));
					}
					infoDialog.open();
				} catch (IOException ex) {
					logger.error("Could not load the ACL file format example.", ex.getMessage());
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Could not load the ACL file format example: " + ex.getMessage());
				}
				
			}
		});
		
		Label filePathLabel = new Label(composite, SWT.NONE);
		filePathLabel.setText("File path:     ");
		Composite browseFileComposite = new Composite(composite, SWT.NONE);
		browseFileComposite.setLayout(new GridLayout(2, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		gridData.horizontalSpan = 2;
		browseFileComposite.setLayoutData(gridData);
		filePathText = new Text(browseFileComposite, SWT.BORDER);
		filePathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseFileButton = new Button(browseFileComposite, SWT.PUSH);
		browseFileButton.setLayoutData(new GridData());
		browseFileButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
		browseFileButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				FileDialog fileDialog = new FileDialog(getShell(), SWT.OPEN);
				String filePath = fileDialog.open();
				if (filePath != null) {
					filePathText.setText(filePath);
				}
			}
		});
						
		return composite;
	}

	protected void buttonPressed(int buttonId) {
		filePath = filePathText.getText();
		switch (fileTypeCombo.getSelectionIndex()) {
			case 0: sourceType = SourceType.POWER_SHELL; break;
			case 1: sourceType = SourceType.EMC_ISILON; break;
			default: sourceType = SourceType.POWER_SHELL; break;
		}
		super.buttonPressed(buttonId);
	}
	
	public String getFilePath() {
		return filePath;
	}
	
	public SourceType getSourceType() {
		return sourceType;
	}
}
