package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.model.DirectoryInfo;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

public class ServerInfoDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(ServerInfoDialog.class.getName());
	
	private Text serverNameText;
	private Spinner numberSubdirectoriesSpinner;
	private Button includeFilesButton;
	private Button includeInheritedPermissionsButton;
	private Button followSymbolicLinksButton;
	private Button scanLocalGroupsButton;
	
	private DirectoryInfo dirInfo;
	private List<DirectoryInfo> existingDirectoyInfoList;
	
	public ServerInfoDialog(Shell shell, DirectoryInfo dirInfo, List<DirectoryInfo> existingDirectoyInfoList) {
		super(shell);
		this.dirInfo = dirInfo;
		this.existingDirectoyInfoList = existingDirectoyInfoList;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.SERVER_16X16));
		shell.setSize(600, 350);
		shell.setText("Server details");
	}
	
	@Override
	protected Control createDialogArea(final Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Server details", ImageFactory.getInstance().getImage(AppImage.SERVER_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.WRAP);
		label.setText("All the network shares of the server will be scanned:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.heightHint = 80;
		label.setLayoutData(gridData);
		
		Label directoryPathLabel = new Label(composite, SWT.NONE);
		directoryPathLabel.setText("Server name:");
		serverNameText = new Text(composite, SWT.BORDER);
		serverNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label numberSubdirectoriesLabel = new Label(composite, SWT.NONE);
		numberSubdirectoriesLabel.setText("Subdirectories depth:");
		Composite numberSubdirectoriesComposite = new Composite(composite, SWT.NONE);
		numberSubdirectoriesComposite.setLayoutData(new GridData());
		gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 10;
		gridLayout.marginLeft = -5;
		gridLayout.marginTop = -5;
		gridLayout.marginBottom = -5;
		numberSubdirectoriesComposite.setLayout(gridLayout);
		numberSubdirectoriesSpinner = new Spinner(numberSubdirectoriesComposite, SWT.BORDER);
		numberSubdirectoriesSpinner.setLayoutData(new GridData());
		numberSubdirectoriesSpinner.setMinimum(0);
		numberSubdirectoriesSpinner.setTextLimit(2);
		numberSubdirectoriesSpinner.setSelection(dirInfo.getNumberOfSubdirectories());
		numberSubdirectoriesSpinner.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
					if (numberSubdirectoriesSpinner.getSelection() == 0 || numberSubdirectoriesSpinner.getSelection() > AclScanner.getNumberOfSubDirsForTrial()) {
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Directory limit", "The trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
						numberSubdirectoriesSpinner.setSelection(AclScanner.getNumberOfSubDirsForTrial());
					}
				}
			}
		});
		
		Label numberSubdirectoriesButtonLabel = new Label(numberSubdirectoriesComposite, SWT.NONE);
		numberSubdirectoriesButtonLabel.setLayoutData(new GridData());
		numberSubdirectoriesButtonLabel.setText("(0 means unlimited)");
				
		Label includeFilesLabel = new Label(composite, SWT.NONE);
		includeFilesLabel.setText("Include files:");
		includeFilesButton = new Button(composite, SWT.CHECK);
		includeFilesButton.setLayoutData(new GridData());
		includeFilesButton.setSelection(dirInfo.isIncludeFiles());
		
		Label includeInheritedPermissionsLabel = new Label(composite, SWT.NONE);
		includeInheritedPermissionsLabel.setText("Include inherited permissions:");
		includeInheritedPermissionsButton = new Button(composite, SWT.CHECK);
		includeInheritedPermissionsButton.setLayoutData(new GridData());
		includeInheritedPermissionsButton.setSelection(dirInfo.isIncludeInheritedPermissions());
		
		Label followSymbolicLinksLabel = new Label(composite, SWT.NONE);
		followSymbolicLinksLabel.setText("Follow symbolic links:");
		followSymbolicLinksButton = new Button(composite, SWT.CHECK);
		followSymbolicLinksButton.setLayoutData(new GridData());
		followSymbolicLinksButton.setSelection(dirInfo.isFollowSymbolicLinks());
		
		Label scanLocalGroupsLabel = new Label(composite, SWT.NONE);
		scanLocalGroupsLabel.setText("Scan local groups on this server:");
		scanLocalGroupsButton = new Button(composite, SWT.CHECK);
		scanLocalGroupsButton.setLayoutData(new GridData());
		scanLocalGroupsButton.setSelection(dirInfo.isFollowSymbolicLinks());
		
		
		
		return composite;
	}

	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			// validate fields
			if (serverNameText.getText().length() == 0) {
				MessageDialog.openError(null, "Details not complete", "Please enter a directory path.");
				return;
			}
			
			// check if the name already exists
			if (dirInfo.getDirectoryId() == 0) {
				for (DirectoryInfo directory : existingDirectoyInfoList) {
					if (directory.getPath().equalsIgnoreCase(serverNameText.getText())) {
						// connection name already exists
						MessageDialog.openError(null, "Server name already exists", "The server name " + serverNameText.getText() + " already exists in the list.");
						return;
					}
				}
			}
		
			dirInfo.setPath(serverNameText.getText());
			dirInfo.setNumberOfSubdirectories(numberSubdirectoriesSpinner.getSelection());
			dirInfo.setIncludeFiles(includeFilesButton.getSelection());
			dirInfo.setIncludeInheritedPermissions(includeInheritedPermissionsButton.getSelection());
			dirInfo.setFollowSymbolicLinks(followSymbolicLinksButton.getSelection());
			dirInfo.setScanLocalGroups(scanLocalGroupsButton.getSelection());
		}
		
		super.buttonPressed(buttonId);
	}

	public DirectoryInfo getDirInfo() {
		return dirInfo;
	}
}
