package com.perdemia.permissionanalyzer.application.jface.dialogs;

import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class TextboxDialog extends DialogWithTopBar {
	private String text;
	private String title;
	private String header;
	private String closeButtonText;
	private String okButtonText;
	private Text contentText;
	
	public TextboxDialog(Shell parentShell, String title, String header, String text) {
		this(parentShell, title, header, text, "Close", null);
	}
	
	public TextboxDialog(Shell parentShell, String title, String header, String text, String closeButtonText, String okButtonText) {
		super(parentShell);
		this.text = text;
		this.title = title;
		this.header = header;
		this.closeButtonText = closeButtonText;
		this.okButtonText = okButtonText;
	}
	
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(title);
		newShell.setSize(1200, 800);
		newShell.setImage(ImageFactory.getInstance().getImage(AppImage.STATUS_TRACE_16X16));
	}
	
	@Override
	protected boolean isResizable()  {
	  return true;
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent, title, ImageFactory.getInstance().getImage(AppImage.INFO_48X48));
		parent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		parent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		composite.setLayout(new GridLayout(1, false));
				
		if (header != null) {
			Link headerLabel = new Link(composite, SWT.NONE);
			headerLabel.setText(header);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			headerLabel.setLayoutData(gridData);
			
			headerLabel.addListener(SWT.Selection,new Listener(){
			    public void handleEvent(Event event) {
			    	ApplicationUtil.setBusyCursor();
			    	Program.launch(event.text);
			    	ApplicationUtil.setDefaultCursor();
			    }
			});
		}			
		
		contentText = new Text(composite, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.READ_ONLY | SWT.WRAP);
		contentText.setText(text);
		contentText.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		return composite;		
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.SELECT_ALL_ID, "Copy to clipboard", false);
		createButton(parent, IDialogConstants.CLOSE_ID, closeButtonText, false);
		
		if (okButtonText != null) {
			createButton(parent, IDialogConstants.OK_ID, okButtonText, false);
		}
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		super.buttonPressed(buttonId);
		if (buttonId == IDialogConstants.CLOSE_ID) {
			cancelPressed();
		}
		
		if (buttonId == IDialogConstants.SELECT_ALL_ID) {
			Clipboard clipboard = new Clipboard(Display.getDefault());
			TextTransfer textTransfer = TextTransfer.getInstance();
	        clipboard.setContents(new String[]{contentText.getText()}, new Transfer[]{textTransfer});
	        clipboard.dispose();
		}
	}
}