package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class WildcardInputDialog extends InputDialog {
	public final static int ADD_TO_INCLUDE_ID = 3;
	public final static int ADD_TO_EXCLUDE_ID = 4;
	private String value;
	private boolean useIncludeButton = true;
	private boolean useExcludeButton = true;
	private boolean useForFolderFilter = false;
	private boolean applyFilterAsANDOpertator = false;
	private boolean includeSubfolders = false;
	
	private Button applyFilterAsANDOpertatorButton;
	private Button includeSubfoldersButton;
	
	public WildcardInputDialog(Shell parentShell, String dialogTitle,
            String dialogMessage, boolean useForFolderFilter){
		super(parentShell, dialogTitle, dialogMessage, "", null);
		this.useForFolderFilter = useForFolderFilter;
	}
	
	public WildcardInputDialog(Shell parentShell, String dialogTitle,
            String dialogMessage, boolean useIncludeButton, boolean useExcludeButton){
		super(parentShell, dialogTitle, dialogMessage, "", null);
		this.useIncludeButton = useIncludeButton;
		this.useExcludeButton = useExcludeButton;
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		parent.getShell().setImage(ImageFactory.getInstance().getImage(AppImage.ADD_WILDCARD_16X16));
		return (Composite) super.createDialogArea(parent);		
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		if (useForFolderFilter) {
			Composite checkboxesComposite = new Composite(parent, SWT.NONE);
			checkboxesComposite.setLayout(new GridLayout(1, false));
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 5;
			checkboxesComposite.setLayoutData(gridData);
			((GridLayout) parent.getLayout()).numColumns++;
			applyFilterAsANDOpertatorButton = new Button(checkboxesComposite, SWT.CHECK);
			applyFilterAsANDOpertatorButton.setText("Apply this wildcard filter only on the result of existing Include filters, instead of the entire directory tree.");
			applyFilterAsANDOpertatorButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					applyFilterAsANDOpertator = applyFilterAsANDOpertatorButton.getSelection();
				}
			});
			
			includeSubfoldersButton = new Button(checkboxesComposite, SWT.CHECK);
			includeSubfoldersButton.setText("Include/exclude subfolders as well");
			includeSubfoldersButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					includeSubfolders = includeSubfoldersButton.getSelection();
				}
			});
		}
		
		if (useIncludeButton) {
			Button includeButton = createButton(parent, ADD_TO_INCLUDE_ID, "Include wildcard", true);
			includeButton.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16));
			((GridData) includeButton.getLayoutData()).widthHint = 180;
			includeButton.pack(true);
		}
		
		if (useExcludeButton) {
			Button excludeButton = createButton(parent, ADD_TO_EXCLUDE_ID, "Exclude wildcard", false);
			excludeButton.setImage(ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16));
			((GridData) excludeButton.getLayoutData()).widthHint = 180;
			excludeButton.pack(true);
		}
		
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
	
	protected void buttonPressed(int buttonId) {
		if ( getText().getText().contains("\\")) {
			if (!MessageDialog.openConfirm(null, "Domain name", "This search text seems to contain a domain name. The filter is only applied on account name, are you sure you want to use this filter?")) {
				return;
			}
		}
		if (buttonId == ADD_TO_INCLUDE_ID) {
			// include button
			value = getText().getText();
			setReturnCode(ADD_TO_INCLUDE_ID);
			close();
		} else if (buttonId == ADD_TO_EXCLUDE_ID) {
			// exclude button
			 value = getText().getText();
			setReturnCode(ADD_TO_EXCLUDE_ID);
			close();
		} else {
			super.buttonPressed(buttonId);
		}
	}
	
	@Override
	public String getValue() {
		return value;
	}

	public boolean isApplyFilterAsANDOpertator() {
		return applyFilterAsANDOpertator;
	}

	public boolean isIncludeSubfolders() {
		return includeSubfolders;
	}
}
