package com.perdemia.permissionanalyzer.application.jface.labelproviders;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;

import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;

public class PathColumnLabelProvider extends ColumnLabelProvider {
	private boolean useFullPath;
	
	public PathColumnLabelProvider(boolean useFullPath) {
		this.useFullPath = useFullPath;
	}
	
	@Override
	public String getText(Object element) {
		if (element instanceof FileInfo) {
			if (useFullPath) {
				return ((FileInfo) element).determineFullPath();
			} else {
				return ((FileInfo) element).getName() + " "; // some file names are cropped if we don't add a space. Probably the required space for the label is not computed correct by SWT?
			}
		} else {
			return "Unknown type";
		}
	}
	
	@Override
	public Image getImage(Object element) {
		return ImageFactory.getInstance().getImage(PermissionUtil.getFileImage(null, (FileInfo) element));
	}
	
	@Override
	public Color getForeground(Object element) {
		if (element instanceof FileInfo) {
			FileInfo file = (FileInfo) element;
			
			boolean allImplicit = true;
			if (file.getEffectivePermissions() != null) {
				for (AclItem aclItem : file.getEffectivePermissions().getPermissions().values()) {
					allImplicit = allImplicit && aclItem.isFlagInheritedAce();
				}
			}
			
			if (allImplicit) {
				return ColorFactory.getInstance().getColor(AppColor.GREY);
			}
		}
		
		return null;
	}
}