package com.perdemia.permissionanalyzer.application.jface.wizard;

import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.jface.dialogs.DialogWithTopBar;

public abstract class WizardDialogWithTopBar extends DialogWithTopBar {
	private Button nextButton;
	private Button previousButton;
	private Button finishButton;
	private int index;
	private List<WizardDialogPage> pages;
	private String dialogTitle;
	private Image dialogImage;
	private StackLayout layout;
	private Composite dialogArea;
	private boolean includeCancel = false;
	
	public WizardDialogWithTopBar(Shell shell, Image image, String title, boolean includeCancel) {
		super(shell);
		this.dialogTitle = title;
		this.dialogImage = image;
		this.includeCancel = includeCancel;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setSize(getWidth(), getHeight());
		shell.setImage(dialogImage);
		shell.setText(dialogTitle);
	}
	
	protected int getWidth() {
		return 1000;
	}
	
	protected int getHeight() {
		return 800;
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		dialogArea = (Composite) super.createDialogArea(container, getPageTitle(0), getPageImage(0));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		layout = new StackLayout();
		dialogArea.setLayout(layout);
		
		pages = createPages(dialogArea);
		
		for (WizardDialogPage page : pages) {
			page.setWizardDialog(this);
		}
				
		return dialogArea;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
		
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 6;
		separator.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		Button additionalButton = getAdditionalButton(parent);
		if (additionalButton != null) {
			((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		}
				
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		previousButton = createButton(parent, IDialogConstants.BACK_ID, "< Previous", true);
		previousButton.setEnabled(false);
		nextButton = createButton(parent, IDialogConstants.NEXT_ID, "Next >", true);
		
		if (includeCancel) {
			finishButton = createButton(parent, IDialogConstants.FINISH_ID, "Finish", true);
			createButton(parent, IDialogConstants.CANCEL_ID, "Cancel", false);
		} else {
			createButton(parent, IDialogConstants.CLOSE_ID, "Close", true);
		}
		
		showPage(0, false);
	}
	
	protected Button getAdditionalButton(Composite parent) {
		return null;
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		switch (buttonId) {
			case IDialogConstants.BACK_ID:
				previous();
				break;
				
			case IDialogConstants.NEXT_ID:
				next();			
				break;
				
			case IDialogConstants.CLOSE_ID:
				setReturnCode(IDialogConstants.CLOSE_ID);
				close();
				return;
			
			case IDialogConstants.FINISH_ID:
				if (doFinish()) {
					setReturnCode(IDialogConstants.FINISH_ID);
					close();
					return;
				}
				
			case IDialogConstants.CANCEL_ID:
				super.buttonPressed(buttonId);
				return;
		}
	}
	
	protected void showPage(int index, boolean previousClicked) {
		setTitle(getPageTitle(index));
		setImage(getPageImage(index));
		
		// show the right page control
		layout.topControl = pages.get(index);
		dialogArea.layout();
		
		enableNext(index != pages.size() - 1);
		enablePrevious(index != 0);
		
		if (finishButton != null) {
			enableFinish(index == pages.size() - 1);
		}
		
		pages.get(index).pageDisplayed(previousClicked);
	}
	
	public void enableNext(boolean enable) {
		nextButton.setEnabled(enable);
	}
	
	public void enablePrevious(boolean enable) {
		previousButton.setEnabled(enable);
	}
	
	public void enableFinish(boolean enable) {
		finishButton.setEnabled(enable);
	}
	
	public List<WizardDialogPage> getPages() {
		return pages;
	}
	
	protected boolean doFinish() {
		return true;
	}
	
	public void next() {
		index++;
		showPage(index, false);
	}
	
	public void previous() {
		index--;
		showPage(index, true);
	}
	
	public abstract List<WizardDialogPage> createPages(Composite parent);
	public abstract String getPageTitle(int index);
	public abstract Image getPageImage(int index);
	
}
