package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogWithTopBar;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Report;

public class CreatePolicyWizardDialog extends WizardDialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(CreatePolicyWizardDialog.class.getName());
	
	public CreatePolicyWizardDialog(Shell shell) {
		super(shell, ImageFactory.getInstance().getImage(AppImage.POLICY_16X16), "Create policy", true);
	}
	
	@Override
	protected int getWidth() {
		return 700;
	}
	
	protected int getHeight() {
		return 700;
	}

	@Override
	public List<WizardDialogPage> createPages(Composite parent) {
		List<WizardDialogPage> pages = new ArrayList<WizardDialogPage>();
		pages.add(new PagePolicyType(parent, SWT.NONE));
		pages.add(new PageMemberFilters(parent, SWT.NONE));
		pages.add(new PagePermissionFilters(parent, SWT.NONE));
		pages.add(new PageFolderFilters(parent, SWT.NONE));
		pages.add(new PagePolicyReportSettings(parent, SWT.NONE));
		pages.add(new PagePolicySummary(parent, SWT.NONE));
		
		return pages;
	}

	@Override
	public String getPageTitle(int index) {
		switch (index) {
			case 0: return "Policy type";
			case 1: return "Member filters";
			case 2: return "Permission filters";
			case 3: return "Folder filters";
			case 4: return "Policy output settings";
			case 5: return "Policy summary";
			default: return null;
		}
	}

	@Override
	public Image getPageImage(int index) {
		AppImage image;
		
		switch (index) {
			case 0: image = AppImage.POLICY_48X48; break;
			case 1: image = AppImage.USER_FILTER_48X48; break;
			case 2: image = AppImage.ACL_48X48; break;
			case 3: image = AppImage.FOLDER_48X48; break;
			case 4: image = AppImage.REPORT_48X48; break;
			case 5: image = AppImage.POLICY_48X48; break;
			default: return null;
		}
		
		return ImageFactory.getInstance().getImage(image);
	}

	@Override
	public boolean doFinish() {
		try {
			Report report = ((PagePolicyReportSettings) getPages().get(4)).buildReportObject();
			report.save();
			
			// save the related filters
			List<Filter> allFilters = new ArrayList<Filter>();
			allFilters.addAll(report.getMemberFilters());
			allFilters.addAll(report.getPermissionFilters());
			allFilters.addAll(report.getFolderFilters());
			
			for (Filter filter : allFilters) {
				filter.setReportId(report.getId());
				filter.save();
			}
			
			return true;
		} catch (Exception ex) {
			logger.error("Error while creating the policy", ex);
			MessageDialog.openError(getShell(), "Error", "Error while creating the policy: " + ex.getMessage());
		}
		return false;
	}
}