package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchFolderDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchMembersDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.WildcardInputDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.policy.PagePolicyType.PolicyType;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;

/**
 * Page that takes care of selecting the right folder filters for the corresponding policy type in page one.
 * 
 * @author Reinier
 * 
 */
class PageFolderFilters extends WizardDialogPage {
	private ArrayList<Filter> folderFilters = new ArrayList<Filter>();
	private TableViewer folderFilterListViewer;
	private Label explainLabel;
	private Button includeFilesButton;
	private Button excludeFoldersButton;
		
	public PageFolderFilters(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 20;
		setLayout(gridLayout);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("You may limit your policy to specific folders or you can exclude folders from your policy. Select the folders you want to add as filter, if you do not select a folder, then the policy applies to all folders.");
		
		// button bar
		Composite buttonBar = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout(4, false);
		layout.marginLeft = -5;
		layout.marginTop = 10;
		buttonBar.setLayout(layout);
		buttonBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
		//search folders
		excludeFoldersButton = new Button(buttonBar, SWT.PUSH);
		excludeFoldersButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		excludeFoldersButton.setText("Select folders");
		excludeFoldersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		excludeFoldersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {				
				SearchFolderDialog searchDialog = new SearchFolderDialog(Display.getCurrent().getActiveShell(), folderFilterListViewer, true, true);
				searchDialog.open();
			}
		});
				
		// Add wildcard value button
		Button addWildCardToMemberListButton = new Button(buttonBar, SWT.PUSH);
		addWildCardToMemberListButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_WILDCARD_16X16));
		addWildCardToMemberListButton.setText("Add wildcard filter");
		addWildCardToMemberListButton.setToolTipText("Add wildcard search text");
		addWildCardToMemberListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		addWildCardToMemberListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean useInclude = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) || getPolicyType().equals(PolicyType.NO_PARTICULAR_MEMBERS);
				
				InputDialog inputDialog = new WildcardInputDialog(
						Display.getCurrent().getActiveShell(),
						"Add wildcard filter on folder or file name",
						"Please enter a wildcard value using the * character. Wildcard values will be used to filter on file or folder name and are not case-sensitive.\r\n\r\nExample: project* or *.exe\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.",
						useInclude, !useInclude);
				int result = inputDialog.open();
				if (result == WildcardInputDialog.ADD_TO_INCLUDE_ID || result == WildcardInputDialog.ADD_TO_EXCLUDE_ID){
					if (!inputDialog.getValue().contains("%") && !inputDialog.getValue().contains("*")) {
						MessageDialog.openWarning(Display.getCurrent().getActiveShell(), "Invalid value", "The search text contains no wildcards, please use the search button to filter on specific folder or file names.");
					} else {
						folderFilters.add(new Filter(Filter.FilterType.FOLDER_NAME, result == WildcardInputDialog.ADD_TO_INCLUDE_ID, inputDialog.getValue(), inputDialog.getValue()));
						folderFilterListViewer.refresh();
					}
				}
			}
		});
		
		// Remove from folder list
		Button removeFromFolderListButton = new Button(buttonBar, SWT.PUSH);
		removeFromFolderListButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeFromFolderListButton.setText("Remove selected filters");
		removeFromFolderListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		removeFromFolderListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) folderFilterListViewer.getSelection();
				for (Iterator iterator = selection.iterator(); iterator.hasNext();) {
					folderFilters.remove(iterator.next());
				}
				folderFilterListViewer.refresh();
			}
		});
		
		//create a one column TableViewer so an image can be used
		folderFilterListViewer = new TableViewer(this, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION);
		gridData = new GridData(GridData.FILL_BOTH);
		folderFilterListViewer.getTable().setLayoutData(gridData);
		
		TableViewerColumn folderColumn = new TableViewerColumn(folderFilterListViewer, SWT.NONE);
		folderColumn.getColumn().setWidth(600);
				
		folderFilterListViewer.setContentProvider(ArrayContentProvider.getInstance());
		folderFilterListViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					Filter filter = (Filter) element;
					if (filter.getType().equals(Filter.FilterType.FOLDER_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.FOLDER_PATH)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_FOLDER_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_FOLDER_16X16);
						}
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Filter filter = (Filter) element;
				switch (columnIndex) {
					case 0: return filter.getName();
					default: return "";
				}
			}
		});
				
		folderFilterListViewer.setInput(folderFilters);

		includeFilesButton = new Button(this, SWT.CHECK);
		includeFilesButton.setText("Include files");
		includeFilesButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_20X20));
		includeFilesButton.setToolTipText("Include files");
		includeFilesButton.setSelection(true);
	}
	
	private PolicyType getPolicyType() {
		return ((PagePolicyType)wizardDialog.getPages().get(0)).getSelectedPolicyType();
	}

	public ArrayList<Filter> getFolderFilters() {
		return folderFilters;
	}

	public Button getIncludeFilesButton() {
		return includeFilesButton;
	}
}
