package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportPreviewDialog;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.Report.FileType;
import com.perdemia.permissionanalyzer.data.Report.ReportType;

/**
 * Page that takes care of selecting all the policy output settings, like the file type, report type, e-mail address and threshold value.
 * 
 * @author Reinier
 * 
 */
class PagePolicyReportSettings extends WizardDialogPage {
	private static Logger logger = LoggerFactory.getLogger(PagePolicyReportSettings.class.getName());
	
	private Label explainLabel;
	private Combo reportTypeCombo;
	private Combo fileTypeCombo;
	private Text filePathText;
	private Text emailAddressText;
	private Spinner emailThresholdSpinner;
		
	public PagePolicyReportSettings(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 20;
		setLayout(gridLayout);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("Configure the output format in case you want to export the policy results to a file each time the policy runs:");
		
		// spacer
		Composite spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 10;
		gridData.horizontalSpan = 2;
		spacer.setLayoutData(gridData);
				
		int labelWidth = 230;
		
		Label reportTypeLabel = new Label(this, SWT.NONE);
		reportTypeLabel.setText("Report type:");
		gridData = new GridData();
		gridData.widthHint = labelWidth;
		reportTypeLabel.setLayoutData(gridData);
		
		reportTypeCombo = new Combo(this, SWT.BORDER | SWT.READ_ONLY);
		reportTypeCombo.add(Report.reportTypeToString(ReportType.EFFECTIVE));
		reportTypeCombo.add(Report.reportTypeToString(ReportType.FOLDERS_AND_USERS_EFFECTIVE));
		reportTypeCombo.add(Report.reportTypeToString(ReportType.ACL));
		reportTypeCombo.add(Report.reportTypeToString(ReportType.ACL_DIRECT_GROUP_MEMBERS));
		reportTypeCombo.add(Report.reportTypeToString(ReportType.ACL_NESTED_GROUP_MEMBERS));
		reportTypeCombo.add(Report.reportTypeToString(ReportType.ALL_MEMBERS));
		reportTypeCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		reportTypeCombo.select(0);
				
		Label fileTypeLabel = new Label(this, SWT.NONE);
		fileTypeLabel.setText("File type:");
		gridData = new GridData();
		gridData.widthHint = labelWidth;
		fileTypeLabel.setLayoutData(gridData);
		
		fileTypeCombo = new Combo(this, SWT.BORDER | SWT.READ_ONLY);
		fileTypeCombo.add(Report.fileTypeToString(FileType.HTML));
		fileTypeCombo.add(Report.fileTypeToString(FileType.CSV));
		fileTypeCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fileTypeCombo.select(0);
		
		Composite filePathComposite = new Composite(this, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		filePathComposite.setLayoutData(gridData);
		filePathComposite.setLayout(new GridLayout(4, false));
		
		Label filePathLabel = new Label(filePathComposite, SWT.NONE);
		filePathLabel.setText("Target file path:");
		filePathLabel.setToolTipText("Use a date placeholder to use the current date in the file path, like [date:yyyy-MM-dd HHmm]");
		gridData = new GridData();
		gridData.widthHint = labelWidth;
		gridData.horizontalIndent = -5;
		filePathLabel.setLayoutData(gridData);
		
		filePathText = new Text(filePathComposite, SWT.BORDER);
		filePathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		filePathText.setToolTipText("Use a date placeholder to use the current date in the file path, like [date:yyyy-MM-dd HHmm]");
				
		Button insertDateButton = new Button(filePathComposite, SWT.PUSH);
		insertDateButton.setImage(ImageFactory.getInstance().getImage(AppImage.DATE_16X16));
		insertDateButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
		    	String prefix = filePathText.getText().substring(0, filePathText.getCaretPosition());
		    	String suffix = filePathText.getText().substring(filePathText.getCaretPosition());
				filePathText.setText(prefix + "[date:yyyy-MM-dd HHmm]" + suffix);
			}
		});
		
		Button browseFileButton = new Button(filePathComposite, SWT.PUSH);
		browseFileButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		browseFileButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				FileDialog dialog = new FileDialog(getShell(), SWT.SAVE);
			    
			    if (fileTypeCombo.getText().equals(Report.fileTypeToString(FileType.CSV))) {
			    	dialog.setFilterNames(new String[] { "CSV Files", "All Files (*.*)" });
			    	dialog.setFilterExtensions(new String[] { "*.csv", "*.*" });
			    } else {
			    	dialog.setFilterNames(new String[] { "HTML Files", "All Files (*.*)" });
			    	dialog.setFilterExtensions(new String[] { "*.html", "*.*" });
			    }
			    String filePath = dialog.open();
			    if (filePath != null) {
			    	filePathText.setText(filePath);
			    }
			}
		});
		

		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		gridData.horizontalSpan = 2;
		spacer.setLayoutData(gridData);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("In the event that the policy contains unwanted permissions, the application can send an e-mail to the configured address. The e-mail template can be customized in the application preferences.");
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 5;
		gridData.horizontalSpan = 2;
		spacer.setLayoutData(gridData);
		
		Label emailAddressLabel = new Label(this, SWT.NONE);
		emailAddressLabel.setText("E-mail recipient:");
		emailAddressLabel.setToolTipText("An e-mail will be sent after the report export.");
		gridData = new GridData();
		gridData.widthHint = labelWidth;
		emailAddressLabel.setLayoutData(gridData);
		
		emailAddressText = new Text(this, SWT.BORDER);
		emailAddressText.setToolTipText("An e-mail will be sent after the report export.");
		emailAddressText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		gridData.horizontalSpan = 2;
		spacer.setLayoutData(gridData);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("A policy displays unwanted permissions and fails when any results are found. You can increase the threshold value when a policy should fail:");
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 5;
		gridData.horizontalSpan = 2;
		spacer.setLayoutData(gridData);
		
		Composite thresholdComposite = new Composite(this, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		thresholdComposite.setLayoutData(gridData);
		thresholdComposite.setLayout(new GridLayout(3, false));
		
		Label emailThresholdLabel = new Label(thresholdComposite, SWT.NONE);
		emailThresholdLabel.setText("Policy alert threshold (file/folder count):   ");
		emailThresholdLabel.setToolTipText("Enter the minimal file/folder count that determines whether or not a policy has failed.");
		gridData = new GridData();
		gridData.widthHint = labelWidth;
		emailThresholdLabel.setLayoutData(gridData);
		
		emailThresholdSpinner = new Spinner(thresholdComposite, SWT.BORDER);
		emailThresholdSpinner.setMinimum(1);
		emailThresholdSpinner.setMaximum(Integer.MAX_VALUE);
		gridData = new GridData();
		gridData.widthHint = 50;
		emailThresholdSpinner.setLayoutData(gridData);
		emailThresholdSpinner.setToolTipText("Enter the minimal file/folder count that determines whether or not a policy has failed.");
		
		Button previewButton = new Button(thresholdComposite, SWT.PUSH);
		previewButton.setImage(ImageFactory.getInstance().getImage(AppImage.POLICY_16X16));
		previewButton.setText("Preview policy results");
		gridData = new GridData();
		gridData.horizontalIndent = 10;
		previewButton.setLayoutData(gridData);
		previewButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				Report report = buildReportObject();
				try {
					ReportPreviewDialog dialog = new ReportPreviewDialog(getShell(), report.getMemberFilters(), report.getPermissionFilters(), report.getFolderFilters(), true);
					dialog.open();
				} catch (DBException ex) {
					logger.error("Error while building the policy preview", ex);
					MessageDialog.openError(getShell(), "Error", "Error while building the policy preview: " + ex.getMessage());
				}
			}
		});
	}
	
	public String getSelectedReportType() {
		return reportTypeCombo.getItem(reportTypeCombo.getSelectionIndex());
	}
	
	public String getSelectedFileType() {
		return fileTypeCombo.getItem(fileTypeCombo.getSelectionIndex());
	}

	public Text getFilePathText() {
		return filePathText;
	}

	public Text getEmailAddressText() {
		return emailAddressText;
	}

	public Spinner getEmailThresholdSpinner() {
		return emailThresholdSpinner;
	}
	
	/**
	 * Builds a new Report object based on the total list of all filters and settings found in the other pages.
	 * @throws DBException 
	 */
	@SuppressWarnings("unchecked")
	public Report buildReportObject() {
		PagePolicyType policyTypePage = (PagePolicyType) wizardDialog.getPages().get(0);
		PageMemberFilters memberPage = (PageMemberFilters) wizardDialog.getPages().get(1);
		PagePermissionFilters permissionPage = (PagePermissionFilters) wizardDialog.getPages().get(2);
		PageFolderFilters folderPage = (PageFolderFilters) wizardDialog.getPages().get(3);
		PagePolicyReportSettings reportPage = (PagePolicyReportSettings) wizardDialog.getPages().get(4);
		
		Report report = new Report();
		report.setName(policyTypePage.getName());
		report.setDescription(policyTypePage.getDescription());
		
		// member filters
		report.setMemberFilters((ArrayList<Filter>) memberPage.getMemberFilters().clone());
		
		// permission filters
		List<Filter> permissionFilters = new ArrayList<Filter>();
		boolean onlyAllowOrDeny = permissionPage.getAllowCheckbox().getSelection() != permissionPage.getDenyCheckbox().getSelection();
		for (int counter = 0; counter < permissionPage.getPermissionCheckboxes().size(); counter++){
			// permission checked?
			if (permissionPage.getPermissionCheckboxes().get(counter).getSelection()) {
				if (onlyAllowOrDeny) {
					if (permissionPage.getAllowCheckbox().getSelection()) {
						permissionFilters.add(new Filter(FilterType.PERMISSION_ALLOW, true, String.valueOf(counter), "Index of the selected permission"));
					} else {
						permissionFilters.add(new Filter(FilterType.PERMISSION_DENY, true, String.valueOf(counter), "Index of the selected permission"));
					}
				} else {
					// both allow and deny
					permissionFilters.add(new Filter(FilterType.PERMISSION_ALLOW, true, String.valueOf(counter), "Index of the selected permission"));
					permissionFilters.add(new Filter(FilterType.PERMISSION_DENY, true, String.valueOf(counter), "Index of the selected permission"));
				}
			}
		}
		
		// all permissions checked?
		if (permissionFilters.size() == 26) {
			// don't filter, any permission should be in the report
			permissionFilters.clear();
		}
		
		report.setPermissionFilters(permissionFilters);
		
		// folder filters
		report.setFolderFilters((ArrayList<Filter>) folderPage.getFolderFilters().clone());
		
		// Output settings
		for (ReportType reportType : ReportType.values()) {
			if (Report.reportTypeToString(reportType).equals(reportPage.getSelectedReportType())) {
				report.setReportType(reportType);
				break;
			}
		}
		
		for (FileType fileType : FileType.values()) {
			if (Report.fileTypeToString(fileType).equals(reportPage.getSelectedFileType())) {
				report.setFileType(fileType);
			}
		}
		
		// check if the target path has an extension
		if (report.getFileType().equals(FileType.HTML) || report.getFileType().equals(FileType.HTML_JQUERY)) {
			if (!reportPage.getFilePathText().getText().isEmpty() && !reportPage.getFilePathText().getText().toLowerCase().endsWith(".htm") && !reportPage.getFilePathText().getText().toLowerCase().endsWith(".html")) {
				reportPage.getFilePathText().setText(reportPage.getFilePathText().getText() + ".html");
			}
		} else {
			if (!reportPage.getFilePathText().getText().isEmpty() && !reportPage.getFilePathText().getText().toLowerCase().endsWith(".csv")) {
				reportPage.getFilePathText().setText(reportPage.getFilePathText().getText() + ".csv");
			}
		}
		
		report.setFilePath(reportPage.getFilePathText().getText());
		report.setEmailAddress(reportPage.getEmailAddressText().getText());
		report.setEmailThreshold(reportPage.getEmailThresholdSpinner().getSelection());
		
		// Filter config
		FilterConfig filterConfig = new FilterConfig(new Filter(FilterType.MEMBER_FILTER_CONFIG, true, "00000", "Configuration for the member filters"),
				new Filter(FilterType.PERMISSION_FILTER_CONFIG, true, "00000", "Configuration for the permissions filters"),
				new Filter(FilterType.FOLDER_FILTER_CONFIG, true, "00000", "Configuration for the file filters"));
		
		filterConfig.setAceNeedAllPermissions(false);
		filterConfig.setHideInheritedPermissions(permissionPage.getHideInheritedPermissionsButton().getSelection());
		filterConfig.setIncludeFolders(true);
		filterConfig.setIncludeFiles(folderPage.getIncludeFilesButton().getSelection());
		filterConfig.setIncludePermissions(true);
		filterConfig.setIncludeBuiltIn(true);
		filterConfig.setIncludeADGroupsAndUsers(true);
		filterConfig.setIncludeDisabledUsers(true);
		filterConfig.setIncludeGroupMembership(memberPage.getIncludeGroupMembershipButton().getSelection());
		filterConfig.setIncludeGroups(memberPage.getIncludeGroupsButton().getSelection());
		filterConfig.setIncludeUsers(memberPage.getIncludeUsersButton().getSelection());
		
		try {
			report.getMemberFilters().add(filterConfig.getMemberConfigFilter());
			report.getPermissionFilters().add(filterConfig.getPermissionConfigFilter());
			report.getFolderFilters().add(filterConfig.getFolderConfigFilter());
		} catch (DBException e) {
			logger.error("Error while building the policy filters", e);
			MessageDialog.openError(getShell(), "Error", "Error while building the policy filters: " + e.getMessage());
		}
		
		return report;
	}
}
