package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Tree;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.FilterRow;
import com.perdemia.permissionanalyzer.application.export.ReportFilters;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.ReportFilterContentProvider;
import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDetailsDialog.ReportDialogMode;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.policy.PagePolicyType.PolicyType;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.Report.FileType;
import com.perdemia.permissionanalyzer.data.Report.ReportType;

/**
 * Page that takes care of selecting the right member filters for the corresponding policy type in page one.
 * 
 * @author Reinier
 * 
 */
class PagePolicySummary extends WizardDialogPage {
	private static Logger logger = LoggerFactory.getLogger(PagePolicySummary.class.getName());
	private TreeViewer filterTreeViewer;
	private Label titleValueLabel;
	private Label descriptionValueLabel;
	private Label reportTypeValueLabel;
	private Label fileTypeValueLabel;
	private Label filePathValueLabel;
	private Label emailValueLabel;
	private Label thresholdValueLabel;
	
	public PagePolicySummary(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 10;
		gridLayout.horizontalSpacing = 30;
		gridLayout.verticalSpacing = 7;
		setLayout(gridLayout);
		
		// explain text
		Label explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("This page shows the summary of your policy with all the filters that will be created:");
		
		// spacer
		Composite spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.heightHint = 10;
		spacer.setLayoutData(gridData);
		
		Label titleLabel = new Label(this, SWT.NONE);
		titleLabel.setText("Policy title: ");
		titleValueLabel = new Label(this, SWT.WRAP);
		titleValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label descriptionLabel = new Label(this, SWT.NONE);
		descriptionLabel.setText("Policy description: ");
		descriptionValueLabel = new Label(this, SWT.WRAP);
		descriptionValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label reportTypeLabel = new Label(this, SWT.NONE);
		reportTypeLabel.setText("Report type: ");
		reportTypeValueLabel = new Label(this, SWT.WRAP);
		reportTypeValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label fileTypeLabel = new Label(this, SWT.NONE);
		fileTypeLabel.setText("File type: ");
		fileTypeValueLabel = new Label(this, SWT.WRAP);
		fileTypeValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label filePathLabel = new Label(this, SWT.NONE);
		filePathLabel.setText("Target file path: ");
		filePathValueLabel = new Label(this, SWT.WRAP);
		filePathValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label emailLabel = new Label(this, SWT.NONE);
		emailLabel.setText("E-mail recipient: ");
		emailValueLabel = new Label(this, SWT.WRAP);
		emailValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label thresholdLabel = new Label(this, SWT.NONE);
		thresholdLabel.setText("Policy threshold: ");
		thresholdValueLabel = new Label(this, SWT.WRAP);
		thresholdValueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.heightHint = 10;
		spacer.setLayoutData(gridData);
				
		Label filtersLabel = new Label(this, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		filtersLabel.setLayoutData(gridData);
		filtersLabel.setText("Filters:");
		
		filterTreeViewer = new TreeViewer(this, SWT.BORDER | SWT.FULL_SELECTION);
		filterTreeViewer.getTree().setLinesVisible(true);
		filterTreeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 2;
		filterTreeViewer.getTree().setLayoutData(gridData);
		filterTreeViewer.setLabelProvider(new LabelProvider(){
			@Override
			public String getText(Object element) {
				return ((FilterRow) element).getLabel();
			}
			
			@Override
			public Image getImage(Object element) {
				return ImageFactory.getInstance().getImage(((FilterRow) element).getImage());
			}
		});
		
		filterTreeViewer.setContentProvider(new ReportFilterContentProvider());
	}
	
	@Override
	public void pageDisplayed(boolean previousClicked) {
		Report report = ((PagePolicyReportSettings) wizardDialog.getPages().get(4)).buildReportObject();
		ReportFilters reportFilters = new ReportFilters();
		
		try {
			reportFilters.setMemberFilters(report.getMemberFilters());
			reportFilters.setPermissionFilters(report.getPermissionFilters());
			reportFilters.setFolderFilters(report.getFolderFilters());
		} catch (DBException e) {
			logger.error("Error while building the summary page", e);
			MessageDialog.openError(getShell(), "Error", "Error while building the summary page: " + e.getMessage());
		}
		
		titleValueLabel.setText(report.getName());
		descriptionValueLabel.setText(report.getDescription());
		reportTypeValueLabel.setText(Report.reportTypeToString(report.getReportType()));
		fileTypeValueLabel.setText(Report.fileTypeToString(report.getFileType()));
		filePathValueLabel.setText(report.getFilePath());
		emailValueLabel.setText(report.getEmailAddress());
		thresholdValueLabel.setText(String.valueOf(report.getEmailThreshold()));
		
		titleValueLabel.getParent().layout(true);
		filterTreeViewer.setInput(reportFilters);	
	}
}
