package com.perdemia.permissionanalyzer.application.model;

import java.io.File;

public class DirectoryInfo {
	private int directoryId = -1;
	private String path = "";
	private int numberOfSubdirectories = 0;
	private boolean includeFiles = true;
	private boolean includeInheritedPermissions = true;
	private boolean followSymbolicLinks = true;
	private boolean scanLocalGroups = true;
	
	/** Setting used when a server is added to the list of directories */
	private boolean includeHiddenSharedDrives = false;
	
	/** Setting used when a server is added to the list of directories */
	private boolean includeHiddenShares = false;
		
	/** Comma separated list of shares that is excluded when a server is scanned */
	private String excludeShares = "IPC$, PRINT$, ADMIN$";
	
	/** Login name used to retrieve the server shares */
	private String serverLoginUsername;
	
	/** Login password used to retrieve the server shares */
	private String serverLoginPassword;
	
	/** Login domain used to retrieve the server shares */
	private String serverLoginDomain;
	
	private SourceType sourceType;
	
	public enum SourceType {
		/** Local directory or network share */
		DIRECTORY,
		/** PowerShell */
		POWER_SHELL,
		/** EMC Isilon NAS */
		EMC_ISILON,
		/** A server name */
		SERVER
	}
	
	/** A DirectoryInfo object (configured in the Scan View) can be temporarily excluded from the scan */
	private boolean enabled = true;
	
	public String getPath() {
		return path;
	}
	public void setPath(String path) {
		// when "c:" is being scanned, then only the application folder will be retrieved. Always use "c:\", including the backslash
		if (path.endsWith(":")) {
			path += File.separator;
		}
		this.path = path;
	}
	public int getNumberOfSubdirectories() {
		return numberOfSubdirectories;
	}
	public void setNumberOfSubdirectories(int numberOfSubdirectories) {
		this.numberOfSubdirectories = numberOfSubdirectories;
	}
	public int getDirectoryId() {
		return directoryId;
	}
	public void setDirectoryId(int directoryId) {
		this.directoryId = directoryId;
	}
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("DirectoryInfo [directoryId=");
		builder.append(directoryId);
		builder.append(", path=");
		builder.append(path);
		builder.append(", numberOfSubdirectories=");
		builder.append(numberOfSubdirectories);
		builder.append(", includeFiles=");
		builder.append(includeFiles);
		builder.append(", includeInheritedPermissions=");
		builder.append(includeInheritedPermissions);
		builder.append(", followSymbolicLinks=");
		builder.append(followSymbolicLinks);
		builder.append(", scanLocalGroups=");
		builder.append(scanLocalGroups);
		builder.append(", includeHiddenSharedDrives=");
		builder.append(includeHiddenSharedDrives);
		builder.append(", excludeShares=");
		builder.append(excludeShares);
		builder.append(", serverLoginUsername=");
		builder.append(serverLoginUsername);
		builder.append(", serverLoginDomain=");
		builder.append(serverLoginDomain);
		builder.append(", enabled=");
		builder.append(enabled);
		builder.append("]");
		return builder.toString();
	}
	public boolean isIncludeFiles() {
		return includeFiles;
	}
	public void setIncludeFiles(boolean includeFiles) {
		this.includeFiles = includeFiles;
	}
	public boolean isEnabled() {
		return enabled;
	}
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	public boolean isFollowSymbolicLinks() {
		return followSymbolicLinks;
	}
	public void setFollowSymbolicLinks(boolean followSymbolicLinks) {
		this.followSymbolicLinks = followSymbolicLinks;
	}
	public boolean isScanLocalGroups() {
		return scanLocalGroups;
	}
	public void setScanLocalGroups(boolean scanLocalGroups) {
		this.scanLocalGroups = scanLocalGroups;
	}
	public boolean isIncludeInheritedPermissions() {
		return includeInheritedPermissions;
	}
	public void setIncludeInheritedPermissions(boolean includeInheritedPermissions) {
		this.includeInheritedPermissions = includeInheritedPermissions;
	}
	public SourceType getSourceType() {
		return sourceType;
	}
	public void setSourceType(SourceType sourceType) {
		this.sourceType = sourceType;
	}
	public boolean isIncludeHiddenSharedDrives() {
		return includeHiddenSharedDrives;
	}
	public void setIncludeHiddenSharedDrives(boolean includeHiddenSharedDrives) {
		this.includeHiddenSharedDrives = includeHiddenSharedDrives;
	}
	public String getExcludeShares() {
		return excludeShares;
	}
	public void setExcludeShares(String excludeShares) {
		this.excludeShares = excludeShares;
	}
	public boolean isIncludeHiddenShares() {
		return includeHiddenShares;
	}
	public void setIncludeHiddenShares(boolean includeHiddenShares) {
		this.includeHiddenShares = includeHiddenShares;
	}
	public String getServerLoginUsername() {
		return serverLoginUsername;
	}
	public void setServerLoginUsername(String serverLoginUsername) {
		this.serverLoginUsername = serverLoginUsername;
	}
	public String getServerLoginPassword() {
		return serverLoginPassword;
	}
	public void setServerLoginPassword(String serverLoginPassword) {
		this.serverLoginPassword = serverLoginPassword;
	}
	public String getServerLoginDomain() {
		return serverLoginDomain;
	}
	public void setServerLoginDomain(String serverLoginDomain) {
		this.serverLoginDomain = serverLoginDomain;
	}
	
	
}
