package com.perdemia.permissionanalyzer.application.parts;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.contentproviders.AllMembersPartContentProvider;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.AllMembersPartContentProviderConfig;
import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.SpecialPermissionLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel.ExplainType;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;

public class AllMembersPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.allmembers";
	private static Logger logger = LoggerFactory.getLogger(AllMembersPart.class.getName());	
	
	private Button loadAllMembersButton;
	private Composite searchComposite;
	private Text searchText;
	private TreeViewer viewer;
		
	@Inject EPartService partService;
		
	@PostConstruct
	public void createComposite(Composite parent) {
		// View layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 5;
		gridLayout.horizontalSpacing = 0;
		parent.setLayout(gridLayout);

		new ExplainLabel(parent, "This tab is not scoped on the selected folder, it displays all the members that are found in the folder tree and that match the filter criteria. For each member it is possible to display ALL applicable explicit Access Control Entries that match the filter criteria by unfolding the member. ", ExplainType.ALL_MEMBERS_TAB);
		
		// load the list with all the members found in the directory tree
		loadAllMembersButton = new Button(parent, SWT.PUSH);
		loadAllMembersButton.setText("Retrieve all members");
		loadAllMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.LOAD_LIST_16X16));
		loadAllMembersButton.setToolTipText("Load the list with all the members found in the directory tree");
		loadAllMembersButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true));
		loadAllMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				
				if (FilterHelper.getInstance().getFilterConfig(false).isInvertFolders()) {
					SyncMessageDialog.showMessage(MessageDialog.WARNING, "Not supported", "The list of all members in the overview is not supported when the filter option \"Invert matching folders\" has been selected.");
					return;
				}
				
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getDefault().getActiveShell());
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){
						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							//try {
								monitor.beginTask("Getting all members from the results...", IProgressMonitor.UNKNOWN);
								
								// load the initial list with all the members
								Display.getDefault().syncExec(new Runnable(){
									public void run() {
										AllMembersPartContentProviderConfig config = new AllMembersPartContentProviderConfig("", FilterHelper.getInstance());
										viewer.setInput(config);
									}
								});
								
								monitor.beginTask("Adding members to the list...", IProgressMonitor.UNKNOWN);
								
								// refresh the TableViewer
								Display.getDefault().asyncExec(new Runnable() {
									public void run() {
										
										searchAllMembers(true);
										
										// show the TreeViewer and hide this button
										((GridData)loadAllMembersButton.getLayoutData()).exclude = true;
										loadAllMembersButton.setVisible(false);
										((GridData)searchComposite.getLayoutData()).exclude = false;
										searchComposite.setVisible(true);
										((GridData)viewer.getTree().getLayoutData()).exclude = false;
										viewer.getTree().setVisible(true);
										searchComposite.getParent().layout(true);
										
									}
								});
//							} catch (DBException ex) {
//								logger.error("Error while getting all members from the results", ex);
//								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting all members from the results: " + ex.getMessage());
//								return;
//							}
						}
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while getting all members from the results", ex);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting all members from the results: " + ex.getMessage());
					return;
				}
			}
		});
		
		// search box
		searchComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData();
		gridData.exclude = true;
		searchComposite.setLayoutData(gridData);
		searchComposite.setLayout(new GridLayout(2, false));
		searchComposite.setVisible(false);
		
		searchText = DelayedKeyListenerText.create(searchComposite, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				searchAllMembers(false);			
			}			
		});
		searchText.setMessage("search user or group");
		gridData = new GridData();
		gridData.widthHint = 300;
		searchText.setLayoutData(gridData);
		searchText.addTraverseListener(new TraverseListener(){
			@Override
			public void keyTraversed(TraverseEvent e) {
				searchAllMembers(true);
			}
		});
				
		Button searchButton = new Button(searchComposite, SWT.PUSH);
		searchButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_USER_16X16));
		searchButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchAllMembers(true);
			}
		});
		
		// Tree with details	
		viewer = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
		viewer.getTree().setLinesVisible(true);
		viewer.getTree().setHeaderVisible(true);
		viewer.setUseHashlookup(true);
		
		//layout data for the tree
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.exclude = true;
		viewer.getTree().setLayoutData(gridData);
		viewer.getTree().setVisible(false);
		
		// Member column
		TreeViewerColumn column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Member");
		column.getColumn().setWidth(400);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return aclItem.getMember(true).getDisplayName(true, false);
					
				} else if (element instanceof Member){
					Member member = (Member) element;
					return member.getDisplayName(true, false);
				}
				return null;
			}
			
			@Override
			public Image getImage(Object element) {
				if (element instanceof AclItem) {
					return ImageFactory.getInstance().getImage(AppImage.ACL_20X20);
				} else {
					Member member = (Member) element;
					if (member.isGroup()){
						return ImageFactory.getInstance().getImage(AppImage.GROUP_20X20);
			        } else {
			        	return ImageFactory.getInstance().getImage(AppImage.USER_20X20);
			        }
				}
			}
						
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});

		// From folder column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("File");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(350);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.getFileId() != null) {
						return aclItem.getFile(true).determineFullPath();
					} else {
						return null;
					}
				} else {
					return null;
				}
			}
						
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
		
		// Permission column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Permission");
		column.getColumn().setWidth(200);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return PermissionUtil.permissionsToString(aclItem, false);					
				}
				return null;
			}
						
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
						
		// Special permission columns
		for (final Integer permission : PermissionUtil.getCustomPermissionOrderList()) {
			//skip SYNCHRONIZE
			if (permission.intValue() != SpecialPermission.SYNCHRONIZE.ordinal()) {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setWidth(30);
				column.getColumn().setResizable(false);
				column.getColumn().setToolTipText(PermissionUtil.specialPermissionToString(permission, false, null));
				column.getColumn().setImage(PermissionUtil.getSpecialPermissionImage(permission, false, true));
				column.setLabelProvider(new SpecialPermissionLabelProvider(permission, true, true));
			}
		}
		
		// Inheritance flags column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Inheritance flags");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(250);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.getFileId() != null) {
						return PermissionUtil.aclEntryFlagsToString(aclItem, aclItem.getFile(true).isDirectory());
					} else {
						return null;
					}
				} else {
					return null; //Member row
				}
			}
			
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
				
		viewer.setContentProvider(new AllMembersPartContentProvider(viewer));
		
		//enable tooltips for this tree viewer
		ColumnViewerToolTipSupport.enableFor(viewer);
		
		// context menu for the Tree
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					// Add to Trace view action
					Action action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							MPart part = partService.findPart(TracePart.ID);
							partService.activate(part, true);
							TracePart tracePart = (TracePart) part.getObject();
							tracePart.applySelectedMember(member);
						}
					};
					action.setText("Pin member in the Trace view");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.TRACE_16X16));
					manager.add(action);
					
					// Show user info action
					action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
										
					// Add as Include filter action
					action = new Action() {
						public void run() {
							Member member;
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, true);
							}
						}
					};
					action.setText("Add selected members as include filter");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.INCLUDE_GROUPS_16X16));
					action.setToolTipText("Adds the selected members as includes in the main filter to scope the data to the selected members.");
					manager.add(action);
					
					// Add as Exclude filter action
					action = new Action() {
						public void run() {
							Member member;
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, false);
							}
						}
					};
					action.setText("Add selected members as exclude filter");
					action.setToolTipText("Adds the selected members as excludes in the main filter.");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.EXCLUDE_GROUPS_16X16));
					manager.add(action);
					
					// Open file in explorer action
					if (selection.getFirstElement() instanceof AclItem) {
						action = new Action() {
							public void run() {
								FileInfo file = ((AclItem) selection.getFirstElement()).getFile(true);
								Program.launch(file.determineFullPath());
							}
						};
						action.setText("Open file in Explorer");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
						manager.add(action);
					}
				}
			}
		});
		
		Menu menu = menuMgr.createContextMenu(viewer.getTree());
		viewer.getTree().setMenu(menu);
	}
	
	/**
	 * Refreshes the list with all members, considering the search text entered by the user
	 * 
	 * @param explicitUserSearchAction true in case the user pressed the search button or ENTER in the search text box
	 */
	private void searchAllMembers(boolean explicitUserSearchAction) {
		String search = searchText.getText().toLowerCase().replace("*", "");
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with % OR the search list is based on the baseList
		if (explicitUserSearchAction ||	search.length() > 2) {
			
			ApplicationUtil.setBusyCursor();
			
			AllMembersPartContentProviderConfig config = new AllMembersPartContentProviderConfig(search, FilterHelper.getInstance());
			viewer.setInput(config);
			viewer.refresh();
			viewer.getTree().getParent().layout(true);
			
			if (((AllMembersPartContentProvider)viewer.getContentProvider()).getFilteredMembers().size() == 0 && explicitUserSearchAction){
				MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "No results", "No members found.");
			}
			
			ApplicationUtil.setDefaultCursor();
		}		
	}
	
	/**
	 * Clears the TableViewer with all the members
	 */
	public void clearAllMembers() {
		viewer.setInput(null);
		viewer.refresh();
		
		// hide the TreeViewer and show the load button
		((GridData)loadAllMembersButton.getLayoutData()).exclude = false;
		loadAllMembersButton.setVisible(true);
		((GridData)searchComposite.getLayoutData()).exclude = true;
		searchComposite.setVisible(false);
		((GridData)viewer.getTree().getLayoutData()).exclude = true;
		viewer.getTree().setVisible(false);
		searchComposite.getParent().layout(true);
	}
	
	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();		
	}
	
	public AclItem getSelectedAclItem() {
		if (viewer != null && !viewer.getSelection().isEmpty()) {
			IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
			if (selection.getFirstElement() instanceof AclItem) {
				return (AclItem) selection.getFirstElement();
			} else {
				return null; //Member selected
			}
		} else {
			return null;
		}
	}
	
	public Member getSelectedMember() {
		if (viewer != null && !viewer.getSelection().isEmpty()) {
			IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
			Member member;
			
			if (selection.getFirstElement() instanceof Member) {
				member = (Member) selection.getFirstElement();
				
				if (member.getAclItemList() == null) {
					((AllMembersPartContentProvider) viewer.getContentProvider()).updateChildCount(member, 0);
				}
			} else {
				member = ((AclItem) selection.getFirstElement()).getViaMember();
			}
			
			if (member != null && member.getId() != null) {
				return member;
			} else {
				return null; // may be a dummy member with name "No explicit ACE's found"
			}
		} else {
			return null;
		}
	}

	public List<Member> getFilteredMembers() {
		return ((AllMembersPartContentProvider)viewer.getContentProvider()).getFilteredMembers();
	}
}
