package com.perdemia.permissionanalyzer.application.parts;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.AlertDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.AlertDialog.AlertType;
import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SaveFilterDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchLdapDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchMembersDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.WildcardInputDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.FilterSet;
import com.perdemia.permissionanalyzer.data.FilterSet.FilterSetType;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.ReportFilterSet;

public class FilterMemberPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.memberfilter";
	private static final Logger logger = LoggerFactory.getLogger(FilterMemberPart.class.getName());
	
	/** Controls for the saved selections */
	private Button deleteSelectionButton;
	private Combo memberSelectionCombo;
	private int memberSelectionComboIndex;
	
	/** Member filter controls */
	private Button includeGroupMembershipButton;
	private Button includeUsersButton;
	private Button includeGroupsButton;
	private Button includeBuiltInButton;
	private Button includeADGroupsAndUsersButton;
	private Button includeDisabledUsersButton;
	
	private TableViewer memberFilterListViewer;
	private TableViewer groupFilterListViewer;
	private java.util.List<FilterSet> filterSets = new ArrayList<FilterSet>();
	
	//TODO Filter on LDAP path (only wildcard filter)
	//TODO button to test wildcard search (display results)
	//TODO Filter: exclude local users/groups

	@Inject	private MDirtyable dirty;
	@Inject private ECommandService commandService;
	@Inject private EHandlerService handlerService;
	
	@PostConstruct
	public void createComposite(Composite parent) {
		ScrolledComposite scrollableComposite = new ScrolledComposite(parent, SWT.NONE | SWT.H_SCROLL | SWT.V_SCROLL);
		scrollableComposite.setMinSize(350, 500);
		scrollableComposite.setExpandHorizontal(true);
		scrollableComposite.setExpandVertical(true);
		scrollableComposite.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		final Composite mainContent = new Composite(scrollableComposite, SWT.NONE);
		mainContent.setLayout(new GridLayout(1, false));
		mainContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		scrollableComposite.setContent(mainContent);
		
		// Title label
		CLabel memberLabel = new CLabel(mainContent, SWT.NONE);
		memberLabel.setImage(ImageFactory.getInstance().getImage(AppImage.USER_FILTER_HEADER_40X32));
		memberLabel.setText("Filter on members");
		memberLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h1");
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.CENTER;
		memberLabel.setLayoutData(gridData);

		// ---------------------------------
		// Saved member filter selections
		// ---------------------------------
		CLabel memberSelectionLabel = new CLabel(mainContent, SWT.NONE);
		memberSelectionLabel.setText("Saved member selections:");
		memberSelectionLabel.setImage(ImageFactory.getInstance().getImage(AppImage.SELECTION_16X16));

		// Combo with filter selections
		Composite filterSelectionComposite = new Composite(mainContent, SWT.NONE);
		filterSelectionComposite.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		filterSelectionComposite.setLayoutData(gridData);
		
		memberSelectionCombo = new Combo(filterSelectionComposite, SWT.READ_ONLY);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		memberSelectionCombo.setLayoutData(gridData);
		memberSelectionCombo.add("Custom");
		memberSelectionCombo.select(0);
		memberSelectionCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				String filterSetName = memberSelectionCombo.getItem(memberSelectionCombo.getSelectionIndex());
				if (!filterSetName.equals("Custom")) {
					loadNewFilterSet(getSelectedFilterSet());
				} else {
					deleteSelectionButton.setEnabled(false);
					memberSelectionComboIndex = 0;
				}
			}
		});
		
		// add saved filter selections
		try {
			filterSets = FilterSet.where("filter_set_type = ?", new Object[]{FilterSet.FilterSetType.MEMBER.ordinal()});
			for (FilterSet filterSet : filterSets) {
				memberSelectionCombo.add(filterSet.getName());
			}		
		} catch (DBException ex) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading available filters: " + ex.getMessage()); 
		}
		
		// save button
		Button saveSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		saveSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.SAVE_16X16));
		saveSelectionButton.setToolTipText("Save selection");
		saveSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		saveSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				saveFilterSet();
			}
		});
		
		// Delete selection button
		deleteSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		deleteSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		deleteSelectionButton.setToolTipText("Delete selection");
		deleteSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		deleteSelectionButton.setEnabled(false);
		deleteSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Delete member selection", "Are you sure you want to delete the member selection " + getSelectedFilterSet().getName() + "?")) {
					try {					
						FilterSet filterSet = getSelectedFilterSet();
						
						// Is this filter set still attached to a report?
						StringBuilder reports = new StringBuilder();
						List<ReportFilterSet> reportFilterSets = ReportFilterSet.where("filterset_id = ?", new Object[]{filterSet.getId()});
						for (ReportFilterSet reportFilterSet : reportFilterSets) {
							Report report = Report.getById(reportFilterSet.getReportId());
							if (reports.length() > 0) {
								reports.append("\r\n");
							}
							reports.append(report.getName());
						}
						
						if (reports.length() > 0) {
							MessageDialog.openError(null, "Used by reports", "This filter set is being used by the following reports. Modify these reports and uncheck the filter set to copy the filters of this set to the reports.\r\n\r\n" + reports.toString());
							return;
						}
						
						FilterSet.delete(filterSet.getId());
						filterSets.remove(filterSet);
						memberSelectionCombo.remove(memberSelectionCombo.getSelectionIndex());
						resetFilters();						
					} catch (DBException ex) {
						MessageDialog.openError(null, "Error", "Error while deleting filter: " + ex.getMessage());
					}
				}
			}
		});
		
		// ---------------------------------
		// Options filter area
		// ---------------------------------
		/*
		CLabel filterOptionsLabel = new CLabel(mainContent, SWT.NONE);
		filterOptionsLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		filterOptionsLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		filterOptionsLabel.setText("Member filter options:");
		filterOptionsLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		*/
		
		// selection to include users / groups
		FormToolkit toolkit = new FormToolkit(parent.getDisplay());
		Section optionsSection = new Section(mainContent, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
		optionsSection.setText("Member filter options");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 10;
		optionsSection.setLayoutData(gridData);
				
		Composite optionsComposite = toolkit.createComposite(optionsSection, SWT.WRAP);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 20;
		optionsComposite.setLayout(gridLayout);
		optionsComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		CLabel userGroupSelectionLabel = new CLabel(optionsComposite, SWT.NONE);
		userGroupSelectionLabel.setText("Include: ");
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		userGroupSelectionLabel.setLayoutData(gridData);
		
		includeUsersButton = new Button(optionsComposite, SWT.CHECK);
		includeUsersButton.setImage(ImageFactory.getInstance().getImage(AppImage.USER_16X16));
		includeUsersButton.setText("Users");
		includeUsersButton.setToolTipText("Include users");
		includeUsersButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeUsers(includeUsersButton.getSelection());
		    }
	    });
		includeUsersButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeUsers(true);
		
		includeGroupsButton = new Button(optionsComposite, SWT.CHECK);
		includeGroupsButton.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
		includeGroupsButton.setText("Groups");
		includeGroupsButton.setToolTipText("Include groups");
		includeGroupsButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeGroups(includeGroupsButton.getSelection());
		    }
	    });
		includeGroupsButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeGroups(true);
		
		includeBuiltInButton = new Button(optionsComposite, SWT.CHECK);
		includeBuiltInButton.setImage(ImageFactory.getInstance().getImage(AppImage.BUILTIN_20X20));
		includeBuiltInButton.setText("Built-in groups");
		includeBuiltInButton.setToolTipText("Include built-in groups");
		includeBuiltInButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeBuiltIn(includeBuiltInButton.getSelection());
		    }
	    });
		includeBuiltInButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeBuiltIn(true);
		
		includeADGroupsAndUsersButton = new Button(optionsComposite, SWT.CHECK);
		includeADGroupsAndUsersButton.setImage(ImageFactory.getInstance().getImage(AppImage.AD_GROUPS_USERS_16X16));
		includeADGroupsAndUsersButton.setText("LDAP groups and users");
		includeADGroupsAndUsersButton.setToolTipText("Include LDAP groups and users");
		includeADGroupsAndUsersButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeADGroupsAndUsers(includeADGroupsAndUsersButton.getSelection());
		    }
	    });
		includeADGroupsAndUsersButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeADGroupsAndUsers(true);
		
		// group membership check box
		//CLabel includeGroupMembershipLabel = new CLabel(optionsComposite, SWT.NONE);
		//includeGroupMembershipLabel.setText("Include group membership:");
		includeGroupMembershipButton = new Button(optionsComposite, SWT.CHECK);
		includeGroupMembershipButton.setText("Group memberships");
		includeGroupMembershipButton.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_MEMBERSHIP_20X20));
		includeGroupMembershipButton.setToolTipText("Include (nested) group membership");
		includeGroupMembershipButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeGroupMembership(includeGroupMembershipButton.getSelection());
		    }
	    });
		includeGroupMembershipButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeGroupMembership(true);
		
		includeDisabledUsersButton = new Button(optionsComposite, SWT.CHECK);
		includeDisabledUsersButton.setText("Disabled users");
		includeDisabledUsersButton.setImage(ImageFactory.getInstance().getImage(AppImage.DISABLED_USER_20X20));
		includeDisabledUsersButton.setToolTipText("Include disabled users");
		includeDisabledUsersButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeDisabledUsers(includeDisabledUsersButton.getSelection());
		    }
	    });
		includeDisabledUsersButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeDisabledUsers(true);
		
		optionsSection.setClient(optionsComposite);
		
		// ---------------------------------
		// Member filter list
		// ---------------------------------
		// header bar
		Composite memberHeaderBar = new Composite(mainContent, SWT.NONE);
		memberHeaderBar.setLayout(new GridLayout(5, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		memberHeaderBar.setLayoutData(gridData);
		
		CLabel memberFilterListLabel = new CLabel(memberHeaderBar, SWT.NONE);
		memberFilterListLabel.setText("Include or exclude users/groups:");
		//memberFilterListLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		//memberFilterListLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		memberFilterListLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		gridData = new GridData(SWT.LEFT, SWT.BOTTOM, true, false);
		memberFilterListLabel.setLayoutData(gridData);
		
		//search members
		Button searchMembersButton = new Button(memberHeaderBar, SWT.PUSH);
		searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		searchMembersButton.setToolTipText("Filter on ACE members");
		searchMembersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), memberFilterListViewer, Filter.FilterType.MEMBER_ID, false, true, true);
				int result = searchDialog.open();
				if (result == SearchMembersDialog.ADD_TO_INCLUDE_ID || result == SearchMembersDialog.ADD_TO_EXCLUDE_ID) {
					dirty.setDirty(true);
				}
			}
		});

		final Button otherMemberFiltersButton = new Button(memberHeaderBar, SWT.PUSH);
		otherMemberFiltersButton.setText(" ... ");
		otherMemberFiltersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		// context menu for the otherMemberFiltersButton
		MenuManager othermemberFiltersMenuMgr = new MenuManager("#OtherMemberFiltersMenu"); //$NON-NLS-1$
		othermemberFiltersMenuMgr.setRemoveAllWhenShown(true);
		othermemberFiltersMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				// Search on wildcard
				Action action = new Action() {
					public void run() {
						InputDialog inputDialog = new WildcardInputDialog(
								Display.getCurrent().getActiveShell(),
								"Add wildcard filter on account name",
								"Please enter a wildcard value using the * character. Wildcard values will be used to filter on account name (without the domain name) and are not case-sensitive.\r\n\r\nExample: John*\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.", false);
						int result = inputDialog.open();
						if (result == WildcardInputDialog.ADD_TO_INCLUDE_ID || result == WildcardInputDialog.ADD_TO_EXCLUDE_ID){
							if (!inputDialog.getValue().contains("%") && !inputDialog.getValue().contains("*")) {
								MessageDialog.openWarning(Display.getCurrent().getActiveShell(), "Invalid value", "The search text contains no wildcards, please use the member search button to filter on specific account names.");
							} else {
								FilterHelper.getInstance().getMemberFilterList().add(new Filter(Filter.FilterType.MEMBER_NAME, result == WildcardInputDialog.ADD_TO_INCLUDE_ID, inputDialog.getValue(), inputDialog.getValue()));
								memberFilterListViewer.refresh();
								dirty.setDirty(true);
							}
						}
					}
				};
				action.setText("Add wildcard search text");
				action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.ADD_WILDCARD_16X16));
				manager.add(action);
				
				// Search on owner
				action = new Action() {
					public void run() {
						SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), memberFilterListViewer, Filter.FilterType.OWNER_ID, false, true, true);
						searchDialog.setTitle("Include or exclude owners (users and groups)");
						int result = searchDialog.open();
						if (result == SearchMembersDialog.ADD_TO_INCLUDE_ID || result == SearchMembersDialog.ADD_TO_EXCLUDE_ID) {
							dirty.setDirty(true);
						}
					}
				};
				action.setText("Filter on owner");
				action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.ACCESS_CONTROL_16X16));
				manager.add(action);
				
				// Use members from another report
				action = new Action() {
					public void run() {
						ReportDialog dialog = new ReportDialog(mainContent.getShell(), false, true);
				    	int result = dialog.open();
				    	Report selectedReport = dialog.getSelectedReport();
				    	
				    	if (result == ReportDialog.ADD_TO_INCLUDE_ID || result == ReportDialog.ADD_TO_EXCLUDE_ID){
							FilterHelper.getInstance().getMemberFilterList().add(new Filter(Filter.FilterType.MEMBER_REPORT, result == ReportDialog.ADD_TO_INCLUDE_ID, String.valueOf(selectedReport.getId()), selectedReport.getName()));
							memberFilterListViewer.refresh();
							dirty.setDirty(true);
						}
					}
				};
				action.setText("Use the members from another report");
				action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.REPORT_16X16));
				manager.add(action);
			}
		});
		Menu otherMemberFiltersMenu = othermemberFiltersMenuMgr.createContextMenu(otherMemberFiltersButton);
		otherMemberFiltersButton.setMenu(otherMemberFiltersMenu);
		otherMemberFiltersButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				otherMemberFiltersButton.getMenu().setVisible(true);
			}
		});
		
		// Remove from member list
		Button removeFromMemberListButton = new Button(memberHeaderBar, SWT.PUSH);
		removeFromMemberListButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeFromMemberListButton.setToolTipText("Remove selected members from the filter list");
		removeFromMemberListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		removeFromMemberListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) memberFilterListViewer.getSelection();
				for (Iterator iterator = selection.iterator(); iterator.hasNext();) {
					FilterHelper.getInstance().getMemberFilterList().remove(iterator.next());
					dirty.setDirty(true);
				}
				memberFilterListViewer.refresh();
			}
		});
		
		//create a one column TableViewer so an image can be used
		memberFilterListViewer = new TableViewer(mainContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION);
		gridData = new GridData(GridData.FILL_BOTH);
		memberFilterListViewer.getTable().setLayoutData(gridData);
		
		final TableViewerColumn memberColumn = new TableViewerColumn(memberFilterListViewer, SWT.NONE);
		memberFilterListViewer.getTable().addControlListener(new ControlListener(){
			@Override
			public void controlMoved(ControlEvent e) {}

			@Override
			public void controlResized(ControlEvent e) {
				memberColumn.getColumn().setWidth(memberFilterListViewer.getTable().getSize().x-4);
			}
		});
		
		memberFilterListViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public void dispose() {}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

			@Override
			public Object[] getElements(Object inputElement) {
				@SuppressWarnings("unchecked")
				List<Filter> filters = (List<Filter>)inputElement;
				List<Filter> result = new ArrayList<Filter>();
				for (Filter filter : filters) {
					if (filter.getType().equals(FilterType.MEMBER_NAME) || filter.getType().equals(FilterType.MEMBER_ID) || filter.getType().equals(FilterType.OWNER_ID) || filter.getType().equals(FilterType.MEMBER_REPORT)) {
						result.add(filter);
					}
				}
				return result.toArray();
			}
		});
		
		memberFilterListViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					Filter filter = (Filter) element;
					if (filter.getType().equals(Filter.FilterType.MEMBER_ID)) {
						if (filter.isMemberGroup()) {
							if (filter.isInclude()) {
								return ImageFactory.getInstance().getImage(AppImage.INCLUDE_GROUPS_16X16);
							} else {
								return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_GROUPS_16X16);
							}
						} else {
							if (filter.isInclude()) {
								return ImageFactory.getInstance().getImage(AppImage.INCLUDE_USERS_16X16);
							} else {
								return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_USERS_16X16);
							}
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.OWNER_ID)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_OWNER_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_OWNER_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_REPORT)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_REPORT_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_REPORT_16X16);
						}
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Filter filter = (Filter) element;
				switch (columnIndex) {
					case 0: return filter.getName();
					default: return "";
				}
			}
		});
				
		memberFilterListViewer.setInput(FilterHelper.getInstance().getMemberFilterList());
				
		// ---------------------------------
		// Group / LDAP OU filter list
		// ---------------------------------
		// header bar
		Composite groupHeaderBar = new Composite(mainContent, SWT.NONE);
		groupHeaderBar.setLayout(new GridLayout(5, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		groupHeaderBar.setLayoutData(gridData);
		
		CLabel groupFilterListLabel = new CLabel(groupHeaderBar, SWT.NONE);
		groupFilterListLabel.setText("All members from group or OU:");
		//groupFilterListLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		//groupFilterListLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		groupFilterListLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		gridData = new GridData(SWT.LEFT, SWT.BOTTOM, true, false);
		groupFilterListLabel.setLayoutData(gridData);
		
		//search ldap OU
		Button searchLdapOUButton = new Button(groupHeaderBar, SWT.PUSH);
		searchLdapOUButton.setImage(ImageFactory.getInstance().getImage(AppImage.LDAP_OU_16X16));
		searchLdapOUButton.setToolTipText("Select LDAP OU");
		searchLdapOUButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchLdapOUButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchLdapDialog searchDialog = new SearchLdapDialog(Display.getCurrent().getActiveShell(), groupFilterListViewer, true, null, true, true);
				int result = searchDialog.open();
				if (result == SearchMembersDialog.ADD_TO_INCLUDE_ID || result == SearchMembersDialog.ADD_TO_EXCLUDE_ID) {
					dirty.setDirty(true);
				}
			}
		});
				
		//search groups
		Button searchGroupsButton = new Button(groupHeaderBar, SWT.PUSH);
		searchGroupsButton.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_16X16));
		searchGroupsButton.setToolTipText("Select groups");
		searchGroupsButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchGroupsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), groupFilterListViewer, Filter.FilterType.MEMBER_GROUP_ID, true, true, true);
				int result = searchDialog.open();
				if (result == SearchMembersDialog.ADD_TO_INCLUDE_ID || result == SearchMembersDialog.ADD_TO_EXCLUDE_ID) {
					dirty.setDirty(true);
				}
			}
		});
		
		// Add wildcard value button
		Button addWildCardToGroupListButton = new Button(groupHeaderBar, SWT.PUSH);
		addWildCardToGroupListButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_WILDCARD_16X16));
		addWildCardToGroupListButton.setToolTipText("Add wildcard search text");
		addWildCardToGroupListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		addWildCardToGroupListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				InputDialog inputDialog = new WildcardInputDialog(
						Display.getCurrent().getActiveShell(),
						"Add wildcard filter on group name",
						"Please enter a wildcard value using the * character. Wildcard values will be used to filter on group name (without the domain name).\r\n\r\nExample: HR*\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.", false);
				int result = inputDialog.open();
				if (result == WildcardInputDialog.ADD_TO_INCLUDE_ID || result == WildcardInputDialog.ADD_TO_EXCLUDE_ID){
					if (!inputDialog.getValue().contains("%") && !inputDialog.getValue().contains("*")) {
						MessageDialog.openWarning(Display.getCurrent().getActiveShell(), "Invalid value", "The search text contains no wildcards, please use the search group button to filter on specific group names.");
					} else {
						if (FilterHelper.checkNumberOfMemberFiltersForTrial(FilterHelper.getInstance().getMemberFilterList(), true)) {
							FilterHelper.getInstance().getMemberFilterList().add(new Filter(Filter.FilterType.MEMBER_GROUP_NAME, result == WildcardInputDialog.ADD_TO_INCLUDE_ID, inputDialog.getValue(), inputDialog.getValue()));
							groupFilterListViewer.refresh();
							dirty.setDirty(true);
						}
					}
				}
			}
		});
		
		// Remove from member list
		Button removeFromGroupListButton = new Button(groupHeaderBar, SWT.PUSH);
		removeFromGroupListButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeFromGroupListButton.setToolTipText("Remove selected items from the filter list");
		removeFromGroupListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		removeFromGroupListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) groupFilterListViewer.getSelection();
				for (Iterator iterator = selection.iterator(); iterator.hasNext();) {
					FilterHelper.getInstance().getMemberFilterList().remove(iterator.next());
					dirty.setDirty(true);
				}
				groupFilterListViewer.refresh();
			}
		});
		
		//create a one column TableViewer so an image can be used
		groupFilterListViewer = new TableViewer(mainContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION);
		gridData = new GridData(GridData.FILL_BOTH);
		groupFilterListViewer.getTable().setLayoutData(gridData);
		
		final TableViewerColumn groupColumn = new TableViewerColumn(groupFilterListViewer, SWT.NONE);
		
		groupFilterListViewer.getTable().addControlListener(new ControlListener(){
			@Override
			public void controlMoved(ControlEvent e) {}

			@Override
			public void controlResized(ControlEvent e) {
				groupColumn.getColumn().setWidth(groupFilterListViewer.getTable().getSize().x-4);
			}
		});
		
		groupFilterListViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public void dispose() {}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

			@Override
			public Object[] getElements(Object inputElement) {
				@SuppressWarnings("unchecked")
				List<Filter> filters = (List<Filter>)inputElement;
				List<Filter> result = new ArrayList<Filter>();
				for (Filter filter : filters) {
					if (filter.getType().equals(FilterType.MEMBER_GROUP_ID) || filter.getType().equals(FilterType.MEMBER_GROUP_NAME) || 
							filter.getType().equals(FilterType.MEMBER_LDAP_OU)) {
						result.add(filter);
					}
				}
				return result.toArray();
			}
		});
		
		groupFilterListViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					Filter filter = (Filter) element;
					if (filter.getType().equals(Filter.FilterType.MEMBER_GROUP_ID)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_GROUPS_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_GROUPS_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_GROUP_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_LDAP_OU)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_LDAP_OU_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_LDAP_OU_16X16);
						}
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Filter filter = (Filter) element;
				switch (columnIndex) {
					case 0: return filter.getName();
					default: return "";
				}
			}
		});
				
		groupFilterListViewer.setInput(FilterHelper.getInstance().getMemberFilterList());
		
		//----------------------------------------------
		// context menu for the tables
		//----------------------------------------------
		
		// context menu for the memberFilterListViewer
		MenuManager memberMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		memberMenuMgr.setRemoveAllWhenShown(true);
		memberMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) memberFilterListViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Filter filter = ((Filter) selection.getFirstElement());
					if (filter.getType().equals(FilterType.MEMBER_ID) || filter.getType().equals(FilterType.OWNER_ID)) {
						Action action = new Action() {
							public void run() {
								MemberInfoDialog dialog = new MemberInfoDialog(null, Integer.parseInt(filter.getValue()));
								dialog.open();
							}
						};
						action.setText("Show member info");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
						manager.add(action);
						
					} else if (filter.getType().equals(FilterType.MEMBER_NAME) || filter.getType().equals(FilterType.MEMBER_REPORT)){
						
						// filter on name with wildcard
						Action action = new Action() {
							public void run() {
								if (filter.getType().equals(FilterType.MEMBER_NAME)) {
									String baseQuery = "select * from permissionanalyzer.member where name_lowercase like ?";
									// don't use ORDER BY member.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
									MemberInfoDialog dialog = new MemberInfoDialog(null, baseQuery, filter.getValue().toLowerCase(), "Members with account name", filter.getValue().toLowerCase(), "3");
									dialog.open();
								} else {
									// load the members from the report configured in the filter
									try {
										Report report = Report.getById(Integer.parseInt(filter.getValue()));
										List<Member> reportMembers = report.getMembersInFolderTree();
										MemberInfoDialog dialog = new MemberInfoDialog(null, reportMembers, "Members found in the folder tree of report", filter.getName());
										dialog.open();
									} catch (Exception ex) {
										logger.error("Error while loading the members in report " + filter.getName(), ex);
										SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading the members in report " + filter.getName() + ": " + ex.getMessage());
									}
								}
							}
						};
						action.setText("Show matching members");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
						manager.add(action);
					}
				}
			}
		});
		Menu memberMenu = memberMenuMgr.createContextMenu(memberFilterListViewer.getTable());
		memberFilterListViewer.getTable().setMenu(memberMenu);
				
		// context menu for the groupFilterListViewer
		MenuManager groupMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		groupMenuMgr.setRemoveAllWhenShown(true);
		groupMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) groupFilterListViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Filter filter = ((Filter) selection.getFirstElement());
					if (filter.getType().equals(FilterType.MEMBER_GROUP_ID)) {
						Action action = new Action() {
							public void run() {
								MemberInfoDialog dialog = new MemberInfoDialog(null, Integer.parseInt(filter.getValue()));
								dialog.open();
							}
						};
						action.setText("Show member info");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
						manager.add(action);
						
					} else if (filter.getType().equals(FilterType.MEMBER_GROUP_NAME)){
						
						// filter on name with wildcard
						Action action = new Action() {
							public void run() {
								// query to select all members of the groups that have the wild card search as name. Only select direct members to display!
								String baseQuery = "select distinct member.*, filter_group.name VIA_GROUP_NAME, filter_group.name_lowercase from permissionanalyzer.member inner join permissionanalyzer.membership on member.id = membership.member_id"
										+ " inner join permissionanalyzer.member filter_group on membership.group_id = filter_group.id"
										+ " where filter_group.name_lowercase like ? and membership.member_id <> membership.group_id and membership.via_group_id is null";					
								
								// don't use ORDER BY member.name_lowercase, filter_group.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
								MemberInfoDialog dialog = new MemberInfoDialog(null, baseQuery, filter.getValue().toLowerCase(), "All members from groups with account name", filter.getValue().toLowerCase(), "3,12");
								dialog.open();
							}
						};
						action.setText("Show matching members");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
						manager.add(action);
					}
					//TODO open SearchMembersDialog for FilterType.MEMBER_LDAP_OU
				}
			}
		});
		Menu groupMenu = groupMenuMgr.createContextMenu(groupFilterListViewer.getTable());
		groupFilterListViewer.getTable().setMenu(groupMenu);
	}
	
	/**
	 * Returns the selected <code>FilterSet</code> based on the name in the dropdown box
	 * 
	 * @return selected <code>FilterSet</code>
	 */
	public FilterSet getSelectedFilterSet() {
		String name = memberSelectionCombo.getItem(memberSelectionCombo.getSelectionIndex());
		
		// get the selected filter set
		for (FilterSet filterSet : filterSets) {
			if (filterSet.getName().equals(name)) {
				try {
					filterSet.loadFilters();
				} catch (DBException e) {
					logger.error("Error while loading the filters for filter set " + filterSet.getName(), e);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading the filters for filter set " + filterSet.getName() + ": " + e.getMessage());
				}
				return filterSet;
			}
		}
		return null;
	}
	
	/** Sets all filters to their default values */
	public void resetFilters() {
		FilterHelper.getInstance().getMemberFilterList().clear();
		FilterHelper.getInstance().getFilterConfig(true).setIncludeGroupMembership(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeUsers(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeGroups(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeBuiltIn(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeADGroupsAndUsers(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeDisabledUsers(true);
				
		refreshFiltersInGUI();
		deleteSelectionButton.setEnabled(false);
	}
	
	/** 
	 * Load a new FilterSet in the GUI. This method is triggered by the onChange of {@link #memberSelectionCombo} or from the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilterSet(FilterSet filterSet) {
		boolean filtersLoaded = false;
		
		// get the selected filter set
		if (filterSet != null) {
			try {
				// get related filters
				filterSet.loadFilters();
				filtersLoaded = loadNewFilters(filterSet.getFilters());
				
			} catch (DBException ex) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading filters for the selected filter set: " + ex.getMessage());
			}
		}
		
		if (filtersLoaded) { // not cancelled?
			deleteSelectionButton.setEnabled(true);
			
			// select the dropdown item (this is not yet done in case the method is called from the ManageReportsHandler
			for (int counter = 0; counter < memberSelectionCombo.getItemCount(); counter++) {
				if (memberSelectionCombo.getItem(counter).equalsIgnoreCase(filterSet.getName())) {
					memberSelectionCombo.select(counter);
					memberSelectionComboIndex = counter;
					break;
				}
			}
		}
		
		return filtersLoaded;
	}
	
	/** 
	 * Load a new Filters in the GUI. This method is used by {@link #loadNewFilterSet(FilterSet)} and triggered by the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilters(List<Filter> filters) {
		if (dirty.isDirty() && !MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Unsaved member filters", "Are you sure you want to overwrite the current member filters?")) {
			//user cancelled, select last index
			memberSelectionCombo.select(memberSelectionComboIndex);
			return false;
		} else {
			//load the new filters
			FilterHelper.getInstance().getMemberFilterList().clear();
			FilterHelper.getInstance().getMemberFilterList().addAll(filters);
			refreshFiltersInGUI();
			return true;
		}
	}
	 
	/**
	 * Updates the GUI after a new set of filters has been loaded. Either by selecting a filter set from the dropdown or by the method {@link #loadNewFilters(List)}. 
	 */
	private void refreshFiltersInGUI() {
		memberFilterListViewer.refresh();
		groupFilterListViewer.refresh();
		includeGroupMembershipButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeGroupMembership());
		includeUsersButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeUsers());
		includeGroupsButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeGroups());
		includeBuiltInButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeBuiltIn());
		includeADGroupsAndUsersButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeADGroupsAndUsers());		
		includeDisabledUsersButton.setSelection(FilterHelper.getInstance().getFilterConfig(true).isIncludeDisabledUsers());
		memberSelectionCombo.select(0);
		dirty.setDirty(false);
	}
	
	@Persist
	private void saveFilterSet() {
		try {
			int selectedIndex = -1;
			FilterSet filterSet = null;
			if (!memberSelectionCombo.getText().equals("Custom")) {
				filterSet = getSelectedFilterSet();
				selectedIndex = filterSets.indexOf(filterSet);
			}
			
			SaveFilterDialog dialog = new SaveFilterDialog(Display.getCurrent().getActiveShell(), filterSets, selectedIndex, "Member filter sets");
			int result = dialog.open(); 
			
			if (result == IDialogConstants.OK_ID) {;
				// existing filter set is selected	
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				
				// remove all the current filters
				Filter.delete("FILTERSET_ID = ?", filterSet.getId());
				filterSet.setFilters(new ArrayList<Filter>());
				
			} else if (result == SaveFilterDialog.CREATE_ID) {
				// a new filter set is created
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				filterSet.setType(FilterSetType.MEMBER);
				filterSet.setFilters(new ArrayList<Filter>());
				filterSet.save();
				
				filterSets.add(filterSet);
				memberSelectionCombo.add(filterSet.getName());
			} else {
				//user cancelled
				return;
			}
			
			// add all the current member filter items to the filter set
			for (Filter filter :  FilterHelper.getInstance().getMemberFilterList()) {
				//make sure to create a new filter
				filter.setId(null);
				filter.setFilterSetId(filterSet.getId());
				filter.save();
				filterSet.getFilters().add(filter);
			}
			
			selectedIndex = filterSets.indexOf(filterSet);
			memberSelectionCombo.select(selectedIndex + 1); //skip the first item "Custom"
			deleteSelectionButton.setEnabled(true);
			
			dirty.setDirty(false);			
		} catch (DBException ex) {
			MessageDialog.openError(null, "Error", "Error while saving filter: " + ex.getMessage());
		}
	}
	
	/**
	 * Adds the given member to the list of member filters using {@link FilterHelper#getMemberFilterList()} and refreshes the ListViewer in the GUI.
	 * @param member
	 * @param isIncludeFilter
	 * @see {@link MemberScopePart}
	 */
	public void addNewMemberFilter(Member member, boolean isIncludeFilter) {
		// check if the filter already exists
		List<Filter> memberFilters = FilterHelper.getInstance().getMemberFilterList();
		
		for (Filter filter : memberFilters) {
			if (filter.getValue().equals(String.valueOf(member.getId()))) {
				SyncMessageDialog.showMessage(MessageDialog.WARNING, "Filter already exists", "There is already a filter for the member " + member.getName(false));
				return;
			}
		}
		
		Filter filter = new Filter(Filter.FilterType.MEMBER_ID, isIncludeFilter, String.valueOf(member.getId()), member.getDisplayName(true, false));					
		filter.setMemberGroup(member.isGroup());
		memberFilters.add(new Filter(Filter.FilterType.MEMBER_ID, isIncludeFilter, String.valueOf(member.getId()), member.getDisplayName(true, false)));
		memberFilterListViewer.refresh();
		
		dirty.setDirty(true);
	}
	
	@Focus
	public void setFocus() {
		memberFilterListViewer.getTable().setFocus();
		
		if (FilterHelper.getInstance().isReloadReportView()) {
			FilterHelper.getInstance().setReloadReportView(false);
			ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.applyfilter", null);
			handlerService.executeHandler( applyFilterCommand );
			new AlertDialog(null, "Apply your filters", "Do not forget to apply your filters using the Run button in the toolbar or the shortcut F5.", AppImage.RUN_48X48, false, AlertType.APPLY_FILTER_BUTTON).open();
		}
	}
}
