package com.perdemia.permissionanalyzer.application.parts;

import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;

import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.DirectoryInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.ImportSharesDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SelectImportFileDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.ColoredTableLabelProvider;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo.SourceType;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.NetworkShareHelper;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

public class ScanFoldersPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.scanfolders";
	private static Logger logger = LoggerFactory.getLogger(ScanFoldersPart.class.getName());
	
	//TODO use famfamfam_silk_icons_v013 (bat icon, exe icon, etc...)
	
	private Button changeButton;
	private CheckboxTableViewer directoryTableViewer;
	private List<DirectoryInfo> directoryList;
	private DirectoryInfo selectedDirectory;
	
	private PreferenceHelper preferences = PreferenceHelper.getInstance();
	
	/**
	 * Constructor, initializes the list of directories
	 */
	public ScanFoldersPart() {
		directoryList = PreferenceHelper.getInstance().getDirectoryList();
	}

	/**
	 * Stores the directory list in the preferences
	 */
	private void scanConfigChanged() {
		PreferenceHelper prefHelper = PreferenceHelper.getInstance();
		prefHelper.setDirectoryList(directoryList);
		prefHelper.storePreferences(false);
		directoryTableViewer.refresh();
	}

	/**
	 * Creates a composite on the given <code>parent</code> composite to display the list of directories to scan
	 * 
	 * @param parent
	 *            parent composite
	 */
	@PostConstruct
	public void createComposite(final Composite parent) {
		//Composite directoryComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		//gridData.minimumHeight = 500;
		parent.setLayoutData(gridData);
		parent.setLayout(new GridLayout(1, false));
		parent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		parent.setBackground(ColorFactory.getInstance().getColor(AppColor.PART_BACKGROUND));
		
		Composite addButtonsComposite = new Composite(parent, SWT.NONE);
		addButtonsComposite.setLayout(new GridLayout(5, false));
		
		Button addDirectory = new Button(addButtonsComposite, SWT.PUSH); 
		addDirectory.setText("Add directory or share");
		addDirectory.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		addDirectory.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL) && directoryList.size() > 1) {
					MessageDialog.openError(null, "Directory limit", "The trial version of Permission Analyzer is limited to 2 directories.");
					return;
				}
				
				DirectoryInfo dirInfo = new DirectoryInfo();
				dirInfo.setSourceType(SourceType.DIRECTORY);
				DirectoryInfoDialog dialog = new DirectoryInfoDialog(parent.getShell(), dirInfo, directoryList);
				if (dialog.open() == IDialogConstants.OK_ID) {
					// determine the current max id
					int maxId = -1;
					for (DirectoryInfo directory : directoryList) {
						maxId = Math.max(maxId, directory.getDirectoryId());
					}
					dirInfo.setDirectoryId(maxId + 1);
	
					directoryList.add(dirInfo);
					scanConfigChanged();
					directoryTableViewer.refresh();
				}
			}
		});
		
		Button discoverNetworkShares = new Button(addButtonsComposite, SWT.PUSH);
		discoverNetworkShares.setText("Discover network shares");
		discoverNetworkShares.setImage(ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_16X16));
		discoverNetworkShares.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				ImportSharesDialog importSharesDialog = new ImportSharesDialog(parent.getShell(), directoryList.size());
				if (importSharesDialog.open() == IDialogConstants.OK_ID) {
					for (DirectoryInfo dirInfo : importSharesDialog.getDirectoryList()) {
						// checked?
						if (dirInfo.isEnabled()) {
							// determine the current max id
							int maxId = -1;
							for (DirectoryInfo directory : directoryList) {
								maxId = Math.max(maxId, directory.getDirectoryId());
							}
							dirInfo.setDirectoryId(maxId + 1);
							directoryList.add(dirInfo);
						}
					}
					
					scanConfigChanged();				
				}
			}
		});
		
		Button addServer = new Button(addButtonsComposite, SWT.PUSH);
		addServer.setText("Add server");
		addServer.setImage(ImageFactory.getInstance().getImage(AppImage.SERVER_16X16));
		addServer.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL) && directoryList.size() > 1) {
					MessageDialog.openError(null, "Directory limit", "The trial version of Permission Analyzer is limited to 2 directories.");
					return;
				}
				
				DirectoryInfo dirInfo = new DirectoryInfo();
				dirInfo.setSourceType(SourceType.SERVER);
				DirectoryInfoDialog dialog = new DirectoryInfoDialog(parent.getShell(), dirInfo, directoryList);
				if (dialog.open() == IDialogConstants.OK_ID) {
					// determine the current max id
					int maxId = -1;
					for (DirectoryInfo directory : directoryList) {
						maxId = Math.max(maxId, directory.getDirectoryId());
					}
					dirInfo.setDirectoryId(maxId + 1);
	
					directoryList.add(dirInfo);
					scanConfigChanged();
				}
			}
		});
		
		Button addPowerShellFile = new Button(addButtonsComposite, SWT.PUSH);
		addPowerShellFile.setText("PowerShell output file");
		addPowerShellFile.setImage(ImageFactory.getInstance().getImage(AppImage.POWER_SHELL_16X16));
		addPowerShellFile.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SelectImportFileDialog importFileDialog = new SelectImportFileDialog(parent.getShell(), SourceType.POWER_SHELL);
				if (importFileDialog.open() == IDialogConstants.OK_ID) {
					DirectoryInfo dirInfo = new DirectoryInfo();
					dirInfo.setSourceType(importFileDialog.getSourceType());
					dirInfo.setPath(importFileDialog.getFilePath());
					dirInfo.setScanLocalGroups(false);
					dirInfo.setNumberOfSubdirectories(LicenseHelper.getEdition().equals(Edition.TRIAL) ? AclScanner.getNumberOfSubDirsForTrial() : 0);
					
					// determine the current max id
					int maxId = -1;
					for (DirectoryInfo directory : directoryList) {
						maxId = Math.max(maxId, directory.getDirectoryId());
					}
					dirInfo.setDirectoryId(maxId + 1);
					directoryList.add(dirInfo);
					scanConfigChanged();
				}
			}
		});
		
		Button addEMCIsilonFile = new Button(addButtonsComposite, SWT.PUSH);
		addEMCIsilonFile.setText("EMC Isilon output file");
		addEMCIsilonFile.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
		addEMCIsilonFile.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SelectImportFileDialog importFileDialog = new SelectImportFileDialog(parent.getShell(), SourceType.EMC_ISILON);
				if (importFileDialog.open() == IDialogConstants.OK_ID) {
					DirectoryInfo dirInfo = new DirectoryInfo();
					dirInfo.setSourceType(importFileDialog.getSourceType());
					dirInfo.setPath(importFileDialog.getFilePath());
					dirInfo.setNumberOfSubdirectories(LicenseHelper.getEdition().equals(Edition.TRIAL) ? AclScanner.getNumberOfSubDirsForTrial() : 0);
					dirInfo.setScanLocalGroups(false);
					
					// determine the current max id
					int maxId = -1;
					for (DirectoryInfo directory : directoryList) {
						maxId = Math.max(maxId, directory.getDirectoryId());
					}
					dirInfo.setDirectoryId(maxId + 1);
					directoryList.add(dirInfo);
					scanConfigChanged();
				}
			}
		});
		
		/*
		Group detailsGroup = new Group(parent, SWT.NONE);
		GridData gd_detailsGroup = new GridData(GridData.FILL_HORIZONTAL);
		//gd_detailsGroup.horizontalIndent = 30;
		detailsGroup.setLayoutData(gd_detailsGroup);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 15;
		detailsGroup.setLayout(gridLayout);

		Label directoryPathLabel = new Label(detailsGroup, SWT.NONE);
		directoryPathLabel.setText("Directory, network share or server name:");
		
		Composite browseDirectoryComposite = new Composite(detailsGroup, SWT.NONE);
		browseDirectoryComposite.setLayout(new GridLayout(4, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		browseDirectoryComposite.setLayoutData(gridData);
		directoryPathText = new Text(browseDirectoryComposite, SWT.BORDER);
		directoryPathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseDirectoryButton = new Button(browseDirectoryComposite, SWT.PUSH);
		browseDirectoryButton.setLayoutData(new GridData());
		browseDirectoryButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		browseDirectoryButton.setToolTipText("Browse directory");
		browseDirectoryButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog directoryDialog = new DirectoryDialog(parent.getShell());
				String dirPath = directoryDialog.open();
				if (dirPath != null) {
					directoryPathText.setText(dirPath);
					sourceType = SourceType.DIRECTORY;
				}
			}
		});
		
		Button browseFileButton = new Button(browseDirectoryComposite, SWT.PUSH);
		browseFileButton.setLayoutData(new GridData());
		browseFileButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
		browseFileButton.setToolTipText("Add file with ACL information");
		browseFileButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SelectImportFileDialog importFileDialog = new SelectImportFileDialog(parent.getShell());
				if (importFileDialog.open() == IDialogConstants.OK_ID) {
					directoryPathText.setText(importFileDialog.getFilePath());
					sourceType = importFileDialog.getSourceType();
				}
			}
		});
		
		Button browseNetworkShareButton = new Button(browseDirectoryComposite, SWT.PUSH);
		browseNetworkShareButton.setLayoutData(new GridData());
		browseNetworkShareButton.setImage(ImageFactory.getInstance().getImage(AppImage.NETWORK_SHARE_16X16));
		browseNetworkShareButton.setToolTipText("Find network shares");
		browseNetworkShareButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ImportSharesDialog importSharesDialog = new ImportSharesDialog(parent.getShell(), directoryList.size());
				if (importSharesDialog.open() == IDialogConstants.OK_ID) {
					for (DirectoryInfo dirInfo : importSharesDialog.getDirectoryList()) {
						// checked?
						if (dirInfo.isEnabled()) {
							// determine the current max id
							int maxId = -1;
							for (DirectoryInfo directory : directoryList) {
								maxId = Math.max(maxId, directory.getDirectoryId());
							}
							dirInfo.setDirectoryId(maxId + 1);
							directoryList.add(dirInfo);
						}
					}
					directoryTableViewer.refresh();
					scanConfigChanged();				
				}
			}
		});

		Label numberSubdirectoriesLabel = new Label(detailsGroup, SWT.NONE);
		numberSubdirectoriesLabel.setText("Subdirectories depth:");
		Composite numberSubdirectoriesComposite = new Composite(detailsGroup, SWT.NONE);
		numberSubdirectoriesComposite.setLayoutData(new GridData());
		gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 10;
		gridLayout.marginLeft = -5;
		gridLayout.marginTop = -5;
		gridLayout.marginBottom = -5;
		numberSubdirectoriesComposite.setLayout(gridLayout);
		numberSubdirectoriesSpinner = new Spinner(numberSubdirectoriesComposite, SWT.BORDER);
		numberSubdirectoriesSpinner.setLayoutData(new GridData());
		numberSubdirectoriesSpinner.setMinimum(0);
		numberSubdirectoriesSpinner.setTextLimit(2);
		Label numberSubdirectoriesButtonLabel = new Label(numberSubdirectoriesComposite, SWT.NONE);
		numberSubdirectoriesButtonLabel.setLayoutData(new GridData());
		numberSubdirectoriesButtonLabel.setText("(0 means unlimited)");
				
		Label includeFilesLabel = new Label(detailsGroup, SWT.NONE);
		includeFilesLabel.setText("Include files:");
		includeFilesButton = new Button(detailsGroup, SWT.CHECK);
		includeFilesButton.setLayoutData(new GridData());
		
		Label includeInheritedPermissionsLabel = new Label(detailsGroup, SWT.NONE);
		includeInheritedPermissionsLabel.setText("Include inherited permissions:");
		includeInheritedPermissionsButton = new Button(detailsGroup, SWT.CHECK);
		includeInheritedPermissionsButton.setLayoutData(new GridData());
		
		Label followSymbolicLinksLabel = new Label(detailsGroup, SWT.NONE);
		followSymbolicLinksLabel.setText("Follow symbolic links:");
		followSymbolicLinksButton = new Button(detailsGroup, SWT.CHECK);
		followSymbolicLinksButton.setLayoutData(new GridData());
		
		Label scanLocalGroupsLabel = new Label(detailsGroup, SWT.NONE);
		scanLocalGroupsLabel.setText("Scan local groups on this server:");
		Composite scanLocalGroupsComposite = new Composite(detailsGroup, SWT.NONE);
		scanLocalGroupsComposite.setLayoutData(new GridData());
		gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 10;
		gridLayout.marginLeft = -5;
		gridLayout.marginTop = -5;
		gridLayout.marginBottom = -5;
		scanLocalGroupsComposite.setLayout(gridLayout);
		scanLocalGroupsButton = new Button(scanLocalGroupsComposite, SWT.CHECK);
		scanLocalGroupsButton.setLayoutData(new GridData());
		Label scanLocalGroupsButtonLabel = new Label(scanLocalGroupsComposite, SWT.NONE);
		scanLocalGroupsButtonLabel.setLayoutData(new GridData());
		// confusing?
		//scanLocalGroupsButtonLabel.setText("(servers will never be scanned more than once)");
				
		Composite detailsButtonBarComposite = new Composite(detailsGroup, SWT.NONE);
		detailsButtonBarComposite.setLayout(new GridLayout(2, false));
		GridData gd_composite = new GridData(GridData.FILL_HORIZONTAL);
		gd_composite.horizontalSpan = 2;
		detailsButtonBarComposite.setLayoutData(gd_composite);

		Button addButton = new Button(detailsButtonBarComposite, SWT.NONE);
		addButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		addButton.setText("Add");
		addButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_16X16));
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL) && directoryList.size() > 1) {
					MessageDialog.openError(null, "Directory limit", "The trial version of Permission Analyzer is limited to 2 directories.");
					return;
				}
				
				DirectoryInfo dirInfo = new DirectoryInfo();
				fillDirectoryInfoFromUI(dirInfo);
				dirInfo.setEnabled(true);
				
				// validate fields
				if (directoryPathText.getText().length() == 0) {
					MessageDialog.openError(null, "Details not complete", "Please enter a directory path.");
					return;
				}

				// check if the name already exists
				for (DirectoryInfo directory : directoryList) {
					if (directory.getPath().equalsIgnoreCase(dirInfo.getPath())) {
						// connection name already exists
						MessageDialog.openError(null, "Directory already exists", "The directory path " + dirInfo.getPath() + " already exists in the list.");
						return;
					}
				}

				// determine the current max id
				int maxId = -1;
				for (DirectoryInfo directory : directoryList) {
					maxId = Math.max(maxId, directory.getDirectoryId());
				}
				dirInfo.setDirectoryId(maxId + 1);

				directoryList.add(dirInfo);
				scanConfigChanged();

				// reset UI fields
				clearDirectoryFields();
			}
		});
		
		*/
		

		directoryTableViewer = CheckboxTableViewer.newCheckList(parent, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
		Table table = directoryTableViewer.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		//gd_table.horizontalIndent = 30;
		table.setLayoutData(gd_table);

		TableViewerColumn pathColumn = new TableViewerColumn(directoryTableViewer, SWT.NONE);
		pathColumn.getColumn().setWidth(350);
		pathColumn.getColumn().setText("Directory path");
		
		TableViewerColumn subDirsColumn = new TableViewerColumn(directoryTableViewer, SWT.NONE);
		subDirsColumn.getColumn().setWidth(75);
		subDirsColumn.getColumn().setText("Depth");
		
		TableViewerColumn includeFilesColumn = new TableViewerColumn(directoryTableViewer, SWT.NONE);
		includeFilesColumn.getColumn().setWidth(100);
		includeFilesColumn.getColumn().setText("Include files");
		
		TableViewerColumn followSymbolicLinksColumn = new TableViewerColumn(directoryTableViewer, SWT.NONE);
		followSymbolicLinksColumn.getColumn().setWidth(150);
		followSymbolicLinksColumn.getColumn().setText("Follow symb. links");

		TableViewerColumn scanLocalGroupsColumn = new TableViewerColumn(directoryTableViewer, SWT.NONE);
		scanLocalGroupsColumn.getColumn().setWidth(150);
		scanLocalGroupsColumn.getColumn().setText("Scan local groups");
		
		directoryTableViewer.setContentProvider(ArrayContentProvider.getInstance());
		directoryTableViewer.setLabelProvider(new ColoredTableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				DirectoryInfo info = (DirectoryInfo) element;
				switch (columnIndex) {
					case 0:
						return info.getPath();
					case 1:
						return String.valueOf(info.getNumberOfSubdirectories());
					case 2:
						return info.isIncludeFiles() ? "Yes" : "No";
					case 3:
						return info.isFollowSymbolicLinks() ? "Yes" : "No";
					case 4:
						return info.isScanLocalGroups() ? "Yes" : "No";
					default:
						return "";
				}
			}

			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					DirectoryInfo dirInfo = (DirectoryInfo)element;
					if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
						if (dirInfo.isEnabled()) {
							return ImageFactory.getInstance().getImage(AppImage.FOLDER_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.FOLDER_DISABLED_16X16);
						}
					} else if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
						if (dirInfo.isEnabled()) {
							return ImageFactory.getInstance().getImage(AppImage.SERVER_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.SERVER_DISABLED_16X16);
						}
					} else if (dirInfo.getSourceType().equals(SourceType.POWER_SHELL)) {
						if (dirInfo.isEnabled()) {
							return ImageFactory.getInstance().getImage(AppImage.POWER_SHELL_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.POWER_SHELL_DISABLED_16X16);
						}
					} else if (dirInfo.getSourceType().equals(SourceType.EMC_ISILON)) {
						if (dirInfo.isEnabled()) {
							return ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.FILE_TXT_NONE_16X16);
						}
					}
					return null;
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (((DirectoryInfo) element).isEnabled()) {
					return null;
				} else {
					return ColorFactory.getInstance().getColor(AppColor.GREY);
				}
			}			
		});

		directoryTableViewer.setCheckStateProvider(new ICheckStateProvider() {
			@Override
			public boolean isChecked(Object element) {
				return ((DirectoryInfo) element).isEnabled();
			}

			@Override
			public boolean isGrayed(Object element) {
				return false;
			}
		});

		directoryTableViewer.setInput(directoryList);

		directoryTableViewer.addCheckStateListener(new ICheckStateListener() {
			@Override
			public void checkStateChanged(CheckStateChangedEvent event) {
				if (!checkForMaximumServers(directoryTableViewer)) {
					// uncheck again
					int index = directoryList.indexOf(event.getElement());
					directoryTableViewer.getTable().getItems()[index].setChecked(false);
				} else {
					// save in preferences
					((DirectoryInfo) event.getElement()).setEnabled(event.getChecked());
					preferences.setDirectoryList(directoryList);
					preferences.storePreferences(false);
					directoryTableViewer.refresh();
				}
			}
		});
		
		directoryTableViewer.addDoubleClickListener(new IDoubleClickListener(){
			@Override
			public void doubleClick(DoubleClickEvent event) {
				changeButton.notifyListeners(SWT.Selection, new Event());
			}
		});
		

		Composite buttonBarComposite = new Composite(parent, SWT.NONE);
		buttonBarComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonBarComposite.setLayout(new GridLayout(4, false));
		
		final Button truncateDirectoryDataButton = new Button(buttonBarComposite, SWT.CHECK);
		truncateDirectoryDataButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		truncateDirectoryDataButton.setText("Clear all data from the database before scanning");
		truncateDirectoryDataButton.setImage(ImageFactory.getInstance().getImage(AppImage.TRASH_16X16));
		truncateDirectoryDataButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
					if (!truncateDirectoryDataButton.getSelection()) {
						truncateDirectoryDataButton.setSelection(true);
						SyncMessageDialog.showMessage(MessageDialog.WARNING, "Not allowed", "The trial version will always clear the database content before scanning.");
						return;
					}
				}
				
				// save in preferences
				preferences.setTruncateData(truncateDirectoryDataButton.getSelection());
				preferences.storePreferences(false);
			}
		});
		truncateDirectoryDataButton.setSelection(preferences.isTruncateData());
		
		final Button testDirectoryButton = new Button(buttonBarComposite, SWT.PUSH);
		testDirectoryButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		testDirectoryButton.setText("Test input source");
		testDirectoryButton.setImage(ImageFactory.getInstance().getImage(AppImage.TEST_16X16));
		testDirectoryButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (directoryTableViewer.getTable().getSelectionIndex() > -1) {
					ApplicationUtil.setBusyCursor();
					
					selectedDirectory = directoryList.get(directoryTableViewer.getTable().getSelectionIndex());
	
					Path path = Paths.get(selectedDirectory.getPath());
					
					switch (selectedDirectory.getSourceType()) {
						case DIRECTORY:
							int numberOfDirectories = 0;
							int numberOfFiles = 0;
							try (DirectoryStream<Path> directoryStream = Files.newDirectoryStream(path)) {
								for (Path nextPath : directoryStream) {
									if (Files.isDirectory(nextPath)) {
										numberOfDirectories++;
									} else {
										numberOfFiles++;
									}
								}
								MessageDialog.openInformation(null, "Directory OK", MessageFormat.format("The directory was read succesfully. Found {0} files and {1} (direct) subdirectories.", numberOfFiles, numberOfDirectories));
							} catch (IOException ex) {
								logger.error("Could not read the directory content using path " + selectedDirectory.getPath(), ex);
								MessageDialog.openError(null, "Failed", "Could not read the directory content: " + ex.getMessage());
							}
							break;
							
						case EMC_ISILON: case POWER_SHELL:
							if (Files.exists(path)) {
								MessageDialog.openInformation(null, "File OK", "The file was read succesfully.");
							} else {
								MessageDialog.openError(null, "Error", "The file was not found on the file system.");
							}
							break;
							
						case SERVER:
							try {
								List<String> shares = NetworkShareHelper.getNetworkShares(selectedDirectory.getPath(), selectedDirectory, false);
								MessageDialog.openInformation(null, "Network shares found", "The following network shares will be scanned: \r\n\r\n" + shares.toString());
								
							} catch (Exception ex) {
								logger.error("Error while retrieving network shares from the server " + selectedDirectory.getPath(), ex);
								MessageDialog.openError(null, "Error", "Error while retrieving network shares from the server: " + ex.getMessage());
							}
							
					}
		
					ApplicationUtil.setDefaultCursor();
				} else {
					MessageDialog.openWarning(null, "No selection", "Please select an item from the table.");
				}
			}
		});
		
		changeButton = new Button(buttonBarComposite, SWT.NONE);
		changeButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		changeButton.setText("Change");
		changeButton.setImage(ImageFactory.getInstance().getImage(AppImage.EDIT_16X16));
		changeButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (directoryTableViewer.getTable().getSelectionIndex() > -1) {
					selectedDirectory = directoryList.get(directoryTableViewer.getTable().getSelectionIndex());
					
					DirectoryInfoDialog dialog = new DirectoryInfoDialog(parent.getShell(), selectedDirectory, directoryList);
					if (dialog.open() == IDialogConstants.OK_ID) {
						scanConfigChanged();
					}
				} else {
					MessageDialog.openWarning(null, "No selection", "Please select an item from the table.");
					
				}
			}
		});

		final Button deleteButton = new Button(buttonBarComposite, SWT.NONE);
		deleteButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		deleteButton.setText("Delete");
		deleteButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		deleteButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				
				@SuppressWarnings("unchecked")
				Iterator<DirectoryInfo> iter = ((StructuredSelection)directoryTableViewer.getSelection()).iterator();
				while(iter.hasNext()) {
					DirectoryInfo dirInfo = iter.next();
					directoryList.remove(dirInfo);
				}
								
				scanConfigChanged();
			}
		});
	}

	public void resetListStatus() {
		for (TableItem item : directoryTableViewer.getTable().getItems()) {
			item.setBackground(ColorFactory.getInstance().getColor(AppColor.WHITE));
		}
	}
	
	public void directoryScanned(final DirectoryInfo directory, final boolean success) {
		int index = directoryList.indexOf(directory);
		if (index != -1) {
			if (success) {
				directoryTableViewer.getTable().getItem(index).setBackground(ColorFactory.getInstance().getColor(AppColor.SCANNED));
			} else {
				directoryTableViewer.getTable().getItem(index).setBackground(ColorFactory.getInstance().getColor(AppColor.ERROR));
			}
		}			
	}
		
	/**
	 * Loops through all the TableItems in the directoryTableViewer and determines the server name for all the checked items.
	 * 
	 * @return true in case the number of servers is allowed.
	 */
	public static boolean checkForMaximumServers(TableViewer tableViewer) {
		int maxServers = Integer.parseInt(AclScanner.licenseLimits.get(LicenseHelper.getEdition().toString() + "_LIMIT_SERVERS"));
		List<String> servers = new ArrayList<String>();
		
		for (TableItem tableItem : tableViewer.getTable().getItems()) {
			if (tableItem.getChecked()) {
				String path = tableItem.getText(0);
				String server;
				if (path.startsWith("\\\\")) { //escaped \\
					int firstBackslash = path.substring(2).indexOf("\\"); //skip the first two backslashes
					if (firstBackslash != -1) {
						server = path.substring(2, firstBackslash + 2); //plus two for the first two backslashes
					} else {
						server = path.substring(2);
					}
				} else {
					server = "localhost";
				}
				
				server = server.toUpperCase();
				
				if (!servers.contains(server)) {
					servers.add(server);
				}
			}
		}
		
		if (servers.size() > maxServers) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Number of servers not allowed", MessageFormat.format("The number of selected servers ({0}) exceeds the maximum number of servers allowed for this license ({1}).", 
					servers.size(), maxServers == 1 ? "1 server" : maxServers + " servers"));
			return false;
		} else {
			return true;
		}
	}
	
	@Focus
	public void setFocus() {
		
	}
}
