package com.perdemia.permissionanalyzer.application.swt;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
/**
 * This class will display a message and info icon and an option to close the message. Once a message is closed it won't be displayed the next time the application starts (using the preferences).
 * 
 * @author Reinier
 *
 */
public class ExplainLabel {
	
	public enum ExplainType {
		FILE_TREE, MEMBERS_TAB, ACL_TAB, TRACE_TAB, ALL_MEMBERS_TAB, REPORTS, POLICIES, REPORT_DETAILS
	}
	
	public ExplainLabel(Composite parent, String text, final ExplainType type) {
		this(parent, text, type, 1);
	}
	
	public ExplainLabel(Composite parent, String text, final ExplainType type, int horizontalSpan) {
		// don't create the label once it has been closed by the user
		if (PreferenceHelper.getInstance().getShowExplainLabels().get(type.ordinal())) {
			// we create a separate image and label, instead of using CLabel because CLabel won't wrap very well (it shows "..." instead of new lines)
			final Composite explainComposite = new Composite(parent, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = horizontalSpan;
			explainComposite.setLayoutData(gridData);
			explainComposite.setLayout(new GridLayout(3, false));
			
			new ImageComposite(explainComposite, ImageFactory.getInstance().getImage(AppImage.INFO_16X16));  
			Label explanation = new Label(explainComposite, SWT.WRAP);
			explanation.setText(text);
			gridData = new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1);
			explanation.setLayoutData(gridData);
			
			ImageComposite closeIcon = new ImageComposite(explainComposite, ImageFactory.getInstance().getImage(AppImage.CLOSE_EXPLAIN_LABEL_16X16));
			closeIcon.setCursor(Display.getDefault().getSystemCursor(SWT.CURSOR_HAND));
			closeIcon.addMouseListener(new MouseAdapter(){
				@Override
				public void mouseUp(MouseEvent e) {
					int result = SyncMessageDialog.showQuestionWithCancelMessage("Hide message", "Do you want to hide this help message permanently?");
					if (result == IDialogConstants.YES_ID) {
						PreferenceHelper.getInstance().getShowExplainLabels().set(type.ordinal(), false);
						PreferenceHelper.getInstance().storePreferences(false);
					}
					
					if (result == IDialogConstants.YES_ID || result == IDialogConstants.NO_ID) {
						// hide the label
						((GridData)explainComposite.getLayoutData()).exclude = true;
						explainComposite.setVisible(false);
						explainComposite.getParent().layout(true);
					}
				}
			});
		}
	}
}
