package com.perdemia.permissionanalyzer.application.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

public class ImageComposite extends Composite {
	private Image image;
	
	public ImageComposite(Composite parent, final Image image) {
		this(parent, image, "", false);
	}
		
	public ImageComposite(Composite parent, final Image image, String tooltip) {
		this(parent, image, tooltip, false);
	}
	
	public ImageComposite(final Composite parent, final Image image, String tooltip, final boolean center) {
		super(parent, SWT.NO_REDRAW_RESIZE);
		this.image = image;
		this.setToolTipText(tooltip);
		if (image != null) {
			GridData gridData;
			if (center) {
				gridData = new GridData(GridData.FILL_BOTH);
			} else {
				gridData = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
				gridData.widthHint = image.getImageData().width;
				gridData.heightHint = image.getImageData().height;
			}
			
			this.setLayoutData(gridData);
		}
		
		this.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				if (getImage() != null) {
					if (center) {
						//e.gc.drawImage(getImage(), (getBounds().width - getImage().getImageData().width) / 2, (getBounds().height - getImage().getImageData().height) / 2);
						e.gc.drawImage(getImage(), (getBounds().width - getImage().getImageData().width) / 2, 0);
					} else {
						e.gc.drawImage(getImage(), 0, 0);
					}
				}
			}
		});
	}

	public void setImage(Image image) {
		this.image = image;
		this.redraw();
	}

	public Image getImage() {
		return image;
	}
}
