package com.perdemia.permissionanalyzer.application.util;

import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

public class ColorFactory {
	/** The singleton instance. */
	private static ColorFactory instance;
	
	/** The color registry. */
	private ColorRegistry colorRegistry;
		
	/** The enum that defines all the colors. */
	public static enum AppColor {
		PART_BACKGROUND, GREY, DISABLED_ROW, WHITE, 
		EXPLICIT_PERMISSION_ALLOW_START, EXPLICIT_PERMISSION_ALLOW_END,
		PERMISSION_DENY_START, PERMISSION_DENY_END,
		EXPLICIT_PERMISSION_DENY_START, EXPLICIT_PERMISSION_DENY_END,
		ERROR, SCANNED, TRIAL_BANNER, ACE_BACKGROUND_COLOR
	}
	
	/**
	 * Disables public constructor (Singleton pattern).
	 */
	private ColorFactory(){
		colorRegistry = new ColorRegistry(Display.getCurrent());
		colorRegistry.put(AppColor.PART_BACKGROUND.name(), new RGB(247, 247, 247));
	    colorRegistry.put(AppColor.GREY.name(), new RGB(128, 128, 128));
	    colorRegistry.put(AppColor.WHITE.name(), new RGB(255, 255, 255));
	    colorRegistry.put(AppColor.DISABLED_ROW.name(), new RGB(240, 240, 240));
	    colorRegistry.put(AppColor.EXPLICIT_PERMISSION_ALLOW_START.name(), new RGB(250, 252, 235));
	    colorRegistry.put(AppColor.EXPLICIT_PERMISSION_ALLOW_END.name(), new RGB(234, 243, 184));
	    colorRegistry.put(AppColor.PERMISSION_DENY_START.name(), new RGB(255, 249, 249));
	    colorRegistry.put(AppColor.PERMISSION_DENY_END.name(), new RGB(255, 204, 204));
	    colorRegistry.put(AppColor.EXPLICIT_PERMISSION_DENY_START.name(), new RGB(255, 215, 215));
	    colorRegistry.put(AppColor.EXPLICIT_PERMISSION_DENY_END.name(), new RGB(255, 149, 149));
	    colorRegistry.put(AppColor.ERROR.name(), new RGB(255, 0, 85));
	    colorRegistry.put(AppColor.SCANNED.name(), new RGB(230, 245, 155));
	    colorRegistry.put(AppColor.TRIAL_BANNER.name(), new RGB(250, 240, 165));
	    colorRegistry.put(AppColor.ACE_BACKGROUND_COLOR.name(), new RGB(255, 255, 240));
	    
	};
	
	/**
	 * Controls the access to this class (Singleton pattern)
	 * 
	 * @return Instance of this class. If it doesn't exist, it will be created.
	 */
	public static ColorFactory getInstance(){
		if (instance == null) {
			instance = new ColorFactory();
		}
		return instance;
	}

	/**
	 * Gets a color from the registry based on the given <code>color</code> enum.
	 * 
	 * @param color enum value
	 * @return Color instance
	 */
	public Color getColor(AppColor color) {
		return colorRegistry.get(color.name());
	}
	
	public void putColor(AppColor color, RGB rgb) {
		colorRegistry.put(color.name(), rgb);
	}
}
