package com.perdemia.permissionanalyzer.application.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ProgramParameters {
	private static Logger logger = LoggerFactory.getLogger(ProgramParameters.class.toString());
	private static String password;
	private static boolean scan;
	private static boolean scanLdap;
	private static boolean scanDirectories;
	private static boolean allReports;
	private static boolean allPolicies;
	private static boolean includeSharedDrives;
	private static boolean includeHiddenShares;
	private static boolean forceScanAllLdapContainers;
	private static boolean forceScanAllDirectoryContainers;
	private static boolean clearData;
	private static List<String> reports = new ArrayList<String>();
	private static List<String> excludeShare = new ArrayList<String>();
	private static List<String> directories = new ArrayList<String>();
	private static List<String> ldapOUs = new ArrayList<String>();
	
	private static boolean processingReports;
	private static boolean processingExcludeShares;
	private static boolean processingDirectories;
	private static boolean processingLdapOUs;
	static {
		String[] args = Platform.getCommandLineArgs();
		int i = 0;
		while (i < args.length) {
			if (args[i].equalsIgnoreCase("-password")) {
				password = args[++i];
			
			} else if (args[i].equalsIgnoreCase("-scan")) {
				scan = true;
			
			} else if (args[i].equalsIgnoreCase("-scanLdap")) {
				scanLdap = true;
			
			} else if (args[i].equalsIgnoreCase("-scanDirectories")) {
				scanDirectories = true;
			
			} else if (args[i].equalsIgnoreCase("-report") || args[i].equalsIgnoreCase("-policy")) {
				reports.add(args[++i]);
				processingReports = true;
			
			} else if (args[i].equalsIgnoreCase("-allReports")) {
				allReports = true;
				
			} else if (args[i].equalsIgnoreCase("-allPolicies")) {
				allPolicies = true;
			
			} else if (args[i].equalsIgnoreCase("-includeSharedDrives")) {
				includeSharedDrives = true;
				
			} else if (args[i].equalsIgnoreCase("-includeHiddenShares")) {
				 includeHiddenShares = true;
				
			} else if (args[i].equalsIgnoreCase("-forceScanAllLdapContainers")) {
				forceScanAllLdapContainers = true;
				
			} else if (args[i].equalsIgnoreCase("-forceScanAllDirectoryContainers")) {
				forceScanAllDirectoryContainers = true;
			
			} else if (args[i].equalsIgnoreCase("-clearData")) {
				clearData = true;
				
			} else if (args[i].equalsIgnoreCase("-excludeShare")) {
				excludeShare.add(args[++i].toUpperCase());
				processingExcludeShares = true;
			
			} else if (args[i].equalsIgnoreCase("-directory")) {
				directories.add(args[++i].toUpperCase());
				processingDirectories = true;
				
			} else if (args[i].equalsIgnoreCase("-ldapOU")) {
				ldapOUs.add(args[++i].toUpperCase());
				processingLdapOUs = true;
				
			} else {
				if (processingReports) {
					reports.add(args[i]);
				} else if (processingExcludeShares) {
					excludeShare.add(args[i].toUpperCase());
				} else if (processingDirectories) {
					directories.add(args[i].toUpperCase());
				} else if (processingLdapOUs) {
					ldapOUs.add(args[i].toUpperCase());
				}
			}
			
			i++;
		}
	}
	
	public static boolean isRunReports() {
		return reports.size() > 0 || allReports || allPolicies;
	}

	public static String getPassword() {
		return password;
	}
	
	public static boolean isScan() {
		return scan || scanLdap || scanDirectories;
	}

	public static List<String> getReports() {
		return reports;
	}

	public static boolean isAllReports() {
		return allReports;
	}

	public static boolean isAllPolicies() {
		return allPolicies;
	}

	public static String getParametersString() {
		StringBuilder builder = new StringBuilder();
		builder.append("ProgramParameters [password=");
		builder.append(password != null ? "present" : "empty");
		builder.append(", scan=");
		builder.append(scan);
		builder.append(", scanLdap=");
		builder.append(scanLdap);
		builder.append(", scanDirectories=");
		builder.append(scanDirectories);
		builder.append(", allReports=");
		builder.append(allReports);
		builder.append(", allPolicies=");
		builder.append(allPolicies);
		builder.append(", forceScanAllLdapContainers=");
		builder.append(forceScanAllLdapContainers);
		builder.append(", forceScanAllDirectoryContainers=");
		builder.append(forceScanAllDirectoryContainers);	
		builder.append(", reports=");
		builder.append(reports);
		builder.append(", includeSharedDrives=");
		builder.append(includeSharedDrives);
		builder.append(", excludeShare=");
		builder.append(excludeShare);
		builder.append(", directories=");
		builder.append(directories);
		builder.append(", ldapOUs=");
		builder.append(ldapOUs);
		builder.append("]");
		return builder.toString();
	}

	public static boolean isScanLdap() {
		return scanLdap;
	}

	public static boolean isScanDirectories() {
		return scanDirectories;
	}

	public static boolean isIncludeSharedDrives() {
		return includeSharedDrives;
	}

	public static List<String> getExcludeShare() {
		return excludeShare;
	}

	public static boolean isForceScanAllLdapContainers() {
		return forceScanAllLdapContainers;
	}

	public static boolean isForceScanAllDirectoryContainers() {
		return forceScanAllDirectoryContainers;
	}

	public static List<String> getDirectories() {
		return directories;
	}

	public static List<String> getLdapOUs() {
		return ldapOUs;
	}
	
	public static boolean isDirectoryInList(String directory) {
		for (String nextDir : directories) {
			if (nextDir.equalsIgnoreCase(directory)) {
				return true;
			}
		}
		
		return false;
	}
	
	public static boolean isLdapOUInList(String ldapOU) {
		for (String nextldapOU : ldapOUs) {
			if (nextldapOU.equalsIgnoreCase(ldapOU)) {
				return true;
			}
		}
		
		return false;
	}

	public static boolean isClearData() {
		return clearData;
	}

	public static boolean isIncludeHiddenShares() {
		return includeHiddenShares;
	}
}
