package com.perdemia.permissionanalyzer.data;

/**
 * Generic exception wrapper for all things DB.
 *
 */
public class DBException extends Exception{
	private static final long serialVersionUID = 1L;
	private String message;

    public DBException(Throwable cause) {
        super(cause);
        this.setStackTrace(cause.getStackTrace());
    }

    public DBException(String message) {
        super(message);
    }

    public DBException(String message, Throwable cause) {
        super(message, cause);
        this.setStackTrace(cause.getStackTrace());
    }

    /**
     *
     * @param query SQL query
     * @param params - array of parameters, can be null
     * @param cause real cause.
     */
    public DBException(String query, Object[] params, Throwable cause) {

        message = cause.toString()+ ", Query: " + query;

        if(params != null && params.length > 0){
            message += ", params: ";
            for (int i = 0; i < params.length; i++) {
                Object param = params[i];
                message += param;
                if (i < params.length - 1)
                    message += ",";
        }
        }

        setStackTrace(cause.getStackTrace());
        initCause(cause);
    }

    @Override
    public String getMessage() {
        return message == null ? super.getMessage() : message;
    }

    public DBException() {
        super();    
    }    
}
