package com.perdemia.permissionanalyzer.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class Domain extends DBObject {
private final static Logger logger = LoggerFactory.getLogger(Domain.class.getName());
	
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"DOMAIN"};
	
	/** Table fields */
	private String domain;
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	public Domain(){}
	
	public Domain(String domain) {
		this.domain = domain;
	}
	
	public Domain(String domain, int id) {
		this.domain = domain;
		this.id = id;
	}
	
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	@SuppressWarnings("unchecked")
	public static List<Domain> getAll() throws DBException {
		return (List<Domain>) getAll(Domain.class, "DOMAIN", 0);
	}
	
	public static Domain getById(Integer id) throws DBException {
		return (Domain) getById(Domain.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<Domain> where(String whereClause, Object[] parameters) throws DBException {
		return where(Domain.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<Domain> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(Domain.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<Domain> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(Domain.class, whereClause, parameters, orderBy, limit);
	}
	
	public static Domain findFirst(String whereClause, Object... parameters) throws DBException {
		return (Domain) findFirst(Domain.class, whereClause, parameters);
	}
	
	public static int delete(Integer id) throws DBException {
		return delete(Domain.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(Domain.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		domain = getNullAsEmptyString(rs, tableAttributes[0]);		
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setString(1, domain);
	}
	
	//========================================
	// GETTERS AND SETTERS
	//========================================
	
	public String getDomain() {
		return domain;
	}

	public void setDomain(String domain) {
		this.domain = domain;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Domain [domain=");
		builder.append(domain);
		builder.append(", id=");
		builder.append(id);
		builder.append("]");
		return builder.toString();
	}
	
	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		Domain.lastId = lastId;
	}	
}
