package com.perdemia.permissionanalyzer.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Holds a mapping between a {@link Member} and a group {@link Member}.
 * 
 * @author Reinier
 *
 */
public class Membership extends DBObject {
	private final static Logger logger = LoggerFactory.getLogger(Membership.class.getName());
	
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"GROUP_ID", "MEMBER_ID", "VIA_GROUP_ID"};
	
	/** Table fields */
	private Integer groupId;
	private Integer memberId;
	private Integer viaGroupId;
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	public Membership(){}
	
	public Membership(Integer groupId, Integer memberId) {
		this.groupId = groupId;
		this.memberId = memberId;
	}
	
	public Membership(Integer groupId, Integer memberId, Integer viaGroupId) {
		this.groupId = groupId;
		this.memberId = memberId;
		this.viaGroupId = viaGroupId;
	}
	
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	public static Membership getById(Integer id) throws DBException {
		return (Membership) getById(Membership.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<Membership> where(String whereClause, Object[] parameters) throws DBException {
		return where(Membership.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<Membership> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(Membership.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<Membership> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(Membership.class, whereClause, parameters, orderBy, limit);
	}
	
	public static Membership findFirst(String whereClause, Object... parameters) throws DBException {
		return (Membership) findFirst(Membership.class, whereClause, parameters);
	}
	
	public static int delete(Integer id) throws DBException {
		return delete(Membership.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(Membership.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		groupId = rs.getInt(tableAttributes[0]);		
		memberId = rs.getInt(tableAttributes[1]);
		viaGroupId = getNullableInteger(rs, tableAttributes[2]);
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setInt(1, groupId);
		ps.setInt(2, memberId);
		setNullableInteger(ps, 3, viaGroupId);
	}

	//========================================
	// GETTERS AND SETTERS
	//========================================
		
	public Integer getGroupId() {
		return groupId;
	}

	public void setGroupId(Integer groupId) {
		this.groupId = groupId;
	}

	public Integer getMemberId() {
		return memberId;
	}

	public void setMemberId(Integer memberId) {
		this.memberId = memberId;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Membership [groupId=");
		builder.append(groupId);
		builder.append(", memberId=");
		builder.append(memberId);
		builder.append(", id=");
		builder.append(id);
		builder.append(", viaGroupId=");
		builder.append(viaGroupId);
		builder.append("]");
		return builder.toString();
	}

	public Integer getViaGroupId() {
		return viaGroupId;
	}

	public void setViaGroupId(Integer viaGroupId) {
		this.viaGroupId = viaGroupId;
	}
	
	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		Membership.lastId = lastId;
	}	
}
