package com.perdemia.permissionanalyzer.data.util;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Report;

public class ReportCache {
	private static final Logger logger = LoggerFactory.getLogger(ReportCache.class.getName());
	
	/** Map with Report objects and Id's */
	private static final Map<Integer, Report> reports = new HashMap<Integer, Report>();
	
	/**
	 * Clears the Map of the cache and forces a refresh from the database whenever the cache is used.
	 */
	public static void clearCache() {
		reports.clear();
	}
	
	public static Report getReport(Integer reportId) throws DBException {
		Report report = reports.get(reportId);
		
		if (report != null && logger.isTraceEnabled()) {
			logger.trace("Returning report from cache: " + report.toString());
		} else {
			// not yet in cache
			report = Report.getById(reportId);
			
			if (report != null) {
				ReportCache.cacheFile(report);
			}
		}
		
		if (report == null) {
			// should never happen
			logger.warn("Report ID " + reportId + " not found in the database!");
			throw new DBException("Report ID " + reportId + " not found in the database!");
		}
		
		return report;
	}
		
	public static void clearFile(Integer reportId) {
		reports.remove(reportId);
	}
	
	public static void cacheFile(Report report) {
		reports.put(report.getId(), report);
	}
}